@@ -1,3 +1,17 @@
+# Copyright 2001-2004 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 package Apache::TestClient;
 
 #this module provides some fallback for when libwww-perl is not installed
@@ -46,8 +60,10 @@ sub request {
       "$method $url HTTP/1.0",
       (map { "$_: $headers->{$_}" } keys %$headers), $CRLF;
 
-    print $s $request;
-    print $s $content if $content;
+    # using send() avoids the need to use SIGPIPE if the server aborts
+    # the connection
+    $s->send($request);
+    $s->send($content) if $content;
 
     $request =~ s/\015//g; #for as_string
 
@@ -58,6 +74,7 @@ sub request {
         }, 'Apache::TestClientRequest'),
         headers_as_string => '',
         method => $method,
+        code   => -1, # unknown
     };
 
     my($response_line, $header_term);
@@ -69,12 +86,12 @@ sub request {
         $res->{headers_as_string} .= $_;
         if (m:^(HTTP/\d+\.\d+)[ \t]+(\d+)[ \t]*(.*?)$eol:io) {
             $res->{protocol} = $1;
-            $res->{code} = $2;
-            $res->{message} = $3;
-            $response_line = 1;
+            $res->{code}     = $2;
+            $res->{message}  = $3;
+            $response_line   = 1;
         }
         elsif (/^([a-zA-Z0-9_\-]+)\s*:\s*(.*?)$eol/o) {
-            $res->{headers}->{$1} = $2;
+            $res->{headers}->{lc $1} = $2;
         }
         elsif (/^$eol$/o) {
             $header_term = 1;
@@ -92,6 +109,10 @@ sub request {
     }
     close $s;
 
+    # an empty body is a valid response
+    $res->{content} = '' 
+        unless exists $res->{content} and defined $res->{content};
+
     $res->{headers_as_string} =~ s/\015//g; #for as_string
 
     bless $res, 'Apache::TestClientResponse';
@@ -110,7 +131,7 @@ package Apache::TestClientResponse;
 
 sub header {
     my($self, $key) = @_;
-    $self->{headers}->{$key};
+    $self->{headers}->{lc $key};
 }
 
 my @headers = qw(Last-Modified Content-Type);
@@ -118,11 +139,12 @@ my @headers = qw(Last-Modified Content-Type);
 for my $header (@headers) {
     no strict 'refs';
     (my $method = lc $header) =~ s/-/_/g;
-    *$method = sub { shift->{headers}->{$header} };
+    *$method = sub { shift->{headers}->{lc $header} };
 }
 
 sub is_success {
     my $code = shift->{code};
+    return 0 unless defined $code && $code;
     $code >= 200 && $code < 300;
 }
 
