@@ -28,16 +28,17 @@ use constant COLOR   => ($ENV{APACHE_TEST_COLOR} && -t STDOUT) ? 1 : 0;
 use constant DEFAULT_PORT => 8529;
 
 use constant IS_MOD_PERL_2       =>
-    eval { require mod_perl && $mod_perl::VERSION >= 1.99 } || 0;
+    eval { require mod_perl2 } || 0;
 
 use constant IS_MOD_PERL_2_BUILD => IS_MOD_PERL_2 &&
-    require Apache::Build && Apache::Build::IS_MOD_PERL_BUILD();
+    require Apache2::Build && Apache2::Build::IS_MOD_PERL_BUILD();
 
 use constant IS_APACHE_TEST_BUILD =>
     grep { -e "$_/lib/Apache/TestConfig.pm" } qw(Apache-Test . ..);
 
 use constant CUSTOM_CONFIG_FILE => 'Apache/TestConfigData.pm';
 
+use lib ();
 use File::Copy ();
 use File::Find qw(finddepth);
 use File::Basename qw(dirname);
@@ -85,7 +86,7 @@ my %vars_to_env = (
    sbindir         => 'Apache sbin/ dir (default is apxs -q SBINDIR)',
    httpd           => 'server to use for testing (default is $bindir/httpd)',
    target          => 'name of server binary (default is apxs -q TARGET)',
-   apxs            => 'location of apxs (default is from Apache::BuildConfig)',
+   apxs            => 'location of apxs (default is from Apache2::BuildConfig)',
    startup_timeout => 'seconds to wait for the server to start (default is 60)',
    httpd_conf      => 'inherit config from this file (default is apxs derived)',
    httpd_conf_extra=> 'inherit additional config from this file',
@@ -171,15 +172,16 @@ sub passenv_makestr {
 
 sub server { shift->{server} }
 
-sub modperl_2_inc_fixup {
-    (IS_MOD_PERL_2 && !IS_MOD_PERL_2_BUILD) ? "use Apache2;\n" : '';
-}
-
 sub modperl_build_config {
+
+    # we don't want to get mp2 preconfigured data in order to be able
+    # to get the interactive tests running.
+    return undef if $ENV{APACHE_TEST_INTERACTIVE_CONFIG_TEST};
+
     eval {
-        require Apache::Build;
+        require Apache2::Build;
     } or return undef;
-    return Apache::Build->build_config;
+    return Apache2::Build->build_config;
 }
 
 sub new_test_server {
@@ -206,8 +208,18 @@ sub new {
         $args->{$key} = $val;
     }
 
-    my $thaw = {};
+    my $top_dir = fastcwd;
+    $top_dir = pop_dir($top_dir, 't');
+    # untaint as we are going to use it a lot later on in -T sensitive
+    # operations (.e.g @INC)
+    $top_dir = $1 if $top_dir =~ /(.*)/;
 
+    # make sure that t/conf/apache_test_config.pm is found
+    # (unfortunately sometimes we get thrown into / by Apache so we
+    # can't just rely on $top_dir
+    lib->import($top_dir);
+
+    my $thaw = {};
     #thaw current config
     for (qw(conf t/conf)) {
         last if eval {
@@ -218,7 +230,7 @@ sub new {
             #something else, which we can't be sure how to load
             bless $thaw, 'Apache::TestConfig';
         };
-    };
+    }
 
     if ($args->{thaw} and ref($thaw) ne 'HASH') {
         #dont generate any new config
@@ -262,8 +274,7 @@ sub new {
         $self->{$_} = delete $args->{$_};
     }
 
-    $vars->{top_dir} ||= fastcwd;
-    $vars->{top_dir} = pop_dir($vars->{top_dir}, 't');
+    $vars->{top_dir} ||= $top_dir;
 
     $self->add_inc;
 
@@ -340,12 +351,23 @@ sub httpd_config {
     my $vars = $self->{vars};
     unless ($vars->{httpd} or $vars->{apxs}) {
 
-        # mod_perl 2.0 build always knows the right httpd location
-        # (and optionally apxs)
-        if (IS_MOD_PERL_2_BUILD) {
-            # XXX: at the moment not sure what could go wrong, but it
-            # shouldn't enter interactive config, which doesn't work
-            # with mod_perl 2.0 build (by design)
+        # mod_perl 2.0 build (almost) always knows the right httpd
+
+        # location (and optionally apxs). if we get here we can't
+        # continue because the interactive config can't work with
+        # mod_perl 2.0 build (by design)
+        if (IS_MOD_PERL_2_BUILD){
+            my $mp2_build = modperl_build_config();
+            # if mod_perl 2 was built against the httpd source it
+            # doesn't know where to find apxs/httpd, so in this case
+            # fall back to interactive config
+            unless ($mp2_build->{MP_APXS}) {
+                die "mod_perl 2 was built against Apache sources, we " .
+                "don't know where httpd/apxs executables are, therefore " .
+                "skipping the test suite execution"
+            }
+
+            # not sure what else could go wrong but we can't continue
             die "something is wrong, mod_perl 2.0 build should have " .
                 "supplied all the needed information to run the tests. " .
                 "Please post lib/Apache/BuildConfig.pm along with the " .
@@ -364,6 +386,9 @@ sub httpd_config {
         # so it never returns
         $self->custom_config_first_time($self->{vars});
     }
+    else {
+        debug "Using httpd: $vars->{httpd}";
+    }
 
     # if we have gotten that far we know at least about the location
     # of httpd and or apxs, so let's save it if we haven't saved any
@@ -1025,11 +1050,10 @@ sub open_cmd {
     my($self, $cmd) = @_;
     # untaint some %ENV fields
     local @ENV{ qw(IFS CDPATH ENV BASH_ENV) };
+    local $ENV{PATH} = untaint_path($ENV{PATH});
 
-    # Temporarly untaint PATH
-    (local $ENV{PATH}) = ( $ENV{PATH} =~ /(.*)/ );
-    # -T disallows relative directories in the PATH
-    $ENV{PATH} = join ':', grep !/^\./, split /:/, $ENV{PATH};
+    # launder for -T
+    $cmd = $1 if $cmd =~ /(.*)/;
 
     my $handle = Symbol::gensym();
     open $handle, "$cmd|" or die "$cmd failed: $!";
@@ -1185,6 +1209,18 @@ sub parse_vhost {
     };
 }
 
+sub find_and_load_module {
+    my ($self, $name) = @_;
+    my $mod_path = $self->find_apache_module($name) or return;
+    my ($sym) = $name =~ m/mod_(\w+)\./;
+
+    if ($mod_path && -e $mod_path) {
+        $self->preamble(IfModule => "!mod_$sym.c",
+                        qq{LoadModule ${sym}_module "$mod_path"\n});
+    }
+    return 1;
+}
+
 sub replace_vhost_modules {
     my $self = shift;
 
@@ -1238,11 +1274,7 @@ sub generate_types_config {
 
     # handle the case when mod_mime is built as a shared object
     # but wasn't included in the system-wide httpd.conf
-    my $mod_mime = $self->find_apache_module('mod_mime.so');
-    if ($mod_mime && -e $mod_mime) {
-        $self->preamble(IfModule => '!mod_mime.c',
-                        qq{LoadModule mime_module "$mod_mime"\n});
-    }
+    $self->find_and_load_module('mod_mime.so');
 
     unless ($self->{inherit_config}->{TypesConfig}) {
         my $types = catfile $self->{vars}->{t_conf}, 'mime.types';
@@ -1251,7 +1283,11 @@ sub generate_types_config {
             print $fh $self->types_config_template;
             close $fh;
         }
-        $self->postamble(TypesConfig => qq("$types"));
+        $self->postamble(<<EOI);
+<IfModule mod_mime.c>
+    TypesConfig "$types"
+</IfModule>
+EOI
     }
 }
 
@@ -1336,25 +1372,21 @@ sub generate_extra_conf {
         push @extra_conf, $generated;
     }
 
-    # if at least one .in file was modified or the derivative is
-    # missing, regenerate them all (so information like assigned port
-    # numbers will be correct)
-    if ($self->extra_conf_files_needing_update) {
-        for my $file (@conf_files) {
-            local $Apache::TestConfig::File = $file;
+    # regenerate .conf files
+    for my $file (@conf_files) {
+        local $Apache::TestConfig::File = $file;
 
-            my $in = Symbol::gensym();
-            open($in, $file) or next;
+        my $in = Symbol::gensym();
+        open($in, $file) or next;
 
-            (my $generated = $file) =~ s/\.in$//;
-            my $out = $self->genfile($generated, $file);
-            $self->replace_vars($in, $out);
+        (my $generated = $file) =~ s/\.in$//;
+        my $out = $self->genfile($generated, $file);
+        $self->replace_vars($in, $out);
 
-            close $in;
-            close $out;
+        close $in;
+        close $out;
 
-            $self->check_vars;
-        }
+        $self->check_vars;
     }
 
     #we changed order to give ssl the first port after DEFAULT_PORT
@@ -1494,6 +1526,8 @@ sub generate_httpd_conf {
 
     my $out = $self->genfile($conf_file);
 
+    $self->find_and_load_module('mod_alias.so');
+
     $self->preamble_run($out);
 
     for my $name (qw(user group)) { #win32/cygwin do not support
@@ -1515,14 +1549,6 @@ sub generate_httpd_conf {
 
     # handle the case when mod_alias is built as a shared object
     # but wasn't included in the system-wide httpd.conf
-    my $mod_alias = $self->find_apache_module('mod_alias.so');
-    if ($mod_alias && -e $mod_alias) {
-        print $out <<EOF;
-<IfModule !mod_alias.c>
-    LoadModule alias_module "$mod_alias"
-</IfModule>
-EOF
-    }
 
     print $out "<IfModule mod_alias.c>\n";
     for (keys %aliases) {
@@ -1618,7 +1644,13 @@ sub which {
 
     return undef unless $program;
 
-    for my $base (map { catfile($_, $program) } File::Spec->path()) {
+    my @dirs = File::Spec->path();
+
+    require Config;
+    my $perl_bin = $Config::Config{bin} || '';
+    push @dirs, $perl_bin if $perl_bin and -d $perl_bin;
+
+    for my $base (map { catfile $_, $program } @dirs) {
         if ($ENV{HOME} and not WIN32) {
             # only works on Unix, but that's normal:
             # on Win32 the shell doesn't have special treatment of '~'
@@ -1640,7 +1672,8 @@ sub apxs {
     return unless $self->{APXS};
     my $val;
     unless (exists $self->{_apxs}{$q}) {
-        local @ENV{ qw(PATH IFS CDPATH ENV BASH_ENV) };
+        local @ENV{ qw(IFS CDPATH ENV BASH_ENV) };
+        local $ENV{PATH} = untaint_path($ENV{PATH});
         my $devnull = devnull();
         my $apxs = shell_ready($self->{APXS});
         $val = qx($apxs -q $q 2>$devnull);
@@ -1661,6 +1694,16 @@ sub apxs {
     $self->{_apxs}{$q};
 }
 
+# Temporarily untaint PATH
+sub untaint_path {
+    my $path = shift;
+    ($path) = ( $path =~ /(.*)/ );
+    # win32 uses ';' for a path separator, assume others use ':'
+    my $sep = WIN32 ? ';' : ':';
+    # -T disallows relative and empty directories in the PATH
+    return join $sep, grep !/^(\.|$)/, split /$sep/, $path;
+}
+
 sub pop_dir {
     my $dir = shift;
 
@@ -1817,12 +1860,52 @@ sub custom_config_path {
     for (@inc) {
         my $candidate = File::Spec->rel2abs(catfile $_, CUSTOM_CONFIG_FILE);
         next unless -e $candidate;
+        # launder for -T
+        ($candidate) = $candidate =~ /^(.*)/;
         return $custom_config_path = $candidate;
     }
 
     return '';
 }
 
+# tries to nuke all occurences of custom config
+# used by things outside the A-T test suite
+sub custom_config_nuke {
+    my $cwd = fastcwd();
+
+    # 1) create a fake empty (blib/)?lib/Apache/TestConfigData.pm
+    # (don't delete it since it may mess up with MakeMaker)
+    my $path = catfile $cwd, "lib", Apache::TestConfig::CUSTOM_CONFIG_FILE;
+    # overwrite the empty stub
+    Apache::TestConfig::custom_config_write($path, '') if -e $path;
+
+    $path = catfile $cwd, "blib", "lib",
+        Apache::TestConfig::CUSTOM_CONFIG_FILE;
+    if (-e $path) {
+        my $mode = (stat _)[2];
+        my $mode_new = $mode | 0200;
+        chmod $mode_new, $path;
+        debug  "emptying $path";
+        Apache::TestConfig::custom_config_write($path, '');
+        chmod $mode, $path;
+    }
+
+    # 2) go through @INC = ~/.apache-test and nuke any occurences of
+    #    CUSTOM_CONFIG_FILE
+    my @inc  = ();
+
+    push @inc, catdir $ENV{HOME}, '.apache-test' if $ENV{HOME};
+
+    push @inc, @INC;
+
+    for (@inc) {
+        my $victim = File::Spec->rel2abs(catfile $_, CUSTOM_CONFIG_FILE);
+        next unless -e $victim;
+        debug "unlinking $victim";
+        unlink $victim;
+    }
+}
+
 sub custom_config_exists {
     # try to load custom config if it wasn't loaded yet (there are
     # many entry points to this API)
@@ -2072,6 +2155,12 @@ sub custom_config_load {
         return;
     }
 
+    if ($ENV{APACHE_TEST_INTERACTIVE_CONFIG_TEST}) {
+        debug "APACHE_TEST_INTERACTIVE_CONFIG_TEST=1 => " .
+            "skipping load of custom config data";
+        return;
+    }
+
     return if $custom_config_loaded;
 
     if (my $custom_config_path = custom_config_path()) {
@@ -2088,6 +2177,14 @@ sub custom_config_first_time {
     my $self = shift;
     my $conf_opts = shift;
 
+    # we can't prompt when STDIN is not attached to tty, unless we
+    # were told that's it OK via env var (in which case some program
+    # will feed the interactive prompts
+    unless (-t STDIN || $ENV{APACHE_TEST_INTERACTIVE_PROMPT_OK}) {
+        error "STDIN is not attached to tty, skip interactive config";
+        Apache::TestRun::skip_test_suite();
+    }
+
     my $vars = $self->{vars};
 
     print qq[
@@ -2117,8 +2214,8 @@ If for some reason you want to skip the test suite, type: skip
     {
         my %choices = ();
         my @tries = qw(httpd httpd2);
-        # Win32 uses Apache or perhaps Apache2, not apache/apache2
-        push @tries, WIN32 ? qw(Apache Apache2) : qw(apache apache2);
+        # Win32 uses Apache not apache
+        push @tries, WIN32 ? qw(Apache) : qw(apache);
         for (grep defined $_,
              map({ catfile $vars->{$_}, $vars->{target} } qw(sbindir bindir)),
              $self->default_httpd, which($vars->{target}),
@@ -2177,7 +2274,7 @@ or via the environment variable APACHE_TEST_APXS. For example:
     # we probably could reconfigure on the fly ($self->configure), but
     # the problem is various cached data which won't be refreshed. so
     # the simplest is just to restart the run from scratch
-    Apache::TestRun::rerun();
+    Apache::TestRun::rerun($vars);
 }
 
 sub _custom_config_prompt_path {
@@ -2234,14 +2331,14 @@ sub _custom_config_prompt_path {
         }
 
         unless (File::Spec->file_name_is_absolute($ans)) {
-            my $cwd = Cwd::cwd();
             warn "The path '$ans' is not an absolute path. " .
                 "Please specify an absolute path\n";
             next;
         }
 
-        warn("'$ans' doesn't exist\n"),     next unless -e $ans;
-        warn("'$ans' is not executable\n"), next unless -x $ans;
+        warn("'$ans' doesn't exist.\n"),     next unless -e $ans;
+        warn("'$ans' is not a file.\n"),     next unless -f _;
+        warn("'$ans' is not executable.\n"), next unless -x _;
 
         return $ans;
     }
@@ -2385,6 +2482,16 @@ configuration phase (C<t/TEST -config>, C<Apache::Test> will
 automatically unshift the I<project/lib> directory into C<@INC>, via
 the autogenerated I<t/conf/modperl_inc.pl> file.
 
+=head2 APACHE_TEST_INTERACTIVE_PROMPT_OK
+
+Normally interactive prompts aren't run when STDIN is not attached to
+a tty. But sometimes there is a program that can answer the prompts
+(e.g. when testing A-T itself). If this variable is true the
+interactive config won't be skipped (if needed).
+
+
+
+
 =head1 AUTHOR
 
 =head1 SEE ALSO
@@ -2421,6 +2528,7 @@ HostnameLookups Off
 </Directory>
 
 <IfModule @THREAD_MODULE@>
+    LockFile             @t_logs@/accept.lock
     StartServers         1
     MinSpareThreads      @MinClients@
     MaxSpareThreads      @MinClients@
@@ -2430,6 +2538,7 @@ HostnameLookups Off
 </IfModule>
 
 <IfModule perchild.c>
+    LockFile             @t_logs@/accept.lock
     NumServers           1
     StartThreads         @MinClients@
     MinSpareThreads      @MinClients@
@@ -2439,6 +2548,7 @@ HostnameLookups Off
 </IfModule>
 
 <IfModule prefork.c>
+    LockFile             @t_logs@/accept.lock
     StartServers         @MinClients@
     MinSpareServers      @MinClients@
     MaxSpareServers      @MinClients@
@@ -2447,6 +2557,7 @@ HostnameLookups Off
 </IfModule>
 
 <IfDefine APACHE1>
+    LockFile             @t_logs@/accept.lock
     StartServers         @MinClients@
     MinSpareServers      @MinClients@
     MaxSpareServers      @MinClients@
