@@ -1,3 +1,17 @@
+# Copyright 2001-2004 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 package Apache::TestConfig; #not TestConfigPerl on purpose
 
 #things specific to mod_perl
@@ -15,7 +29,7 @@ my %libmodperl  = (1 => 'libperl.so', 2 => 'mod_perl.so');
 sub configure_libmodperl {
     my $self = shift;
 
-    my $server = $self->{server};
+    my $server  = $self->{server};
     my $libname = $server->version_of(\%libmodperl);
     my $vars = $self->{vars};
 
@@ -79,7 +93,9 @@ sub configure_libmodperl {
         $cfg = "#$msg";
         debug $msg;
     }
+
     $self->preamble(IfModule => '!mod_perl.c', $cfg);
+
 }
 
 sub configure_inc {
@@ -109,9 +125,9 @@ sub configure_inc {
 }
 
 sub write_pm_test {
-    my($self, $module, $base, $sub) = @_;
+    my($self, $module, $sub, @base) = @_;
 
-    my $dir = catfile $self->{vars}->{t_dir}, $base;
+    my $dir = catfile $self->{vars}->{t_dir}, @base;
     my $t = catfile $dir, "$sub.t";
     return if -e $t;
 
@@ -161,10 +177,13 @@ sub configure_startup_pl {
         my $include_pl = catfile $self->{vars}->{t_conf}, 'modperl_inc.pl';
         my $fh = $self->genfile($include_pl);
         for (reverse @$inc) {
+            next unless $_;
             print $fh "use lib '$_';\n";
         }
-        my $fixup = Apache::TestConfig->modperl_2_inc_fixup();
-        print $fh $fixup;
+        my $tlib = catdir $self->{vars}->{t_dir}, 'lib';
+        if (-d $tlib) {
+            print $fh "use lib '$tlib';\n";
+        }
 
         # if Apache::Test is used to develop a project, we want the
         # project/lib directory to be first in @INC (loaded last)
@@ -207,7 +226,8 @@ sub set_handler {
 sub set_connection_handler {
     my($self, $module, $args) = @_;
     my $port = $self->new_vhost($module);
-    $self->postamble(Listen => $port);
+    my $vars = $self->{vars};
+    $self->postamble(Listen => '0.0.0.0:' . $port);
 }
 
 my %add_hook_config = (
@@ -465,14 +485,6 @@ sub configure_pm_tests_sort {
 sub configure_pm_tests {
     my $self = shift;
 
-    # since server wasn't started yet, the modules in blib under
-    # Apache2 can't be seen. So we must load Apache2.pm, without which
-    # run_apache_test_config might fail to require modules
-    require mod_perl;
-    if ($mod_perl::VERSION > 1.99) {
-        require Apache2;
-    }
-
     my @entries = ();
     $self->configure_pm_tests_pick(\@entries);
     $self->configure_pm_tests_sort(\@entries);
@@ -488,9 +500,11 @@ sub configure_pm_tests {
 
         $self->run_apache_test_configure($file, $module, $status);
 
-        my($base, $sub) =
+        my @base =
             map { s/^test//i; $_ } split '::', $module;
 
+        my $sub = pop @base;
+
         my $hook = ($subdir eq 'Hooks' ? $hooks{$sub} : '')
             || $hooks{$subdir} || $subdir;
 
@@ -531,7 +545,7 @@ sub configure_pm_tests {
             $self->postamble($self->$container($module), \@args) if @args;
         }
 
-        $self->write_pm_test($module, lc $base, lc $sub);
+        $self->write_pm_test($module, lc $sub, map { lc } @base);
     }
 }
 
