@@ -60,7 +60,6 @@ EOF
     my $cover;
 
     if (eval { require Devel::Cover }) {
-                                                                                                                             
         my $atdir = File::Spec->catfile($ENV{HOME}, '.apache-test');
 
         $cover = <<"EOF"
@@ -75,7 +74,7 @@ EOF
     else {
 
         $cover = <<'EOF';
-                                                                                                                             
+
 testcover :
 	@echo "Cannot run testcover action unless Devel::Cover is installed"
 EOF
@@ -100,10 +99,10 @@ test_config :
 	$(PASSENV) \
 	$(FULLPERL) -I$(INST_ARCHLIB) -I$(INST_LIB) \
 	t/TEST $(APACHE_TEST_EXTRA_ARGS) -conf
-                                                                                                                             
+
 cmodules: test_config
 	cd c-modules && $(MAKE) all
-                                                                                                                             
+
 cmodules_clean: test_config
 	cd c-modules && $(MAKE) clean
 EOF
@@ -117,8 +116,6 @@ sub generate_script {
 
     my $body = "BEGIN { eval { require blib && blib->import; } }\n";
 
-    $body .= Apache::TestConfig->modperl_2_inc_fixup;
-
     my %args = @Apache::TestMM::Argv;
     while (my($k, $v) = each %args) {
         $v =~ s/\|/\\|/g;
@@ -247,7 +244,6 @@ our example it'll look for F<t/TEST.PL>. The autogenerated script
 F<t/TEST> will include the contents of F<t/TEST.PL>, and special
 directives, including any configuration options passed via
 C<L<filter_args()|/C_filter_args_>> called from F<Makefile.PL>, special
-fixup code, etc. If no argument is passed to C<generate_script()>,
-it will create a file named F<t/TEST> by default.
+fixup code, etc.
 
 =cut
