@@ -67,14 +67,8 @@ sub post_config {
     # string and various variations made by distributions which mangle
     # that string
 
-    # Apache/2.0.50-dev
-    ($self->{rev})   = $self->{version} =~ m|^Apache/(\d)\.|;
-
-    # Apache-AdvancedExtranetServer/1.3.29 (Mandrake Linux/1mdk)
-    ($self->{rev}) ||= $self->{version} =~ m|^Apache.*?/(\d)\.|;
-
-    # IBM_HTTP_SERVER/1.3.19  Apache/1.3.20 (Unix)
-    ($self->{rev}) ||= $self->{version} =~ m|^.*?Apache.*?/(\d)\.|;
+    # Foo-Apache-Bar/x.y.z
+    ($self->{rev}) = $self->{version} =~ m|/(\d)\.|;
 
     if ($self->{rev}) {
         debug "Matched Apache revision $self->{version} $self->{rev}";
@@ -105,6 +99,7 @@ my @apache_logs = qw(
 error_log access_log httpd.pid
 apache_runtime_status rewrite_log
 ssl_engine_log ssl_request_log
+cgisock
 );
 
 sub clean {
@@ -159,10 +154,20 @@ sub args {
 my %one_process = (1 => '-X', 2 => '-D ONE_PROCESS');
 
 sub start_cmd {
-    my $self = shift;
+    my $self  = shift;
+
+    my $args   = $self->args;
+    my $config = $self->{config};
+    my $vars   = $config->{vars};
+    my $httpd  = $vars->{httpd};
+
+    my $one_process = $self->{run}->{opts}->{'one-process'}
+        ? $self->version_of(\%one_process)
+        : '';
+
     #XXX: threaded mpm does not respond to SIGTERM with -D ONE_PROCESS
-    my $args = $self->args;
-    return "$self->{config}->{vars}->{httpd} $args";
+
+    return "$httpd $one_process $args";
 }
 
 sub default_gdbinit {
