@@ -1,4 +1,4 @@
-# Copyright 2001-2004 The Apache Software Foundation
+# Copyright 2001-2005 The Apache Software Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -50,9 +50,9 @@ my %aliases = (
     cache_table     => 'cache_table_common',
     cache_it        => 'cache_it',
     read_script     => 'read_script',
-    rewrite_shebang => 'rewrite_shebang',
+    shebang_to_perl => 'shebang_to_perl',
     get_script_name => 'get_script_name',
-    chdir_file      => 'chdir_file_normal',
+    chdir_file      => 'NOP',
     get_mark_line   => 'get_mark_line',
     compile         => 'compile',
     error_check     => 'error_check',
