@@ -1,13 +1,30 @@
+# Copyright 2001-2005 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 package ModPerl::RegistryLoader;
 
 use strict;
 use warnings;
 
 use ModPerl::RegistryCooker ();
+use Apache2::ServerUtil ();
 use APR::Pool ();
 
-use Apache::Const -compile => qw(OK HTTP_OK OPT_EXECCGI);
 use Carp;
+use File::Spec ();
+
+use Apache2::Const -compile => qw(OK HTTP_OK OPT_EXECCGI);
 
 our @ISA = ();
 
@@ -54,7 +71,8 @@ sub handler {
             $self->warn("Trying to guess filename based on uri")
                 if $self->{debug};
 
-            $filename = Apache::server_root_relative($self->{pool}, $guess);
+            $filename = File::Spec->catfile(Apache2::ServerUtil::server_root,
+                                            $guess);
             unless (-e $filename) {
                 $self->warn("Cannot find guessed file: $filename",
                             "provide \$filename or 'trans' sub");
@@ -90,13 +108,13 @@ sub handler {
 
 sub get_server_name { return $_[0]->{virthost} if exists $_[0]->{virthost} }
 sub filename { shift->{filename} }
-sub status { Apache::HTTP_OK }
+sub status { Apache2::Const::HTTP_OK }
 sub my_finfo    { shift->{filename} }
 sub uri      { shift->{uri} }
 sub path_info {}
-sub allow_options { Apache::OPT_EXECCGI } #will be checked again at run-time
+sub allow_options { Apache2::Const::OPT_EXECCGI } #will be checked again at run-time
 sub log_error { shift; die @_ if $@; warn @_; }
-sub run { return Apache::OK } # don't run the script
+sub run { return Apache2::Const::OK } # don't run the script
 sub server { shift }
 sub is_virtual { exists shift->{virthost} }
 
@@ -109,12 +127,17 @@ sub namespace_root {
 
 # override Apache class methods called by Modperl::Registry*. normally
 # only available at request-time via blessed request_rec pointer
-sub my_slurp_filename {
+sub slurp_filename {
     my $r = shift;
+    my $tainted = @_ ? shift : 1;
     my $filename = $r->filename;
     open my $fh, $filename or die "can't open $filename: $!";
     local $/;
     my $code = <$fh>;
+    unless ($tainted) {
+        ($code) = $code =~ /(.*)/s; # untaint
+    }
+    close $fh;
     return \$code;
 }
 
@@ -130,7 +153,7 @@ sub load_package {
 
 sub warn {
     my $self = shift;
-    Apache->warn(__PACKAGE__ . ": @_\n");
+    Apache2->warn(__PACKAGE__ . ": @_\n");
 }
 
 1;
