@@ -2,14 +2,21 @@ use strict;
 use warnings FATAL => 'all';
 
 use ModPerl::RegistryLoader ();
-use Apache::Server ();
-use Apache::ServerUtil ();
-use Apache::Process ();
+
+use Apache2::ServerRec ();
+use Apache2::ServerUtil ();
+use Apache2::Process ();
 
 use DirHandle ();
 
-my $pool = Apache->server->process->pool;
-my $base_dir = Apache::Server::server_root_relative($pool, "cgi-bin");
+my $proc = Apache2::ServerUtil->server->process;
+my $pool = $proc->pool;
+
+# can't use catfile with server_root as it contains unix dir
+# separators and in a few of our particular tests we compare against
+# win32 separators. in general avoid using server_root_relative in your
+# code, see the manpage for more details
+my $base_dir = Apache2::ServerUtil::server_root_relative($pool, "cgi-bin");
 
 # test the scripts pre-loading by explicitly specifying uri => filename
 my $rl = ModPerl::RegistryLoader->new(package => "ModPerl::Registry");
@@ -26,7 +33,7 @@ for my $file (qw(basic.pl env.pl)) {
     sub trans {
         my $uri = shift; 
         $uri =~ s|^/registry_bb/|cgi-bin/|;
-        return Apache::Server::server_root_relative($pool, $uri);
+        return Apache2::ServerUtil::server_root_relative($pool, $uri);
     }
 
     my $rl = ModPerl::RegistryLoader->new(
