@@ -1,4 +1,4 @@
-# Copyright 2003-2004 The Apache Software Foundation
+# Copyright 2003-2005 The Apache Software Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,8 +23,9 @@ use File::Spec::Functions qw(catdir catfile splitdir);
 use File::Basename;
 use File::Find;
 
-use Apache::Build ();
+use Apache2::Build ();
 use ModPerl::MM;
+use constant WIN32 => Apache2::Build::WIN32;
 
 our %PM; #add files to installation
 
@@ -41,12 +42,14 @@ my $apache_test_dir = catdir Cwd::getcwd(), "Apache-Test", "lib";
 sub mod_install {
     q{$(PERL) -I$(INST_LIB) -I$(PERL_LIB) \\}."\n" .
     qq{-I$apache_test_dir -MModPerl::BuildMM \\}."\n" .
-    q{-e "ModPerl::MM::install({@ARGV},'$(VERBINST)',0,'$(UNINST)');"}."\n";
+    q{-e "ExtUtils::Install::install({@ARGV},'$(VERBINST)',0,'$(UNINST)');"}."\n";
 }
 
+my $build;
+
 sub build_config {
     my $key = shift;
-    my $build = Apache::Build->build_config;
+    $build ||= Apache2::Build->build_config;
     return $build unless $key;
     $build->{$key};
 }
@@ -68,7 +71,7 @@ sub my_import {
 sub WriteMakefile {
     my %args = @_;
 
-    my $build = build_config();
+    $build ||= build_config();
     ModPerl::MM::my_import(__PACKAGE__);
 
     my $inc = $build->inc;
@@ -80,7 +83,7 @@ sub WriteMakefile {
 
     my $libs;
     my @libs = ();
-    if (Apache::Build::BUILD_APREXT) {
+    if (Apache2::Build::BUILD_APREXT) {
         # in order to decouple APR/APR::* from mod_perl.so,
         # link these modules against the static MP_APR_LIB lib,
         # rather than the mod_perl lib (which would demand mod_perl.so
@@ -129,17 +132,11 @@ sub WriteMakefile {
 }
 
 my %always_dynamic = map { $_, 1 }
-  qw(ModPerl::Const Apache::Const APR::Const APR APR::PerlIO);
+  qw(ModPerl::Const Apache2::Const APR::Const APR APR::PerlIO);
 
 sub ModPerl::BuildMM::MY::constants {
     my $self = shift;
-    my $build = build_config();
-
-    #install everything relative to the Apache2/ subdir
-    if ($build->{MP_INST_APACHE2}) {
-        $self->{INST_ARCHLIB} = catdir $self->{INST_ARCHLIB}, 'Apache2';
-        $self->{INST_LIB} = catdir $self->{INST_LIB}, 'Apache2';
-    }
+    $build ||= build_config();
 
     #"discover" xs modules.  since there is no list hardwired
     #any module can be unpacked in the mod_perl-2.xx directory
@@ -191,7 +188,12 @@ sub ModPerl::BuildMM::MY::postamble {
     # add the code to glue the existing pods to the .pm files in blib.
     # create a dependency on pm_to_blib subdirs linkext targets to
     # allow 'make -j'
-    my @target = ('glue_pods: pm_to_blib subdirs linkext');
+    require ExtUtils::MakeMaker;
+    my $mm_ver = $ExtUtils::MakeMaker::VERSION;
+    my $pm_to_blib = ($mm_ver >= 6.22 && $mm_ver <= 6.25)
+        ? "pm_to_blib.ts"
+        : "pm_to_blib";
+    my @target = ("glue_pods: $pm_to_blib subdirs linkext");
 
     if (-d $doc_root) {
         my $build = build_config();
@@ -201,8 +203,6 @@ sub ModPerl::BuildMM::MY::postamble {
 
         my $cwd = Cwd::getcwd();
         my $blib_dir = catdir qw(blib lib);
-        $blib_dir = catdir $blib_dir, "Apache2"
-            if $build->{MP_INST_APACHE2};
 
         # those autogenerated under WrapXS/
         # those living under xs/
@@ -234,6 +234,8 @@ sub ModPerl::BuildMM::MY::postamble {
         }
 
         while (my($pm, $blib) = each %pms) {
+            $pm   =~ s|/\./|/|g; # clean the path
+            $blib =~ s|/\./|/|g; # clean the path
             my @segm = splitdir $blib;
             for my $i (1..2) {
                 # try APR.pm and APR/Bucket.pm
@@ -246,6 +248,17 @@ sub ModPerl::BuildMM::MY::postamble {
                     '$(FULLPERL) -I$(INST_LIB) ' .
                     "-I$apache_test_dir -MModPerl::BuildMM " .
                     "-e ModPerl::BuildMM::glue_pod $pm $podpath $blib";
+
+                # Win32 doesn't normally install man pages
+                next if WIN32;
+
+                # manify while we're at it
+                my (undef, $man, undef) = $blib =~ m!(blib/lib/)(.*)(\.pm)!;
+                $man =~ s!/!::!g;
+
+                push @target,
+                    '$(NOECHO) $(POD2MAN_EXE) --section=3 ' .
+                    "$podpath \$(INST_MAN3DIR)/$man.\$(MAN3EXT)"
             }
         }
 
@@ -271,8 +284,10 @@ sub glue_pod {
     die "expecting 3 arguments: pm, pod, dst" unless @ARGV == 3;
     my($pm, $pod, $dst) = @ARGV;
 
-    die "glue_pod: can't find $pm"  unless -e $pm;
-    die "glue_pod: can't find $dst" unless -e $dst;
+    # it's possible that the .pm file is not existing
+    # (e.g. ThreadMutex.pm is not created on unless
+    # $apr_config->{HAS_THREADS})
+    return unless -e $pm && -e $dst;
 
     # have we already glued the doc?
     exit 0 unless -s $pm == -s $dst;
@@ -295,7 +310,7 @@ sub glue_pod {
 
 sub ModPerl::BuildMM::MY::post_initialize {
     my $self = shift;
-    my $build = build_config();
+    $build ||= build_config();
     my $pm = $self->{PM};
 
     while (my($k, $v) = each %PM) {
@@ -304,51 +319,31 @@ sub ModPerl::BuildMM::MY::post_initialize {
         }
     }
 
-    # prefix typemap with Apache/ so when installed in the
+    # prefix typemap with Apache2/ so when installed in the
     # perl-lib-tree it won't be picked by non-mod_perl modules
     if (exists $pm->{'lib/typemap'} ) {
-        $pm->{'lib/typemap'} = '$(INST_ARCHLIB)/auto/Apache/typemap';
-    }
-
-    #not everything in MakeMaker uses INST_LIB
-    #so we have do fixup a few PMs to make sure *everything*
-    #gets installed into Apache2/
-    if ($build->{MP_INST_APACHE2}) {
-        while (my($k, $v) = each %$pm) {
-            #up one from the Apache2/ subdir
-            #so it can be found for 'use Apache2 ()'
-            next if $v =~ /Apache2\.pm$/;
-
-            # another module generated by A-T that needs to go to the
-            # normal @INC
-            next if $v =~ /TestConfigData\.pm$/;
-
-            #move everything else to the Apache2/ subdir
-            #unless already specified with \$(INST_LIB)
-            #or already in Apache2/
-            unless ($v =~ /Apache2/) {
-                $v =~ s{ (blib[/\\:]lib) }{ catdir $1, 'Apache2'}xe;
-            }
-
-            $pm->{$k} = $v;
-        }
+        $pm->{'lib/typemap'} = '$(INST_ARCHLIB)/auto/Apache2/typemap';
     }
 
     '';
 }
 
+my $apr_config;
+
 sub ModPerl::BuildMM::MY::libscan {
     my($self, $path) = @_;
 
-    my $apr_config = build_config()->get_apr_config();
+    $apr_config ||= $build->get_apr_config();
 
     if ($path =~ m/(Thread|Global)Mutex/) {
         return unless $apr_config->{HAS_THREADS};
     }
 
-    return '' if $path =~ m/\.(pl|cvsignore)$/;
-    return '' if (basename dirname $path) eq 'CVS';
+    return '' if $path =~ /DummyVersions.pm/;
+
+    return '' if $path =~ m/\.pl$/;
     return '' if $path =~ m/~$/;
+    return '' if $path =~ /\B\.svn\b/;
 
     $path;
 }
