@@ -1,4 +1,4 @@
-# Copyright 2000-2004 The Apache Software Foundation
+# Copyright 2000-2005 The Apache Software Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,14 +20,14 @@ use warnings FATAL => 'all';
 use Config;
 use File::Spec::Functions qw(catfile catdir);
 
-use mod_perl ();
-use Apache::Build ();
+use mod_perl2 ();
+use Apache2::Build ();
 
 use Apache::TestConfig ();
 use Apache::TestTrace;
 
 our $VERSION = '0.01';
-our @ISA = qw(Apache::Build);
+our @ISA = qw(Apache2::Build);
 
 my %handlers = (
     Process    => [qw(ChildInit ChildExit)], #Restart PreConfig
@@ -364,6 +364,7 @@ AP_INIT_ITERATE("$h_name", $name, NULL, \\
 
 EOF
             print $c_fh <<EOF;
+
 $protostr
 {
     $prototype->{cfg}->{get};
@@ -487,22 +488,28 @@ my %trace = (
     'd' => 'directive processing',
     'e' => 'environment variables',
     'f' => 'filters',
-    'g' => 'Perl runtime interaction',
+    'g' => 'globals management',
     'h' => 'handlers',
     'i' => 'interpreter pool management',
     'm' => 'memory allocations',
     'o' => 'I/O',
-    's' => 'perl sections',
+    'r' => 'Perl runtime interaction',
+    's' => 'Perl sections',
     't' => 'benchmark-ish timings',
 );
 
 sub generate_trace {
     my($self, $h_fh) = @_;
 
-    my $v = $self->{build}->{VERSION};
+    my $v     = $self->{build}->{VERSION};
+    my $api_v = $self->{build}->{API_VERSION};
 
     print $h_fh qq(#define MP_VERSION_STRING "mod_perl/$v"\n);
 
+    # this needs to be a string, not an int, because of the
+    # macro definition.  patches welcome.
+    print $h_fh qq(#define MP_API_VERSION "$api_v"\n);
+
     my $i = 1;
     my @trace = sort keys %trace;
     my $opts = join '', @trace;
@@ -570,6 +577,7 @@ sub generate_largefiles {
 
 sub ins_underscore {
     $_[0] =~ s/([a-z])([A-Z])/$1_$2/g;
+    $_[0] =~ s/::/_/g;
 }
 
 sub canon_uc {
@@ -635,14 +643,16 @@ my %sources = (
    generate_largefiles         => {h => 'modperl_largefiles.h'},
    generate_constants          => {h => 'modperl_constants.h',
                                    c => 'modperl_constants.c'},
+   generate_exports            => {c => 'modperl_exports.c'},
 );
 
 my @c_src_names = qw(interp tipool log config cmd options callback handler
                      gtop util io io_apache filter bucket mgv pcw global env
                      cgi perl perl_global perl_pp sys module svptr_table
-                     const constants apache_compat);
+                     const constants apache_compat error debug
+                     common_util common_log);
 my @h_src_names = qw(perl_unembed);
-my @g_c_names = map { "modperl_$_" } qw(hooks directives flags xsinit);
+my @g_c_names = map { "modperl_$_" } qw(hooks directives flags xsinit exports);
 my @c_names   = ('mod_perl', (map "modperl_$_", @c_src_names));
 sub c_files { [map { "$_.c" } @c_names, @g_c_names] }
 sub o_files { [map { "$_.o" } @c_names, @g_c_names] }
@@ -651,7 +661,8 @@ sub o_pic_files { [map { "$_.lo" } @c_names, @g_c_names] }
 my @g_h_names = map { "modperl_$_" } qw(hooks directives flags trace
                                         largefiles);
 my @h_names = (@c_names, map { "modperl_$_" } @h_src_names,
-               qw(types time apache_includes perl_includes));
+               qw(types time apache_includes perl_includes apr_includes
+                  apr_compat common_includes common_types));
 sub h_files { [map { "$_.h" } @h_names, @g_h_names] }
 
 sub clean_files {
@@ -774,12 +785,12 @@ sub generate {
     #$self->generate_constants_pod();
 }
 
-my $constant_prefixes = join '|', qw{APR?};
+my $constant_prefixes = join '|', qw{APR? MODPERL_RC};
 
 sub generate_constants {
     my($self, $h_fh, $c_fh) = @_;
 
-    require Apache::ConstantsTable;
+    require Apache2::ConstantsTable;
 
     print $c_fh qq{\#include "modperl_const.h"\n};
     print $h_fh "#define MP_ENOCONST -3\n\n";
@@ -798,7 +809,12 @@ my %shortcuts = (
 
 #backwards compat with older httpd/apr
 #XXX: remove once we require newer httpd/apr
-my %ifdef = map { $_, 1 } qw(APLOG_TOCLIENT APR_LIMIT_NOFILE);
+my %ifdef = map { $_, 1 } 
+    qw(APLOG_TOCLIENT APR_LIMIT_NOFILE), # added in ???
+    qw(AP_MPMQ_STARTING AP_MPMQ_RUNNING AP_MPMQ_STOPPING 
+       AP_MPMQ_MPM_STATE), # added in 2.0.49
+    qw(APR_FPROT_USETID APR_FPROT_GSETID
+       APR_FPROT_WSTICKY APR_FOPEN_LARGEFILE); # added in 2.0.50?
 
 sub constants_ifdef {
     my $name = shift;
@@ -817,9 +833,10 @@ sub constants_lookup_code {
 
     %alias = %shortcuts;
 
-    my $postfix = lc $class;
+    my $postfix = canon_lc(lc $class);
     my $package = $class . '::';
     my $package_len = length $package;
+    my($first_let) = $class =~ /^(\w)/;
 
     my $func = canon_func(qw(constants lookup), $postfix);
     my $proto = "SV \*$func(pTHX_ const char *name)";
@@ -830,7 +847,7 @@ sub constants_lookup_code {
 
 $proto
 {
-    if (*name == 'A' && strnEQ(name, "$package", $package_len)) {
+    if (*name == '$first_let' && strnEQ(name, "$package", $package_len)) {
         name += $package_len;
     }
 
@@ -891,7 +908,7 @@ EOF
 sub generate_constants_lookup {
     my($h_fh, $c_fh) = @_;
 
-    while (my($class, $groups) = each %$Apache::ConstantsTable) {
+    while (my($class, $groups) = each %$Apache2::ConstantsTable) {
         my $constants = [map { @$_ } values %$groups];
 
         constants_lookup_code($h_fh, $c_fh, $constants, $class);
@@ -901,7 +918,7 @@ sub generate_constants_lookup {
 sub generate_constants_group_lookup {
     my($h_fh, $c_fh) = @_;
 
-    while (my($class, $groups) = each %$Apache::ConstantsTable) {
+    while (my($class, $groups) = each %$Apache2::ConstantsTable) {
         constants_group_lookup_code($h_fh, $c_fh, $class, $groups);
     }
 }
@@ -911,7 +928,7 @@ sub constants_group_lookup_code {
     my @tags;
     my @code;
 
-    $class = lc $class;
+    $class = canon_lc(lc $class);
     while (my($group, $constants) = each %$groups) {
 	push @tags, $group;
         my $name = join '_', 'MP_constants', $class, $group;
@@ -955,7 +972,7 @@ EOF
 }
 
 my %seen_const = ();
-# generates APR::Const and Apache::Const manpages in ./tmp/
+# generates APR::Const and Apache2::Const manpages in ./tmp/
 sub generate_constants_pod {
     my($self) = @_;
 
@@ -1011,7 +1028,7 @@ EOF
 sub generate_constants_lookup_doc {
     my($data) = @_;
 
-    while (my($class, $groups) = each %$Apache::ConstantsTable) {
+    while (my($class, $groups) = each %$Apache2::ConstantsTable) {
         my $constants = [map { @$_ } values %$groups];
 
         constants_lookup_code_doc($constants, $class, $data);
@@ -1021,7 +1038,7 @@ sub generate_constants_lookup_doc {
 sub generate_constants_group_lookup_doc {
     my($data) = @_;
 
-    while (my($class, $groups) = each %$Apache::ConstantsTable) {
+    while (my($class, $groups) = each %$Apache2::ConstantsTable) {
         constants_group_lookup_code_doc($class, $groups, $data);
     }
 }
@@ -1077,6 +1094,19 @@ sub constants_lookup_code_doc {
     }
 }
 
+sub generate_exports {
+    my($self, $c_fh) = @_;
+    require ModPerl::WrapXS;
+    ModPerl::WrapXS->generate_exports($c_fh);
+}
+
+# src/modules/perl/*.c files needed to build APR/APR::* outside
+# of mod_perl.so
+sub src_apr_ext {
+    return map { "modperl_$_" } (qw(error bucket),
+                                  map { "common_$_" } qw(util log));
+}
+
 1;
 __END__
 
