@@ -1,3 +1,17 @@
+# Copyright 2001-2005 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 package ModPerl::FunctionMap;
 
 use strict;
@@ -70,7 +84,7 @@ sub guess_prefix {
     (my $modprefix = ($entry->{class} || $entry->{module}) . '_') =~ s/::/__/g;
     (my $guess = lc $modprefix) =~ s/_+/_/g;
 
-    $guess =~ s/(apache)_/($1|ap)_{1,2}/;
+    $guess =~ s/(apache2)_/($1|ap)_{1,2}/;
 
     if ($name =~ s/^($guess|$modprefix).*/$1/i) {
         $prefix = $1;
@@ -121,6 +135,9 @@ sub parse {
 
         if ($name =~ s/^(\W)// or not $cur{MODULE} or $disabled) {
             #notimplemented or cooked by hand
+            die qq[function '$name' appears more than once in xs/maps files]
+                if $map->{$name};
+
             $map->{$name} = undef;
             push @{ $self->{disabled}->{ $1 || '!' } }, $name;
             next;
@@ -147,6 +164,9 @@ sub parse {
               {$1 . ModPerl::WrapXS::make_prefix($2, $cur{CLASS})}e;
         }
 
+        die qq[function '$name' appears more than once in xs/maps files]
+            if $map->{$name};
+
         my $entry = $map->{$name} = {
            name        => $alias || $name,
            dispatch    => $dispatch,
