@@ -1,4 +1,4 @@
-# Copyright 2002-2004 The Apache Software Foundation
+# Copyright 2002-2005 The Apache Software Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -34,32 +34,39 @@ my @add_files = qw{
     Apache-Test/META.yml
 };
 
-sub get_cvs_files {
+sub get_svn_files {
     my @files;
 
     my $cwd = Cwd::cwd();
 
     finddepth({ follow => 1, wanted => sub {
-        return unless $_ eq 'Entries';
+        return unless $_ eq 'entries';
+        return unless $File::Find::dir =~ /\.svn$/;
 
         my $dir = dirname $File::Find::dir;
         $dir =~ s,^$cwd/?,,;
 
         open my $fh, $_ or die "open $_: $!";
         while (my $line = <$fh>) {
-            my $file = (split '/', $line)[1];
-            next if !$file or -d "../$file" or $file =~ /^\./;
-
-            push @files, $dir ? "$dir/$file" : $file;
+             if ($line =~ /name\s*=\s*"([^"]*)"/) {
+                my $file = $1;
+                next if $file eq 'svn:this_dir';
+                next if !$file or -d "../$file" or $file =~ /^\./;
+                push @files, $dir ? "$dir/$file" : $file;
+             }
         }
         close $fh;
+
     }}, $cwd);
 
+    # files to add which aren't under svn
+    push @files, qw(lib/ModPerl/DummyVersions.pm);
+
     return @files;
 }
 
 sub mkmanifest {
-    my @files = (@add_files, get_cvs_files());
+    my @files = (@add_files, get_svn_files());
 
     my $matches = maniskip();
     open my $fh, '>', 'MANIFEST' or die "open MANIFEST: $!";
@@ -109,6 +116,16 @@ eliza
 t/perl/ithreads.t
 t/perl/ithreads2.t
 t/response/TestPerl/ithreads.pm
+t/response/TestPerl/ithreads_args.pm
+t/response/TestPerl/ithreads_eval.pm
 # incomplete
 t/apr-ext/perlio
-t/modperl/merge3
+# PAUSE breaks if a dist has more than one META.yml. the top-level
+# META.yml already excludes Apache-Test from indexing
+Apache-Test/META.yml
+# exclude Apache-Test/MANIFEST since it confuses the mp2 build (e.g it
+# wants Apache-Test/META.yml which we don't supply, see above)
+Apache-Test/MANIFEST
+
+# this is an internal to developers sub-project
+Apache-Test/Apache-TestItSelf
