@@ -1,4 +1,4 @@
-# Copyright 2001-2004 The Apache Software Foundation
+# Copyright 2001-2005 The Apache Software Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,7 @@ use warnings FATAL => 'all';
 
 use constant GvUNIQUE => 0; #$] >= 5.008;
 use Apache::TestTrace;
-use Apache::Build ();
+use Apache2::Build ();
 use ModPerl::Code ();
 use ModPerl::TypeMap ();
 use ModPerl::MapUtil qw(function_table xs_glue_dirs);
@@ -35,8 +35,8 @@ my(@xs_includes) = ('mod_perl.h',
 
 my @global_structs = qw(perl_module);
 
-my $build = Apache::Build->build_config;
-push @global_structs, 'MP_debug_level' unless Apache::Build::WIN32;
+my $build = Apache2::Build->build_config;
+push @global_structs, 'MP_debug_level' unless Apache2::Build::WIN32;
 
 sub new {
     my $class = shift;
@@ -187,8 +187,8 @@ sub get_structures {
     my $self = shift;
     my $typemap = $self->typemap;
 
-    require Apache::StructureTable;
-    for my $entry (@$Apache::StructureTable) {
+    require Apache2::StructureTable;
+    for my $entry (@$Apache2::StructureTable) {
         my $struct = $typemap->map_structure($entry);
         next unless $struct;
 
@@ -254,6 +254,37 @@ $attrs
     OUTPUT:
     RETVAL
 
+EOF
+            }
+            elsif ($access_mode eq 'r+w_startup_dup') {
+
+                my $convert = $cast !~ /\bchar\b/
+                    ? "mp_xs_sv2_$cast"
+                    : "SvPV_nolen";
+
+                $code = <<EOF;
+$type
+$name(obj, val=Nullsv)
+    $class obj
+    SV *val
+
+    PREINIT:
+    $preinit
+$attrs
+
+    CODE:
+    RETVAL = ($cast) obj->$name;
+
+    if (items > 1) {
+         SV *dup = get_sv("_modperl_private::server_rec_$name", TRUE);
+         MP_CROAK_IF_THREADS_STARTED("setting $name");
+         sv_setsv(dup, val);
+         obj->$name = ($cast)$convert(dup);
+    }
+
+    OUTPUT:
+    RETVAL
+
 EOF
             }
             elsif ($access_mode eq 'rw_char_undef') {
@@ -317,7 +348,7 @@ sub prepare {
 sub class_dirname {
     my($self, $class) = @_;
     my($base, $sub) = split '::', $class;
-    return "$self->{DIR}/$base" unless $sub; #Apache | APR
+    return "$self->{DIR}/$base" unless $sub; #Apache2 | APR
     return $sub if $sub eq $self->{DIR}; #WrapXS
     return "$base/$sub";
 }
@@ -364,6 +395,13 @@ sub open_class_file {
     return $fh;
 }
 
+sub module_version {
+    local $_ = shift;
+    require mod_perl2;
+    # XXX: for now APR gets its libapr-0.9 version
+    return /^APR/ ? "0.009000" : "$mod_perl2::VERSION";
+}
+
 sub write_makefilepl {
     my($self, $class) = @_;
 
@@ -381,16 +419,18 @@ sub write_makefilepl {
     $deps = Dumper $deps;
 
     my $noedit_warning = $self->ModPerl::Code::noedit_warning_hash();
+    require mod_perl2;
+    my $version = module_version($class);
 
     print $fh <<EOF;
 $noedit_warning
 
-use lib qw(../../../lib); #for Apache::BuildConfig
+use lib qw(../../../lib); #for Apache2::BuildConfig
 use ModPerl::BuildMM ();
 
 ModPerl::BuildMM::WriteMakefile(
     'NAME'    => '$class',
-    'VERSION' => '0.01',
+    'VERSION' => '$version',
     'depend'  => $deps,
 );
 EOF
@@ -517,7 +557,7 @@ sub write_xs {
         $last_prefix = $prefix if $prefix;
 
         if ($func->{name} =~ /^mpxs_/) {
-            #e.g. mpxs_Apache__RequestRec_
+            #e.g. mpxs_Apache2__RequestRec_
             my $class_prefix = class_c_prefix($class);
             if ($func->{name} =~ /$class_prefix/) {
                 $prefix = class_mpxs_prefix($class);
@@ -576,13 +616,14 @@ sub write_pm {
 
     my $base   = (split '::', $module)[0];
     unless (-e "lib/$base/XSLoader.pm") {
-        $base = 'Apache';
+        $base = 'Apache2';
     }
     my $loader = join '::', $base, 'XSLoader';
 
     my $fh = $self->open_class_file($module, '.pm');
     my $noedit_warning = $self->ModPerl::Code::noedit_warning_hash();
     my $use_apr = ($module =~ /^APR::\w+$/) ? 'use APR ();' : '';
+    my $version = module_version($module);
 
     print $fh <<EOF;
 $noedit_warning
@@ -595,7 +636,7 @@ use warnings FATAL => 'all';
 $isa
 $use_apr
 use $loader ();
-our \$VERSION = '0.01';
+our \$VERSION = '$version';
 $loader\::load __PACKAGE__;
 
 $code
@@ -606,11 +647,11 @@ EOF
 }
 
 my %typemap = (
-    'Apache::RequestRec' => 'T_APACHEOBJ',
+    'Apache2::RequestRec' => 'T_APACHEOBJ',
     'apr_time_t'         => 'T_APR_TIME',
     'APR::Table'         => 'T_HASHOBJ',
     'APR::Pool'          => 'T_POOLOBJ',
-    'APR::OS::Thread'    => 'T_UVOBJ',
+    'apr_size_t *'       => 'T_UVPTR',
 );
 
 sub write_typemap {
@@ -673,7 +714,7 @@ sub write_lookup_method_file {
             $name =~ s/^DEFINE_//;
 
             if ($name =~ /^mpxs_/) {
-                #e.g. mpxs_Apache__RequestRec_
+                #e.g. mpxs_Apache2__RequestRec_
                 my $class_prefix = class_c_prefix($class);
                 if ($name =~ /$class_prefix/) {
                     $prefix = class_mpxs_prefix($class);
@@ -760,10 +801,10 @@ sub _get_objects {
 # exists, the second field is for extra comments (e.g. when there is
 # no replacement method)
 my $methods_compat = {
-    # Apache::
+    # Apache2::
     gensym            => ['Symbol::gensym',
                           'or use "open my $fh, $file"'],
-    module            => ['Apache::Module::loaded',
+    module            => ['Apache2::Module::loaded',
                           ''],
     define            => ['exists_config_define',
                           ''],
@@ -774,7 +815,7 @@ my $methods_compat = {
     can_stack_handlers=> [undef,
                           'there is no more need for that method in mp2'],
 
-    # Apache::RequestRec
+    # Apache2::RequestRec
     soft_timeout      => [undef,
                           'there is no more need for that method in mp2'],
     hard_timeout      => [undef,
@@ -797,8 +838,8 @@ my $methods_compat = {
                           ''],
     post_connection   => ['cleanup_register',
                           ''],
-    content           => [undef, # XXX: Apache::Request::what?
-                          'use CGI.pm or Apache::Request instead'],
+    content           => [undef, # XXX: Apache2::Request::what?
+                          'use CGI.pm or Apache2::Request instead'],
     clear_rgy_endav   => ['special_list_clear',
                           ''],
     stash_rgy_endav   => [undef,
@@ -843,8 +884,12 @@ my $methods_compat = {
                           ''],
     escape_uri        => ['unescape_path',
                           ''],
+    escape_url        => ['escape_path',
+                          'and requires a pool object'],
     unescape_uri      => ['unescape_url',
                           ''],
+    unescape_url_info => [undef,
+                          'use CGI::Util::unescape() instead'],
     escape_html       => [undef, # XXX: will be ap_escape_html
                           'ap_escape_html now requires a pool object'],
     parsedate         => ['parse_http',
@@ -899,7 +944,7 @@ sub print_method {
     while (@args) {
          my $method = shift @args;
          my $object = (@args && 
-             (ref($args[0]) || $args[0] =~ /^(Apache|ModPerl|APR)/))
+             (ref($args[0]) || $args[0] =~ /^(Apache2|ModPerl|APR)/))
              ? shift @args
              : undef;
          print( (lookup_method($method, $object))[0]);
@@ -976,7 +1021,7 @@ sub lookup_method {
         else {
             my %modules = map { $_->[MODULE] => 1 } @items;
             # remove dups if any (e.g. $s->add_input_filter and
-            # $r->add_input_filter are loaded by the same Apache::Filter)
+            # $r->add_input_filter are loaded by the same Apache2::Filter)
             my @modules = keys %modules;
             my $hint;
             if (@modules == 1) {
@@ -1070,12 +1115,39 @@ EOF
     close $fh;
 }
 
+sub write_module_versions_file {
+    my $self = shift;
+
+    my $file = catfile "lib", "ModPerl", "DummyVersions.pm";
+    debug "creating $file";
+    open my $fh, ">$file" or die "Can't open $file: $!";
+
+    my $noedit_warning = $self->ModPerl::Code::noedit_warning_hash();
+    print $fh "$noedit_warning\n";
+
+    my @modules = keys %{ $self->{XS} };
+    push @modules, qw(ModPerl::MethodLookup);
+
+    my $len = 0;
+    for (@modules) {
+        $len = length $_ if length $_ > $len;
+    }
+
+    require mod_perl2;
+    $len += length '$::VERSION';
+    for (@modules) {
+        my $ver = module_version($_);
+        printf $fh "package %s;\n%-${len}s = %s;\n\n",
+            $_, '$'.$_."::VERSION", $ver;
+    }
+}
+
 sub generate {
     my $self = shift;
 
     $self->prepare;
 
-    for (qw(ModPerl::WrapXS Apache APR ModPerl)) {
+    for (qw(ModPerl::WrapXS Apache2 APR ModPerl)) {
         $self->write_makefilepl($_);
     }
 
@@ -1087,8 +1159,8 @@ sub generate {
 
     $self->get_functions;
     $self->get_structures;
-    $self->write_export_file('exp') if Apache::Build::AIX;
-    $self->write_export_file('def') if Apache::Build::WIN32;
+    $self->write_export_file('exp') if Apache2::Build::AIX;
+    $self->write_export_file('def') if Apache2::Build::WIN32;
 
     while (my($module, $functions) = each %{ $self->{XS} }) {
 #        my($root, $sub) = split '::', $module;
@@ -1101,6 +1173,7 @@ sub generate {
     }
 
     $self->write_lookup_method_file;
+    $self->write_module_versions_file;
 }
 
 #three .sym files are generated:
@@ -1150,6 +1223,10 @@ sub func_is_static {
     if (my $attr = $entry->{attr}) {
         return 1 if grep { $_ eq 'static' } @$attr;
     }
+    
+    #C::Scan doesnt always pickup static __inline__
+    return 1 if $entry->{name} =~ /^mpxs_/o;
+    
     return 0;
 }
 
@@ -1183,7 +1260,11 @@ sub export_file_format_def {
 
 my $ithreads_exports = join '|', qw{
 modperl_cmd_interp_
-modperl_interp_ modperl_list_ modperl_tipool_
+modperl_interp_
+modperl_list_
+modperl_tipool_
+modperl_svptr_table_clone$
+modperl_mgv_require_module$
 };
 
 sub export_func_handle {
@@ -1204,7 +1285,7 @@ sub write_export_file {
 
     my %files = (
         modperl => $ModPerl::FunctionTable,
-        apache  => $Apache::FunctionTable,
+        apache2 => $Apache2::FunctionTable,
         apr     => $APR::FunctionTable,
     );
 
@@ -1232,9 +1313,6 @@ sub write_export_file {
             next if $self->func_is_static($entry);
             my $name = $entry->{name};
 
-            #C::Scan doesnt always pickup static __inline__
-            next if $name =~ /^mpxs_/o;
-
             my $fh = $self->export_func_handle($entry, $handles);
 
             print $fh $self->$format($name);
@@ -1266,5 +1344,43 @@ sub stats {
     return \%stats;
 }
 
+sub generate_exports {
+    my($self, $fh) = @_;
+
+    if (!$build->should_build_apache) {
+        print $fh <<"EOF";
+/* This is intentionnaly left blank, only usefull for static build */
+const void *modperl_ugly_hack = NULL;
+EOF
+        return;
+    }
+    
+    print $fh <<"EOF";
+/* 
+ * This is indeed a ugly hack!
+ * See also src/modules/perl/mod_perl.c for modperl_ugly_hack
+ * If we don't build such a list of exported API functions, the over-zealous
+ * linker can and will remove the unused functions completely. In order to
+ * avoid this, we create this object and modperl_ugly_hack to create a 
+ * dependency between all the exported API and mod_perl.c
+ */
+const void *modperl_ugly_hack = NULL;
+EOF
+
+    for my $entry (@$ModPerl::FunctionTable) {
+        next if $self->func_is_static($entry);
+        unless (Apache2::Build::PERL_HAS_ITHREADS) {
+            next if $entry->{name} =~ /^($ithreads_exports)/;
+        }
+        ( my $name ) = $entry->{name} =~ /^modperl_(.*)/;
+        print $fh <<"EOF";
+#ifndef modperl_$name
+const void *modperl_hack_$name = (const void *)modperl_$name;
+#endif
+
+EOF
+    }
+}
+
 1;
 __END__
