@@ -1,4 +1,4 @@
-/* Copyright 2000-2004 The Apache Software Foundation
+/* Copyright 2000-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -35,6 +35,22 @@ int modperl_threads_started(void)
     return MP_threads_started;
 }
 
+static int MP_threaded_mpm = 0;
+
+int modperl_threaded_mpm(void)
+{
+    return MP_threaded_mpm;
+}
+
+/* sometimes non-threaded mpm also needs to know whether it's still
+ * starting up or after post_config) */
+static int MP_post_post_config_phase = 0;
+
+int modperl_post_post_config_phase(void)
+{
+    return MP_post_post_config_phase;
+}
+
 #ifndef USE_ITHREADS
 static apr_status_t modperl_shutdown(void *data)
 {
@@ -56,7 +72,7 @@ static apr_status_t modperl_shutdown(void *data)
 #endif
 
 static const char *MP_xs_loaders[] = {
-    "Apache", "APR", NULL,
+    "Apache2", "APR", NULL,
 };
 
 #define MP_xs_loader_name "%s::XSLoader::BOOTSTRAP"
@@ -123,14 +139,26 @@ static void modperl_xs_init(pTHX)
  * the parent process will run the cleanups since server_pool is a subpool
  * of pconf.  we manually clear the server_pool to run cleanups in the
  * child processes
+ *
+ * the "server_user_pool" is a subpool of the "server_pool", this is
+ * the pool which is exposed to users, so that they can register
+ * cleanup callbacks. This is needed so that the perl cleanups won't
+ * be run before user cleanups are executed.
+ *
  */
 static apr_pool_t *server_pool = NULL;
+static apr_pool_t *server_user_pool = NULL;
 
 apr_pool_t *modperl_server_pool(void)
 {
     return server_pool;
 }
 
+apr_pool_t *modperl_server_user_pool(void)
+{
+    return server_user_pool;
+}
+
 static void set_taint_var(PerlInterpreter *perl)
 {
     dTHXa(perl);
@@ -147,12 +175,12 @@ static void set_taint_var(PerlInterpreter *perl)
 
 #ifdef MP_COMPAT_1X
     {
-        GV *gv = gv_fetchpv("Apache::__T", GV_ADDMULTI, SVt_PV);
+        GV *gv = gv_fetchpv("Apache2::__T", GV_ADDMULTI, SVt_PV);
         sv_setiv(GvSV(gv), PL_tainting);
         SvREADONLY_on(GvSV(gv));
     }
 #endif /* MP_COMPAT_1X */
-    
+
 }
 
 PerlInterpreter *modperl_startup(server_rec *s, apr_pool_t *p)
@@ -174,10 +202,10 @@ PerlInterpreter *modperl_startup(server_rec *s, apr_pool_t *p)
     if (MP_init_status != 2) {
         server_rec *base_server = modperl_global_get_server_rec();
         PerlInterpreter *base_perl;
-        
+
         MP_init_status = 2; /* calls itself, so set the flag early */
         base_perl = modperl_startup(base_server, p);
-        
+
         if (base_server == s ) {
             return base_perl;
         }
@@ -242,7 +270,7 @@ PerlInterpreter *modperl_startup(server_rec *s, apr_pool_t *p)
     /* workaround perl5.8.0/glibc bug */
     PL_reentrant_buffer->_crypt_struct.current_saltbits = 0;
 #endif
-    
+
     perl_run(perl);
 
 #ifdef USE_ITHREADS
@@ -256,7 +284,7 @@ PerlInterpreter *modperl_startup(server_rec *s, apr_pool_t *p)
     PL_endav = endav;
 
     set_taint_var(perl);
-    
+
     MP_TRACE_i(MP_FUNC, "constructed interpreter=0x%lx\n",
                (unsigned long)perl);
 
@@ -267,12 +295,28 @@ PerlInterpreter *modperl_startup(server_rec *s, apr_pool_t *p)
 #endif
 
 #ifdef MP_COMPAT_1X
-    av_push(GvAV(PL_incgv),
-            newSVpv(ap_server_root_relative(p, ""), 0));
-    av_push(GvAV(PL_incgv),
-            newSVpv(ap_server_root_relative(p, "lib/perl"), 0));
+    {
+        char *path, *path1;
+        apr_finfo_t finfo;
+        /* 1) push @INC, $ServerRoot */
+        av_push(GvAV(PL_incgv), newSVpv(ap_server_root, 0));
+
+        /* 2) push @INC, $ServerRoot/lib/perl only if it exists */
+        apr_filepath_merge(&path, ap_server_root, "lib",
+                           APR_FILEPATH_NATIVE, p);
+        apr_filepath_merge(&path1, path, "perl",
+                           APR_FILEPATH_NATIVE, p);
+        if (APR_SUCCESS == apr_stat(&finfo, path1, APR_FINFO_TYPE, p)) {
+            if (finfo.filetype == APR_DIR) {
+                av_push(GvAV(PL_incgv), newSVpv(path1, 0));
+            }
+        }
+    }
 #endif /* MP_COMPAT_1X */
-    
+
+    /* base perl and each vhost +Parent should have this init'ed */
+    modperl_handler_anon_init(aTHX_ p);
+
     if (!modperl_config_apply_PerlRequire(s, scfg, perl, p)) {
         exit(1);
     }
@@ -310,7 +354,7 @@ int modperl_init_vhost(server_rec *s, apr_pool_t *p,
 
     MP_TRACE_i(MP_FUNC, "Init vhost %s: s=0x%lx, base_s=0x%lx\n",
                vhost, s, base_server);
-    
+
     if (base_server == s) {
         MP_TRACE_i(MP_FUNC, "base server is not vhost, skipping %s\n",
                    vhost);
@@ -360,7 +404,7 @@ int modperl_init_vhost(server_rec *s, apr_pool_t *p,
             modperl_interp_init(s, p, perl);
         }   
 #endif
-        
+
         if (!modperl_config_apply_PerlRequire(s, scfg, perl, p)) {
             return HTTP_INTERNAL_SERVER_ERROR;
         }
@@ -397,7 +441,7 @@ void modperl_init(server_rec *base_server, apr_pool_t *p)
                                    base_server->server_hostname));
 
 #ifndef USE_ITHREADS
-    if (base_scfg->threaded_mpm) {
+    if (modperl_threaded_mpm()) {
         ap_log_error(APLOG_MARK, APLOG_ERR, 0, base_server,
                      "cannot use threaded MPM without ithreads enabled Perl");
         exit(1);
@@ -420,6 +464,17 @@ void modperl_init(server_rec *base_server, apr_pool_t *p)
 
 }
 
+static int modperl_post_config_require(server_rec *s, apr_pool_t *p)
+{
+    for (; s; s=s->next) {
+        MP_dSCFG(s);
+        if (!modperl_config_apply_PerlPostConfigRequire(s, scfg, p)) {
+            return FALSE;   
+        }
+    }
+    return TRUE;
+}
+
 #ifdef USE_ITHREADS
 static void modperl_init_clones(server_rec *s, apr_pool_t *p)
 {
@@ -428,7 +483,7 @@ static void modperl_init_clones(server_rec *s, apr_pool_t *p)
     char *base_name = modperl_server_desc(s, p);
 #endif /* MP_TRACE */
 
-    if (!base_scfg->threaded_mpm) {
+    if (!modperl_threaded_mpm()) {
         MP_TRACE_i(MP_FUNC, "no clones created for non-threaded mpm\n");
         return;
     }
@@ -469,16 +524,17 @@ static void modperl_init_clones(server_rec *s, apr_pool_t *p)
 
 void modperl_init_globals(server_rec *s, apr_pool_t *pconf)
 {
-    int threaded_mpm;
-    ap_mpm_query(AP_MPMQ_IS_THREADED, &threaded_mpm);
+    ap_mpm_query(AP_MPMQ_IS_THREADED, &MP_threaded_mpm);
 
     MP_TRACE_g(MP_FUNC, "mod_perl globals are configured\n");
-    
+
     modperl_global_init_pconf(pconf, pconf);
-    modperl_global_init_threaded_mpm(pconf, threaded_mpm);
     modperl_global_init_server_rec(pconf, s);
 
     modperl_tls_create_request_rec(pconf);
+
+    /* init the counter to 0 */
+    modperl_global_anon_cnt_init(pconf);
 }
 
 /*
@@ -487,9 +543,17 @@ void modperl_init_globals(server_rec *s, apr_pool_t *pconf)
  */
 static apr_status_t modperl_sys_init(void)
 {
-#if 0 /*XXX*/
-    PERL_SYS_INIT(0, NULL);
+    int argc = 0;
+    char **argv = NULL, **env = NULL;
 
+    MP_TRACE_i(MP_FUNC, "mod_perl sys init\n");
+
+    /* not every OS uses those vars in PERL_SYS_INIT3 macro */
+    argc = argc; argv = argv; env = env;
+
+    PERL_SYS_INIT3(&argc, &argv, &env);
+
+#if 0 /*XXX*/
 #ifdef PTHREAD_ATFORK
     if (!ap_exists_config_define("PERL_PTHREAD_ATFORK_DONE")) {
         PTHREAD_ATFORK(Perl_atfork_lock,
@@ -513,14 +577,19 @@ static apr_status_t modperl_sys_init(void)
 static apr_status_t modperl_sys_term(void *data)
 {
     MP_init_status = 0;
+    MP_threads_started = 0;
+    MP_post_post_config_phase = 0;
+
+    MP_PERL_FREE_THREAD_KEY_WORKAROUND;
+
+    MP_TRACE_i(MP_FUNC, "mod_perl sys term\n");
 
     modperl_env_unload();
 
     modperl_perl_pp_unset_all();
 
-#if 0 /*XXX*/
     PERL_SYS_TERM();
-#endif
+
     return APR_SUCCESS;
 }
 
@@ -531,12 +600,20 @@ int modperl_hook_init(apr_pool_t *pconf, apr_pool_t *plog,
         return OK;
     }
 
+    MP_TRACE_i(MP_FUNC, "mod_perl hook init\n");
+
     MP_init_status = 1; /* now starting */
 
+    modperl_restart_count_inc(s);
+
     apr_pool_create(&server_pool, pconf);
+    apr_pool_tag(server_pool, "mod_perl server pool");
+
+    apr_pool_create(&server_user_pool, pconf);
+    apr_pool_tag(server_user_pool, "mod_perl server user pool");
 
     modperl_sys_init();
-    apr_pool_cleanup_register(pconf, NULL,
+    apr_pool_cleanup_register(server_pool, NULL,
                               modperl_sys_term, apr_pool_cleanup_null);
 
     modperl_init(s, pconf);
@@ -579,14 +656,26 @@ static int modperl_hook_pre_connection(conn_rec *c, void *csd)
     return OK;
 }
 
-static int modperl_hook_post_config(apr_pool_t *pconf, apr_pool_t *plog,
-                                    apr_pool_t *ptemp, server_rec *s)
+static int modperl_hook_post_config_last(apr_pool_t *pconf, apr_pool_t *plog,
+                                         apr_pool_t *ptemp, server_rec *s)
 {
+    /* in the threaded environment, no server_rec/process_rec
+     * modifications should be done beyond this point */
 #ifdef USE_ITHREADS
     MP_dSCFG(s);
     dTHXa(scfg->mip->parent->perl);
 #endif
 
+    if (!modperl_post_config_require(s, pconf)) {
+        exit(1);
+    }
+
+    if (modperl_threaded_mpm()) {
+        MP_threads_started = 1;
+    }
+
+    MP_post_post_config_phase = 1;
+
 #ifdef MP_TRACE
     /* httpd core open_logs handler re-opens s->error_log, which might
      * change, even though it still points to the same physical file
@@ -605,7 +694,7 @@ static int modperl_hook_post_config(apr_pool_t *pconf, apr_pool_t *plog,
         modperl_trace_logfile_set(dup);
     }
 #endif
-    
+
     ap_add_version_component(pconf, MP_VERSION_STRING);
     ap_add_version_component(pconf,
                              Perl_form(aTHX_ "Perl/v%vd", PL_patchlevel));
@@ -620,21 +709,7 @@ static int modperl_hook_post_config(apr_pool_t *pconf, apr_pool_t *plog,
     ap_log_error(APLOG_MARK, APLOG_INFO, 0, s,
                  "mod_perl: using Perl HASH_SEED: %"UVuf, MP_init_hash_seed);
 #endif
-    
-    return OK;
-}
 
-static int modperl_hook_post_config_last(apr_pool_t *pconf, apr_pool_t *plog,
-                                         apr_pool_t *ptemp, server_rec *s)
-{
-    MP_dSCFG(s);
-
-    /* in the threaded environment, no server_rec/process_rec
-     * modifications should be done beyond this point */
-    if (scfg->threaded_mpm) {
-        MP_threads_started = 1;
-    }
-    
     return OK;
 }
 
@@ -650,7 +725,7 @@ static int modperl_hook_create_request(request_rec *r)
      * wasn't turned off and MpDirPARSE_HEADERS is on
      */
     MpReqPARSE_HEADERS_On(rcfg);
-    
+
     return OK;
 }
 
@@ -742,7 +817,7 @@ void modperl_register_hooks(apr_pool_t *p)
     APR_REGISTER_OPTIONAL_FN(modperl_interp_unselect);
 #endif
 
-    /* for <IfDefine MODPERL2> and Apache->define("MODPERL2") */
+    /* for <IfDefine MODPERL2> and Apache2->define("MODPERL2") */
     *(char **)apr_array_push(ap_server_config_defines) =
         apr_pstrdup(p, "MODPERL2");
 
@@ -752,9 +827,6 @@ void modperl_register_hooks(apr_pool_t *p)
     ap_hook_open_logs(modperl_hook_init,
                       NULL, NULL, APR_HOOK_FIRST);
 
-    ap_hook_post_config(modperl_hook_post_config,
-                        NULL, NULL, APR_HOOK_FIRST);
-
     ap_hook_post_config(modperl_hook_post_config_last,
                         NULL, NULL, APR_HOOK_REALLY_LAST);
 
@@ -812,6 +884,8 @@ static const command_rec modperl_cmds[] = {
     MP_CMD_SRV_ITERATE("PerlSwitches", switches, "Perl Switches"),
     MP_CMD_DIR_ITERATE("PerlModule", modules, "PerlModule"),
     MP_CMD_DIR_ITERATE("PerlRequire", requires, "PerlRequire"),
+    MP_CMD_SRV_ITERATE("PerlConfigRequire", config_requires, "PerlConfigRequire"),
+    MP_CMD_SRV_ITERATE("PerlPostConfigRequire", post_config_requires, "PerlPostConfigRequire"),
     MP_CMD_DIR_ITERATE("PerlOptions", options, "Perl Options"),
     MP_CMD_DIR_ITERATE("PerlInitHandler", init_handlers, "Subroutine name"),
     MP_CMD_DIR_TAKE2("PerlSetVar", set_var, "PerlSetVar"),
@@ -825,7 +899,7 @@ static const command_rec modperl_cmds[] = {
                      "filter[;filter]"),
     MP_CMD_DIR_TAKE1("PerlSetOutputFilter", set_output_filter,
                      "filter[;filter]"),
-    
+
     MP_CMD_DIR_RAW_ARGS_ON_READ("=pod", pod, "Start of POD"),
     MP_CMD_DIR_RAW_ARGS_ON_READ("=back", pod, "End of =over"),
     MP_CMD_DIR_RAW_ARGS_ON_READ("=cut", pod_cut, "End of POD"),
@@ -1031,6 +1105,17 @@ int modperl_response_handler_cgi(request_rec *r)
     return retval;
 }
 
+/* This ugly hack pulls in any function listed in
+ * modperl_exports.c. Otherwise, the over-zealous
+ * linker would remove unused api functions
+ */
+const void *modperl_suck_in_ugly_hack(void);
+const void *modperl_suck_in_ugly_hack(void)
+{
+    extern const void *modperl_ugly_hack;
+    return modperl_ugly_hack;
+}
+
 module AP_MODULE_DECLARE_DATA perl_module = {
     STANDARD20_MODULE_STUFF, 
     modperl_config_dir_create, /* dir config creater */
