@@ -1,4 +1,4 @@
-/* Copyright 2000-2004 The Apache Software Foundation
+/* Copyright 2000-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -95,13 +95,26 @@ extern module AP_MODULE_DECLARE_DATA perl_module;
 
 int modperl_threads_started(void);
 int modperl_threaded_mpm(void);
+int modperl_post_post_config_phase(void);
 
 #define MP_CROAK_IF_THREADS_STARTED(what)                       \
     if (modperl_threads_started()) {                            \
         Perl_croak(aTHX_ "Can't run '%s' in the threaded "      \
                    "environment after server startup", what);   \
     }
-    
+  
+#define MP_CROAK_IF_THREADED_MPM(what)                          \
+    if (modperl_threaded_mpm()) {                               \
+        Perl_croak(aTHX_ "Can't run '%s' in a threaded mpm",    \
+                   what);                                       \
+    }
+
+#define MP_CROAK_IF_POST_POST_CONFIG_PHASE(what)                \
+    if (modperl_post_post_config_phase()) {                     \
+        Perl_croak(aTHX_ "Can't run '%s' after server startup", \
+                   what);                                       \
+    }
+
 int modperl_init_vhost(server_rec *s, apr_pool_t *p,
                        server_rec *base_server);
 void modperl_init(server_rec *s, apr_pool_t *p);
@@ -114,6 +127,7 @@ int modperl_hook_pre_config(apr_pool_t *p, apr_pool_t *plog,
                             apr_pool_t *ptemp);
 void modperl_register_hooks(apr_pool_t *p);
 apr_pool_t *modperl_server_pool(void);
+apr_pool_t *modperl_server_user_pool(void);
 PerlInterpreter *modperl_startup(server_rec *s, apr_pool_t *p);
 int modperl_perl_destruct_level(void);
 void xs_init(pTHX);
@@ -140,4 +154,48 @@ typedef void MP_FUNC_NONSTD_T(modperl_var_modify_t) (apr_table_t *,
 
 APR_DECLARE_OPTIONAL_FN(apr_status_t,modperl_interp_unselect,(void *));
 
+/*
+ * perl context overriding and restoration is required when
+ * PerlOptions +Parent/+Clone is used in vhosts, and perl is used to
+ * at the server startup. So that <Perl> sections, PerlLoadModule,
+ * PerlModule and PerlRequire are all run using the right perl context
+ * and restore to the original context when they are done.
+ *
+ * As of perl-5.8.3 it's unfortunate that it uses PERL_GET_CONTEXT and
+ * doesn't rely on the passed pTHX internally. When and if perl is
+ * fixed to always use pTHX if available, this context switching mess
+ * can be removed.
+ */
+#ifdef USE_ITHREADS
+
+#define MP_PERL_CONTEXT_DECLARE                 \
+    PerlInterpreter *orig_perl = NULL;          \
+    pTHX;
+
+#define MP_PERL_CONTEXT_STORE                   \
+    orig_perl = PERL_GET_CONTEXT;
+
+#define MP_PERL_CONTEXT_OVERRIDE(new_perl)      \
+    aTHX = new_perl;                            \
+    PERL_SET_CONTEXT(aTHX);
+
+#define MP_PERL_CONTEXT_STORE_OVERRIDE(new_perl)        \
+    MP_PERL_CONTEXT_STORE;                              \
+    MP_PERL_CONTEXT_OVERRIDE(new_perl)
+
+#define MP_PERL_CONTEXT_RESTORE                 \
+    if (orig_perl) {                            \
+        PERL_SET_CONTEXT(orig_perl);            \
+    }
+
+#else /* #ifdef USE_ITHREADS */
+
+#define MP_PERL_CONTEXT_DECLARE
+#define MP_PERL_CONTEXT_STORE
+#define MP_PERL_CONTEXT_OVERRIDE(new_perl)
+#define MP_PERL_CONTEXT_STORE_OVERRIDE(new_perl)
+#define MP_PERL_CONTEXT_RESTORE
+
+#endif /* end of #ifdef USE_ITHREADS */
+
 #endif /*  MOD_PERL_H */
