@@ -1,4 +1,4 @@
-/* Copyright 2001-2004 The Apache Software Foundation
+/* Copyright 2001-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,7 +16,7 @@
 #ifndef MODPERL_BUCKET_H
 #define MODPERL_BUCKET_H
 
-apr_bucket *modperl_bucket_sv_create(pTHX_ SV *sv, apr_off_t offset,
-                                     apr_size_t len);
+apr_bucket *modperl_bucket_sv_create(pTHX_ apr_bucket_alloc_t *list, SV *sv, 
+                                     apr_off_t offset, apr_size_t len);
 
 #endif /* MODPERL_BUCKET_H */
