@@ -1,4 +1,4 @@
-/* Copyright 2001-2004 The Apache Software Foundation
+/* Copyright 2001-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,42 +15,6 @@
 
 #include "mod_perl.h"
 
-#ifdef USE_ITHREADS
-
-/*
- * perl context overriding and restoration is required when
- * PerlOptions +Parent/+Clone is used in vhosts, and perl is used to
- * at the server startup. So that <Perl> sections, PerlLoadModule,
- * PerlModule and PerlRequire are all run using the right perl context
- * and restore to the original context when they are done.
- *
- * As of perl-5.8.3 it's unfortunate that it uses PERL_GET_CONTEXT and
- * doesn't rely on the passed pTHX internally. When and if perl is
- * fixed to always use pTHX if available, this context switching mess
- * can be removed.
- */
-
-#define MP_PERL_DECLARE_CONTEXT \
-    PerlInterpreter *orig_perl; \
-    pTHX;
-
-/* XXX: .htaccess support cannot use this perl with threaded MPMs */
-#define MP_PERL_OVERRIDE_CONTEXT    \
-    orig_perl = PERL_GET_CONTEXT;   \
-    aTHX = scfg->mip->parent->perl; \
-    PERL_SET_CONTEXT(aTHX);
-
-#define MP_PERL_RESTORE_CONTEXT     \
-    PERL_SET_CONTEXT(orig_perl);
-
-#else
-
-#define MP_PERL_DECLARE_CONTEXT
-#define MP_PERL_OVERRIDE_CONTEXT
-#define MP_PERL_RESTORE_CONTEXT
-
-#endif
-
 /* This ensures that a given directive is either in Server context
  * or in a .htaccess file, usefull for things like PerlRequire
  */
@@ -116,7 +80,7 @@ char *modperl_cmd_push_filter_handlers(MpAV **handlers,
                    "the filter attributes available)\n",
                    modperl_pid_tid(p), h->name);
     }
-    
+
     if (!*handlers) {
         *handlers = modperl_handler_array_new(p);
         MP_TRACE_d(MP_FUNC, "created handler stack\n");
@@ -139,7 +103,7 @@ static char *modperl_cmd_push_httpd_filter_handlers(MpAV **handlers,
      * filters */
     MpHandlerFAKE_On(h);
     h->attrs = MP_FILTER_HTTPD_HANDLER;
-        
+
     if (!*handlers) {
         *handlers = modperl_handler_array_new(p);
         MP_TRACE_d(MP_FUNC, "created handler stack\n");
@@ -169,30 +133,26 @@ MP_CMD_SRV_DECLARE(trace)
     return NULL;
 }
 
+/* this test shows whether the perl for the current s is running
+ * (either base or vhost) */
 static int modperl_vhost_is_running(server_rec *s)
 {
 #ifdef USE_ITHREADS
-    MP_dSCFG(s);
-    int is_vhost = (s != modperl_global_get_server_rec());
-
-    if (is_vhost && scfg->mip) {
-        return TRUE;
+    if (s->is_virtual){
+        MP_dSCFG(s);
+        return scfg->mip ? TRUE : FALSE;
     }
-    else {
-        return FALSE;
-    }
-#else
-    return modperl_is_running();
 #endif
+
+    return modperl_is_running();
+
 }
 
 MP_CMD_SRV_DECLARE(switches)
 {
     server_rec *s = parms->server;
     MP_dSCFG(s);
-    if (s->is_virtual
-        ? modperl_vhost_is_running(s)
-        : modperl_is_running() ) {
+    if (modperl_vhost_is_running(s)) {
         return modperl_cmd_too_late(parms);
     }
     MP_TRACE_d(MP_FUNC, "arg = %s\n", arg);
@@ -203,10 +163,11 @@ MP_CMD_SRV_DECLARE(switches)
 MP_CMD_SRV_DECLARE(modules)
 {
     MP_dSCFG(parms->server);
-    MP_PERL_DECLARE_CONTEXT;
+    modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
+    MP_PERL_CONTEXT_DECLARE;
 
     MP_CHECK_SERVER_OR_HTACCESS_CONTEXT;
-    
+
     if (modperl_is_running() &&
         modperl_init_vhost(parms->server, parms->pool, NULL) != OK)
     {
@@ -217,12 +178,16 @@ MP_CMD_SRV_DECLARE(modules)
         char *error = NULL;
 
         MP_TRACE_d(MP_FUNC, "load PerlModule %s\n", arg);
-        
-        MP_PERL_OVERRIDE_CONTEXT;
+
+        MP_PERL_CONTEXT_STORE_OVERRIDE(scfg->mip->parent->perl);
         if (!modperl_require_module(aTHX_ arg, FALSE)) {
             error = SvPVX(ERRSV);
         }
-        MP_PERL_RESTORE_CONTEXT;
+        else {
+            modperl_env_sync_srv_env_hash2table(aTHX_ parms->pool, scfg);
+            modperl_env_sync_dir_env_hash2table(aTHX_ parms->pool, dcfg);
+        }
+        MP_PERL_CONTEXT_RESTORE;
 
         return error;
     }
@@ -236,10 +201,11 @@ MP_CMD_SRV_DECLARE(modules)
 MP_CMD_SRV_DECLARE(requires)
 {
     MP_dSCFG(parms->server);
-    MP_PERL_DECLARE_CONTEXT;
+    modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
+    MP_PERL_CONTEXT_DECLARE;
 
     MP_CHECK_SERVER_OR_HTACCESS_CONTEXT;
-    
+
     if (modperl_is_running() &&
         modperl_init_vhost(parms->server, parms->pool, NULL) != OK)
     {
@@ -251,11 +217,15 @@ MP_CMD_SRV_DECLARE(requires)
 
         MP_TRACE_d(MP_FUNC, "load PerlRequire %s\n", arg);
 
-        MP_PERL_OVERRIDE_CONTEXT;
+        MP_PERL_CONTEXT_STORE_OVERRIDE(scfg->mip->parent->perl);
         if (!modperl_require_file(aTHX_ arg, FALSE)) {
             error = SvPVX(ERRSV);
         }
-        MP_PERL_RESTORE_CONTEXT;
+        else {
+            modperl_env_sync_srv_env_hash2table(aTHX_ parms->pool, scfg);
+            modperl_env_sync_dir_env_hash2table(aTHX_ parms->pool, dcfg);
+        }
+        MP_PERL_CONTEXT_RESTORE;
 
         return error;
     }
@@ -266,71 +236,90 @@ MP_CMD_SRV_DECLARE(requires)
     }
 }
 
-static MP_CMD_SRV_DECLARE2(handle_vars)
-{
-    MP_dSCFG(parms->server);
-    modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
-    const char *name = parms->cmd->name;
+MP_CMD_SRV_DECLARE(config_requires)
+{    
+    /* we must init earlier than normal */
+    modperl_run();
 
-    /* PerlSetVar and PerlAddVar logic.  here's the deal...
-     *
-     * cfg->configvars holds the final PerlSetVar/PerlAddVar configuration
-     * for a given server or directory.  however, getting to that point
-     * is kind of tricky, due to the add-style nature of PerlAddVar.
-     *
-     * the solution is to use cfg->setvars to hold PerlSetVar entries
-     * and cfg->addvars to hold PerlAddVar entries, each serving as a
-     * placeholder for when we need to know what's what in the merge routines.
-     *
-     * however, for the initial pass, apr_table_setn and apr_table_addn
-     * will properly build the configvars table, which will be visible to
-     * startup scripts trying to access per-server configurations.
-     *
-     * the end result is that we need to populate all three tables in order
-     * to keep things straight later on see merge_table_config_vars in
-     * modperl_config.c
+    /* PerlConfigFile is only different from PerlRequires by forcing
+     * an immediate init.
      */
-    modperl_table_modify_t func =
-        strEQ(name, "PerlSetVar") ? apr_table_setn : apr_table_addn;
+    return modperl_cmd_requires(parms, mconfig, arg);
+}
 
-    apr_table_t *table =
-        strEQ(name, "PerlSetVar") ? dcfg->setvars : dcfg->addvars;
+MP_CMD_SRV_DECLARE(post_config_requires)
+{
+    apr_pool_t *p = parms->temp_pool;
+    modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
+    MP_dSCFG(parms->server);
 
-    func(table, arg1, arg2);
-    func(dcfg->configvars, arg1, arg2);
+    modperl_require_file_t *require = apr_pcalloc(p, sizeof(*require));
+    MP_TRACE_d(MP_FUNC, "push PerlPostConfigRequire for %s\n", arg);
+    require->file = arg;
+    require->dcfg = dcfg;
+
+    *(modperl_require_file_t **)
+        apr_array_push(scfg->PerlPostConfigRequire) = require;
+
+    return NULL;
+}
+
+static void modperl_cmd_addvar_func(apr_table_t *configvars,
+                                    apr_table_t *setvars,
+                                    const char *key, const char *val)
+{
+    apr_table_addn(configvars, key, val);
+}
+
+/*  Conceptually, setvar is { unsetvar; addvar; } */
+
+static void modperl_cmd_setvar_func(apr_table_t *configvars,
+                                    apr_table_t *setvars,
+                                    const char * key, const char *val)
+{
+    apr_table_setn(setvars, key, val);
+    apr_table_setn(configvars, key, val);
+}
+
+static const char *modperl_cmd_modvar(modperl_var_modify_t varfunc,
+                                      cmd_parms *parms,
+                                      modperl_config_dir_t *dcfg,
+                                      const char *arg1, const char *arg2)
+{
+    varfunc(dcfg->configvars, dcfg->setvars, arg1, arg2);
 
     MP_TRACE_d(MP_FUNC, "%s DIR: arg1 = %s, arg2 = %s\n",
-               name, arg1, arg2);
+               parms->cmd->name, arg1, arg2);
 
-    /* make available via Apache->server->dir_config */
+    /* make available via Apache2->server->dir_config */
     if (!parms->path) {
-        table = strEQ(name, "PerlSetVar") ? scfg->setvars : scfg->addvars;
-
-        func(table, arg1, arg2);
-        func(scfg->configvars, arg1, arg2);
+        MP_dSCFG(parms->server);
+        varfunc(scfg->configvars, scfg->setvars, arg1, arg2);
 
         MP_TRACE_d(MP_FUNC, "%s SRV: arg1 = %s, arg2 = %s\n",
-                   name, arg1, arg2);
+                   parms->cmd->name, arg1, arg2);
     }
 
     return NULL;
 }
 
-MP_CMD_SRV_DECLARE2(set_var)
-{
-    return modperl_cmd_handle_vars(parms, mconfig, arg1, arg2);
-}
-
 MP_CMD_SRV_DECLARE2(add_var)
 {
-    return modperl_cmd_handle_vars(parms, mconfig, arg1, arg2);
+    modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
+    return modperl_cmd_modvar(modperl_cmd_addvar_func, parms, dcfg, arg1, arg2);
+}
+
+MP_CMD_SRV_DECLARE2(set_var)
+{
+    modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
+    return modperl_cmd_modvar(modperl_cmd_setvar_func, parms, dcfg, arg1, arg2);
 }
 
 MP_CMD_SRV_DECLARE2(set_env)
 {
     MP_dSCFG(parms->server);
     modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
- 
+
 #ifdef ENV_IS_CASELESS /* i.e. WIN32 */
     /* we turn off env magic during hv_store later, so do this now,
      * else lookups on keys with lowercase characters will fails
@@ -344,6 +333,13 @@ MP_CMD_SRV_DECLARE2(set_env)
     if (!parms->path) {
         /* will be propagated to environ */
         apr_table_setn(scfg->SetEnv, arg1, arg2);
+        /* sync SetEnv => %ENV only for the top-level values */
+        if (modperl_vhost_is_running(parms->server)) {
+            MP_PERL_CONTEXT_DECLARE;
+            MP_PERL_CONTEXT_STORE_OVERRIDE(scfg->mip->parent->perl);
+            modperl_env_hv_store(aTHX_ arg1, arg2);
+            MP_PERL_CONTEXT_RESTORE;
+        }
     }
 
     apr_table_setn(dcfg->SetEnv, arg1, arg2);
@@ -355,7 +351,7 @@ MP_CMD_SRV_DECLARE(pass_env)
 {
     MP_dSCFG(parms->server);
     char *val = getenv(arg);
-    
+
 #ifdef ENV_IS_CASELESS /* i.e. WIN32 */
     /* we turn off env magic during hv_store later, so do this now,
      * else lookups on keys with lowercase characters will fails
@@ -363,9 +359,15 @@ MP_CMD_SRV_DECLARE(pass_env)
      */
     modperl_str_toupper((char *)arg);
 #endif
-    
+
     if (val) {
         apr_table_setn(scfg->PassEnv, arg, apr_pstrdup(parms->pool, val));
+        if (modperl_vhost_is_running(parms->server)) {
+            MP_PERL_CONTEXT_DECLARE;
+            MP_PERL_CONTEXT_STORE_OVERRIDE(scfg->mip->parent->perl);
+            modperl_env_hv_store(aTHX_ arg, val);
+            MP_PERL_CONTEXT_RESTORE;
+        }
         MP_TRACE_d(MP_FUNC, "arg = %s, val = %s\n", arg, val);
     }
     else {
@@ -381,7 +383,7 @@ MP_CMD_SRV_DECLARE(options)
     modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
     int is_per_dir = parms->path ? 1 : 0;
     modperl_options_t *opts = is_per_dir ? dcfg->flags : scfg->flags;
-    apr_pool_t *p = parms->pool;
+    apr_pool_t *p = parms->temp_pool;
     const char *error;
 
     MP_TRACE_d(MP_FUNC, "arg = %s\n", arg);
@@ -445,11 +447,11 @@ MP_CMD_SRV_DECLARE(perl)
     if (!endp) {
         return modperl_cmd_unclosed_directive(parms);
     }
-    
+
     MP_CHECK_SERVER_OR_HTACCESS_CONTEXT;
 
     arg = apr_pstrndup(p, arg, endp - arg);
-   
+
     if ((errmsg = modperl_cmd_parse_args(p, arg, &args))) {
         return errmsg;
     }
@@ -460,7 +462,7 @@ MP_CMD_SRV_DECLARE(perl)
         if (strEQ(line, "</Perl>")) {
             break;
         }
-        
+
         /*XXX: Less than optimal */
         code = apr_pstrcat(p, code, line, "\n", NULL);
     }
@@ -469,7 +471,7 @@ MP_CMD_SRV_DECLARE(perl)
     if (!*current) {
         *current = apr_pcalloc(p, sizeof(**current));
     }
-    
+
     (*current)->filename = parms->config_file->name;
     (*current)->line_num = line_num;
     (*current)->directive = apr_pstrdup(p, "Perl");
@@ -479,38 +481,37 @@ MP_CMD_SRV_DECLARE(perl)
     return NULL;
 }
 
-#define MP_DEFAULT_PERLSECTION_HANDLER "Apache::PerlSections"
-#define MP_DEFAULT_PERLSECTION_PACKAGE "Apache::ReadConfig"
+#define MP_DEFAULT_PERLSECTION_HANDLER "Apache2::PerlSections"
+#define MP_DEFAULT_PERLSECTION_PACKAGE "Apache2::ReadConfig"
 #define MP_PERLSECTIONS_SAVECONFIG_SV \
-    get_sv("Apache::Server::SaveConfig", FALSE)
+    get_sv("Apache2::PerlSections::Save", FALSE)
 
 MP_CMD_SRV_DECLARE(perldo)
 {
     apr_pool_t *p = parms->pool;
     server_rec *s = parms->server;
+    modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
     apr_table_t *options;
     modperl_handler_t *handler = NULL;
     const char *pkg_name = NULL;
     ap_directive_t *directive = parms->directive;
-#ifdef USE_ITHREADS
     MP_dSCFG(s);
-    MP_PERL_DECLARE_CONTEXT;
-#endif
+    MP_PERL_CONTEXT_DECLARE;
 
     if (!(arg && *arg)) {
         return NULL;
     }
 
     MP_CHECK_SERVER_OR_HTACCESS_CONTEXT;
-    
+
     /* we must init earlier than normal */
     modperl_run();
 
     if (modperl_init_vhost(s, p, NULL) != OK) {
         return "init mod_perl vhost failed";
     }
-    
-    MP_PERL_OVERRIDE_CONTEXT;
+
+    MP_PERL_CONTEXT_STORE_OVERRIDE(scfg->mip->parent->perl);
 
     /* data will be set by a <Perl> section */
     if ((options = directive->data)) {
@@ -523,13 +524,13 @@ MP_CMD_SRV_DECLARE(perldo)
             handler_name = apr_pstrdup(p, MP_DEFAULT_PERLSECTION_HANDLER);
             apr_table_set(options, "handler", handler_name);
         }
-        
+
         handler = modperl_handler_new(p, handler_name);
-            
+
         if (!(pkg_base = apr_table_get(options, "package"))) {
             pkg_base = apr_pstrdup(p, MP_DEFAULT_PERLSECTION_PACKAGE);
         }
-       
+
         pkg_namespace = modperl_file2package(p, directive->filename);
 
         pkg_name = apr_psprintf(p, "%s::%s::line_%d", 
@@ -547,28 +548,32 @@ MP_CMD_SRV_DECLARE(perldo)
         arg = apr_pstrcat(p, "package ", pkg_name, ";", line_header,
                           arg, NULL);
     }
-    
+
     {
+        SV *code = newSVpv(arg, 0);
         GV *gv = gv_fetchpv("0", TRUE, SVt_PV);
-        ENTER;
+        ENTER;SAVETMPS;
         save_scalar(gv); /* local $0 */
         sv_setpv_mg(GvSV(gv), directive->filename);
-        eval_pv(arg, FALSE);
-        LEAVE;
+        eval_sv(code, G_SCALAR|G_KEEPERR);
+        SvREFCNT_dec(code);
+        modperl_env_sync_srv_env_hash2table(aTHX_ p, scfg);
+        modperl_env_sync_dir_env_hash2table(aTHX_ p, dcfg);
+        FREETMPS;LEAVE;
     }
-    
+
     if (SvTRUE(ERRSV)) {
-        MP_PERL_RESTORE_CONTEXT;
+        MP_PERL_CONTEXT_RESTORE;
         return SvPVX(ERRSV);
     }
-    
+
     if (handler) {
         int status;
         SV *saveconfig = MP_PERLSECTIONS_SAVECONFIG_SV;
         AV *args = Nullav;
-        
+
         modperl_handler_make_args(aTHX_ &args,
-                                  "Apache::CmdParms", parms,
+                                  "Apache2::CmdParms", parms,
                                   "APR::Table", options,
                                   NULL);
 
@@ -577,22 +582,19 @@ MP_CMD_SRV_DECLARE(perldo)
         SvREFCNT_dec((SV*)args);
 
         if (!(saveconfig && SvTRUE(saveconfig))) {
-            HV *symtab = (HV*)gv_stashpv(pkg_name, FALSE);
-            if (symtab) {
-                modperl_clear_symtab(aTHX_ symtab);
-            }
+            modperl_package_unload(aTHX_ pkg_name);
         }
-        
+
         if (status != OK) {
             char *error = SvTRUE(ERRSV) ? SvPVX(ERRSV) :
                 apr_psprintf(p, "<Perl> handler %s failed with status=%d",
                              handler->name, status);
-            MP_PERL_RESTORE_CONTEXT;
+            MP_PERL_CONTEXT_RESTORE;
             return error;
         }
     }
 
-    MP_PERL_RESTORE_CONTEXT;
+    MP_PERL_CONTEXT_RESTORE;
     return NULL;
 }
 
@@ -642,8 +644,6 @@ MP_CMD_SRV_DECLARE(END)
  */
 MP_CMD_SRV_DECLARE(load_module)
 {
-    apr_pool_t *p = parms->pool;
-    server_rec *s = parms->server;
     const char *errmsg;
 
     MP_TRACE_d(MP_FUNC, "PerlLoadModule %s\n", arg);
@@ -655,7 +655,7 @@ MP_CMD_SRV_DECLARE(load_module)
         return errmsg;
     }
 
-    return modperl_module_add(p, s, arg);
+    return NULL;
 }
 
 /* propogate filters insertion ala SetInputFilter */
@@ -664,7 +664,7 @@ MP_CMD_SRV_DECLARE(set_input_filter)
     MP_dSCFG(parms->server);
     modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
     char *filter;
-    
+
     if (!MpSrvENABLE(scfg)) {
         return apr_pstrcat(parms->pool,
                            "Perl is disabled for server ",
@@ -691,7 +691,7 @@ MP_CMD_SRV_DECLARE(set_output_filter)
     MP_dSCFG(parms->server);
     modperl_config_dir_t *dcfg = (modperl_config_dir_t *)mconfig;
     char *filter;
-    
+
     if (!MpSrvENABLE(scfg)) {
         return apr_pstrcat(parms->pool,
                            "Perl is disabled for server ",
@@ -756,7 +756,7 @@ MP_CMD_SRV_DECLARE_FLAG(setup_env)
 
 #define MP_INTERP_SCOPE_DIR_USAGE \
     MP_INTERP_SCOPE_USAGE MP_INTERP_SCOPE_DIR_OPTS
- 
+
 #define MP_INTERP_SCOPE_SRV_OPTS \
     "connection, " MP_INTERP_SCOPE_DIR_OPTS
 
