@@ -1,5 +1,5 @@
 
-/* Copyright 2000-2004 The Apache Software Foundation
+/* Copyright 2000-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,6 +25,12 @@
 #define MP_INLINE APR_INLINE
 #endif
 
+#ifdef CYGWIN
+#define MP_STATIC
+#else
+#define MP_STATIC static
+#endif
+
 #ifdef WIN32
 #   define MP_FUNC_T(name)          (_stdcall *name)
 #   define MP_FUNC_NONSTD_T(name)   (*name)
