@@ -1,4 +1,4 @@
-/* Copyright 2000-2004 The Apache Software Foundation
+/* Copyright 2000-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,6 +20,8 @@ void *modperl_config_dir_create(apr_pool_t *p, char *dir)
     modperl_config_dir_t *dcfg = modperl_config_dir_new(p);
 
     dcfg->location = dir;
+
+    MP_TRACE_d(MP_FUNC, "dir %s\n", dir);
     
 #ifdef USE_ITHREADS
     /* defaults to per-server scope */
@@ -43,6 +45,11 @@ static apr_table_t *modperl_table_overlap(apr_pool_t *p,
      * note that this is equivalent to apr_table_overlap except a new
      * table is generated, which is required (otherwise we would clobber
      * the existing parent or child configurations)
+     *
+     * note that this is *not* equivalent to apr_table_overlap, although
+     * I think it should be, because apr_table_overlap seems to clear
+     * its first argument when the tables have different pools. I think
+     * this is wrong -- rici
      */
     apr_table_t *merge = apr_table_overlay(p, base, add);
 
@@ -59,54 +66,29 @@ static apr_table_t *modperl_table_overlap(apr_pool_t *p,
 #define merge_table_overlap_item(item) \
     mrg->item = modperl_table_overlap(p, base->item, add->item)
 
-static apr_table_t *merge_table_config_vars(apr_pool_t *p,
-                                            apr_table_t *configvars,
-                                            apr_table_t *set,
-                                            apr_table_t *add)
+static apr_table_t *merge_config_add_vars(apr_pool_t *p,
+                                          const apr_table_t *base,
+                                          const apr_table_t *unset,
+                                          const apr_table_t *add)
 {
-    apr_table_t *base = apr_table_copy(p, configvars);
-    apr_table_t *merged_config_vars;
+    apr_table_t *temp = apr_table_copy(p, base);
 
     const apr_array_header_t *arr;
     apr_table_entry_t *entries;
     int i;
 
-    /* configvars already contains a properly merged PerlSetVar/PerlAddVar
-     * configuration for the base (parent), so all we need to do is merge
-     * the add (child) configuration into it properly.
-     *
-     * any PerlSetVar settings in the add (child) config need to reset
-     * existing entries in the base (parent) config, or generate a
-     * new entry where none existed previously.  PerlAddVar settings
-     * are merged into that.
-     *
-     * unfortunately, there is no set of apr functions to do this for us - 
-     * apr_compress_table would be ok, except it always merges mulit-valued
-     * keys into one, regardless of the APR_OVERLAP_TABLES flag.  that is,
-     * regardless of whether newer entries are set or merged into existing
-     * entries, the entire table is _always_ compressed.  this is no good -
-     * we need separate entries for existing keys, not a single (compressed)
-     * entry.
-     *
-     * fortunately, the logic here is simple.  first, (re)set the base (parent)
-     * table where a PerlSetVar entry exists in the add (child) configuration.
-     * then, just overlay the PerlAddVar configuration into it.
-     */
-
-    arr = apr_table_elts(set);
+    /* for each key in unset do apr_table_unset(temp, key); */
+    arr = apr_table_elts(unset);
     entries  = (apr_table_entry_t *)arr->elts;
 
     /* hopefully this is faster than using apr_table_do  */
     for (i = 0; i < arr->nelts; i++) {
-        apr_table_setn(base, entries[i].key, entries[i].val);
+        if (entries[i].key) {
+            apr_table_unset(temp, entries[i].key);
+        }
     }
 
-    /* at this point, all the PerlSetVar merging has happened.  add in the 
-     * add (child) PerlAddVar entries and we're done
-     */
-    merged_config_vars = apr_table_overlay(p, base, add);
-      
-    return merged_config_vars;
+    return apr_table_overlay(p, temp, add);
 }
 
 #define merge_handlers(merge_flag, array) \
@@ -127,8 +109,9 @@ void *modperl_config_dir_merge(apr_pool_t *p, void *basev, void *addv)
         *add  = (modperl_config_dir_t *)addv,
         *mrg  = modperl_config_dir_new(p);
 
-    MP_TRACE_d(MP_FUNC, "basev==0x%lx, addv==0x%lx\n", 
-               (unsigned long)basev, (unsigned long)addv);
+    MP_TRACE_d(MP_FUNC, "basev==0x%lx, addv==0x%lx, mrg==0x%lx\n", 
+               (unsigned long)basev, (unsigned long)addv,
+               (unsigned long)mrg);
 
 #ifdef USE_ITHREADS
     merge_item(interp_scope);
@@ -137,22 +120,14 @@ void *modperl_config_dir_merge(apr_pool_t *p, void *basev, void *addv)
     mrg->flags = modperl_options_merge(p, base->flags, add->flags);
 
     merge_item(location);
-    
+
     merge_table_overlap_item(SetEnv);
 
     /* this is where we merge PerlSetVar and PerlAddVar together */
-    mrg->configvars = merge_table_config_vars(p,
-                                              base->configvars,
-                                              add->setvars, add->addvars);
-
-    /* note we don't care about merging dcfg->setvars or dcfg->addvars
-     * specifically - what is important to merge is dfcg->configvars.
-     * but we need to keep track of the entries for this config, so
-     * the merged values are simply the values for the add (current)
-     * configuration.
-     */
-    mrg->setvars = add->setvars;
-    mrg->addvars = add->addvars;
+    mrg->configvars = merge_config_add_vars(p,
+                                            base->configvars,
+                                            add->setvars, add->configvars);
+    merge_table_overlap_item(setvars);
 
     /* XXX: check if Perl*Handler is disabled */
     for (i=0; i < MP_HANDLER_NUM_PER_DIR; i++) {
@@ -183,16 +158,17 @@ modperl_config_srv_t *modperl_config_srv_new(apr_pool_t *p)
 
     scfg->PerlModule  = apr_array_make(p, 2, sizeof(char *));
     scfg->PerlRequire = apr_array_make(p, 2, sizeof(char *));
+    scfg->PerlPostConfigRequire =
+        apr_array_make(p, 1, sizeof(modperl_require_file_t *));
 
     scfg->argv = apr_array_make(p, 2, sizeof(char *));
 
     scfg->setvars = apr_table_make(p, 2);
-    scfg->addvars = apr_table_make(p, 2);
     scfg->configvars = apr_table_make(p, 2);
 
     scfg->PassEnv = apr_table_make(p, 2);
     scfg->SetEnv = apr_table_make(p, 2);
-    
+
 #ifdef MP_USE_GTOP
     scfg->gtop = modperl_gtop_new(p);
 #endif        
@@ -223,11 +199,10 @@ modperl_config_dir_t *modperl_config_dir_new(apr_pool_t *p)
     dcfg->flags = modperl_options_new(p, MpDirType);
 
     dcfg->setvars = apr_table_make(p, 2);
-    dcfg->addvars = apr_table_make(p, 2);
     dcfg->configvars = apr_table_make(p, 2);
 
     dcfg->SetEnv = apr_table_make(p, 2);
-    
+
     MP_TRACE_d(MP_FUNC, "new dcfg: 0x%lx\n", (unsigned long)dcfg);
 
     return dcfg;
@@ -248,7 +223,7 @@ static void dump_argv(modperl_config_srv_t *scfg)
 char **modperl_config_srv_argv_init(modperl_config_srv_t *scfg, int *argc)
 {
     modperl_config_srv_argv_push("-e;0");
-    
+
     *argc = scfg->argv->nelts;
 
     MP_TRACE_g_do(dump_argv(scfg));
@@ -260,8 +235,6 @@ void *modperl_config_srv_create(apr_pool_t *p, server_rec *s)
 {
     modperl_config_srv_t *scfg = modperl_config_srv_new(p);
 
-    ap_mpm_query(AP_MPMQ_IS_THREADED, &scfg->threaded_mpm);
-
     if (!s->is_virtual) {
 
         /* give a chance to MOD_PERL_TRACE env var to set
@@ -277,7 +250,7 @@ void *modperl_config_srv_create(apr_pool_t *p, server_rec *s)
          */
         modperl_init_globals(s, p);
     }
-    
+
     MP_TRACE_d(MP_FUNC, "p=0x%lx, s=0x%lx, virtual=%d\n",
                p, s, s->is_virtual);
 
@@ -311,31 +284,24 @@ void *modperl_config_srv_merge(apr_pool_t *p, void *basev, void *addv)
         *add  = (modperl_config_srv_t *)addv,
         *mrg  = modperl_config_srv_new(p);
 
-    MP_TRACE_d(MP_FUNC, "basev==0x%lx, addv==0x%lx\n", 
-               (unsigned long)basev, (unsigned long)addv);
+    MP_TRACE_d(MP_FUNC, "basev==0x%lx, addv==0x%lx, mrg==0x%lx\n", 
+               (unsigned long)basev, (unsigned long)addv,
+               (unsigned long)mrg);
 
     merge_item(modules);
     merge_item(PerlModule);
     merge_item(PerlRequire);
+    merge_item(PerlPostConfigRequire);
 
     merge_table_overlap_item(SetEnv);
     merge_table_overlap_item(PassEnv);
- 
+
     /* this is where we merge PerlSetVar and PerlAddVar together */
-    mrg->configvars = merge_table_config_vars(p,
-                                              base->configvars,
-                                              add->setvars, add->addvars);
+    mrg->configvars = merge_config_add_vars(p,
+                                            base->configvars,
+                                            add->setvars, add->configvars);
+    merge_table_overlap_item(setvars);
 
-    /* note we don't care about merging dcfg->setvars or dcfg->addvars
-     * specifically - what is important to merge is dfcg->configvars.
-     * but we need to keep track of the entries for this config, so
-     * the merged values are simply the values for the add (current)
-     * configuration.
-     */
-    mrg->setvars = add->setvars;
-    mrg->addvars = add->addvars;
-
-    merge_item(threaded_mpm);
     merge_item(server);
 
 #ifdef USE_ITHREADS
@@ -478,6 +444,40 @@ int modperl_config_apply_PerlRequire(server_rec *s,
     return TRUE;
 }
 
+int modperl_config_apply_PerlPostConfigRequire(server_rec *s,
+                                               modperl_config_srv_t *scfg,
+                                               apr_pool_t *p)
+{
+    modperl_require_file_t **requires;
+    int i;
+    MP_PERL_CONTEXT_DECLARE;
+
+    requires = (modperl_require_file_t **)scfg->PerlPostConfigRequire->elts;
+    for (i = 0; i < scfg->PerlPostConfigRequire->nelts; i++){
+        int retval;
+
+        MP_PERL_CONTEXT_STORE_OVERRIDE(scfg->mip->parent->perl);
+        retval = modperl_require_file(aTHX_ requires[i]->file, TRUE);
+        modperl_env_sync_srv_env_hash2table(aTHX_ p, scfg);
+        modperl_env_sync_dir_env_hash2table(aTHX_ p, requires[i]->dcfg);
+        MP_PERL_CONTEXT_RESTORE;
+
+        if (retval) {
+            MP_TRACE_d(MP_FUNC, "loaded Perl file: %s for server %s\n",
+                       requires[i]->file, modperl_server_desc(s, p));
+        }
+        else {
+            ap_log_error(APLOG_MARK, APLOG_ERR, 0, s,
+                         "Can't load Perl file: %s for server %s, exiting...",
+                         requires[i]->file, modperl_server_desc(s, p));
+            
+            return FALSE;
+        }
+    }
+
+    return TRUE;
+}
+
 typedef struct {
     AV *av;
     I32 ix;
@@ -561,6 +561,20 @@ const char *modperl_config_insert(pTHX_ server_rec *s,
     return errmsg;
 }
 
+const char *modperl_config_insert_parms(pTHX_ cmd_parms *parms, 
+                                        SV *lines)
+{
+    return modperl_config_insert(aTHX_ 
+                                 parms->server, 
+                                 parms->pool, 
+                                 parms->temp_pool,
+                                 parms->override, 
+                                 parms->path,
+                                 parms->context,
+                                 lines);
+}
+
+
 const char *modperl_config_insert_server(pTHX_ server_rec *s, SV *lines)
 {
     int override = (RSRC_CONF | OR_ALL) & ~(OR_AUTHCFG | OR_LIMIT);
