@@ -1,4 +1,4 @@
-/* Copyright 2001-2004 The Apache Software Foundation
+/* Copyright 2001-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -40,7 +40,7 @@ static void new_constsub(pTHX_ constants_lookup lookup,
         GvSHARED_on(*gvp);
 #endif
     }
-    
+
     /* export into callers namespace */
     if (caller_stash) {
         GV *alias = *(GV **)hv_fetch(caller_stash,
@@ -64,12 +64,12 @@ int modperl_const_compile(pTHX_ const char *classname,
     constants_group_lookup group_lookup;
 
     if (strnEQ(classname, "APR", 3)) {
-        lookup       = modperl_constants_lookup_apr;
-        group_lookup = modperl_constants_group_lookup_apr;
+        lookup       = modperl_constants_lookup_apr_const;
+        group_lookup = modperl_constants_group_lookup_apr_const;
     }
-    else if (strnEQ(classname, "Apache", 6)) {
-        lookup       = modperl_constants_lookup_apache;
-        group_lookup = modperl_constants_group_lookup_apache;
+    else if (strnEQ(classname, "Apache2", 7)) {
+        lookup       = modperl_constants_lookup_apache2_const;
+        group_lookup = modperl_constants_group_lookup_apache2_const;
     }
     else {
         lookup       = modperl_constants_lookup_modperl;
@@ -116,8 +116,8 @@ XS(XS_modperl_const_compile)
     }
 
     classname = *(stashname + 1) == 'P'
-        ? "APR" 
-        : (*stashname == 'A' ? "Apache" : "ModPerl");
+        ? "APR::Const" 
+        : (*stashname == 'A' ? "Apache2::Const" : "ModPerl");
     arg = SvPV(ST(1),n_a);
 
     for (i=2; i<items; i++) {
