@@ -1,4 +1,4 @@
-/* Copyright 2001-2004 The Apache Software Foundation
+/* Copyright 2001-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,7 +16,8 @@
 #include "mod_perl.h"
 
 static const char *MP_error_strings[] = {
-    "exit was called", /* MODPERL_RC_EXIT */
+    "exit was called",           /* MODPERL_RC_EXIT */
+    "filter handler has failed", /* MODPERL_FILTER_ERROR */
 };
 
 #define MP_error_strings_size \
@@ -26,7 +27,7 @@ char *modperl_error_strerror(pTHX_ apr_status_t rc)
 {
     char *ptr;
     char buf[256];
-        
+
     if (rc >= APR_OS_START_USERERR &&
         rc < APR_OS_START_USERERR + MP_error_strings_size) {
         /* custom mod_perl errors */
@@ -36,11 +37,11 @@ char *modperl_error_strerror(pTHX_ apr_status_t rc)
         /* apache apr errors */
         ptr = apr_strerror(rc, buf, sizeof(buf));
     }
-        
+
     /* must copy the string and not return a pointer to the local
      * address. Using a single (per interpreter) static buffer.
      */
-    return Perl_form(aTHX_ "%s", ptr);
+    return Perl_form(aTHX_ "%s", ptr ? ptr : "unknown error");
 }
 
 
@@ -65,7 +66,7 @@ void modperl_croak(pTHX_ apr_status_t rc, const char* func)
     HV *stash;
     HV *data;
     int is_tainted = PL_tainted;
-    
+
     /* see the explanation above */
     if (is_tainted) {
         TAINT_NOT;
@@ -74,11 +75,11 @@ void modperl_croak(pTHX_ apr_status_t rc, const char* func)
     if (is_tainted) {
         TAINT;
     }
-    
+
     if (SvTRUE(ERRSV)) {
         Perl_croak(aTHX_ "%s", SvPV_nolen(ERRSV));   
     }
-    
+
     stash = gv_stashpvn("APR::Error", 10, FALSE);
     data = newHV();
     /* $@ = bless {}, "APR::Error"; */
