@@ -1,4 +1,4 @@
-/* Copyright 2001-2004 The Apache Software Foundation
+/* Copyright 2001-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -66,18 +66,11 @@
                    );                               \
     }
 
-/* Restore previously saved value of $@, warning if a new error was
- * generated */
+/* Restore previously saved value of $@. if there was a filter error
+ * it'd have been logged by modperl_errsv call following
+ * modperl_callback */
 #define MP_FILTER_RESTORE_ERRSV(tmpsv)                  \
     if (tmpsv) {                                        \
-        if (SvTRUE(ERRSV)) {                            \
-            Perl_warn(aTHX_ "%s", SvPVX(ERRSV));        \
-            MP_TRACE_f(MP_FUNC, MP_FILTER_NAME_FORMAT   \
-                       "error: %s",                     \
-                        MP_FILTER_NAME(filter->f),      \
-                        SvPVX(ERRSV)                    \
-                        );                              \
-        }                                               \
         sv_setsv(ERRSV, tmpsv);                         \
         MP_TRACE_f(MP_FUNC, MP_FILTER_NAME_FORMAT       \
                    "Restoring $@='%s'",                 \
@@ -173,7 +166,7 @@ MP_INLINE apr_status_t modperl_wbucket_pass(modperl_wbucket_t *wb,
 
         MP_TRACE_f(MP_FUNC, "\n\n\tparsing headers: %db [%s]\n", len,
                    MP_TRACE_STR_TRUNC(wb->pool, buf, len));
-        
+
         status = modperl_cgi_header_parse(r, (char *)buf, &len, &body);
 
         wb->header_parse = 0; /* only once per-request */
@@ -296,7 +289,7 @@ static apr_status_t modperl_filter_f_cleanup(void *data)
         }
         ctx->perl = NULL;
     }
-    
+
     return APR_SUCCESS;
 }
 
@@ -324,7 +317,11 @@ modperl_filter_t *modperl_filter_new(ap_filter_t *f,
         return NULL;
     }
     filter = (modperl_filter_t *)apr_pcalloc(temp_pool, sizeof(*filter));
-                
+
+#ifdef MP_DEBUG
+    apr_pool_tag(temp_pool, "mod_perl temp filter");
+#endif
+
     filter->temp_pool = temp_pool;
     filter->mode      = mode;
     filter->f         = f;
@@ -357,13 +354,13 @@ modperl_filter_t *modperl_filter_new(ap_filter_t *f,
 
 static void modperl_filter_mg_set(pTHX_ SV *obj, modperl_filter_t *filter)
 {
-    sv_magic(SvRV(obj), Nullsv, '~', NULL, -1);
+    sv_magic(SvRV(obj), Nullsv, PERL_MAGIC_ext, NULL, -1);
     SvMAGIC(SvRV(obj))->mg_ptr = (char *)filter;
 }
 
 modperl_filter_t *modperl_filter_mg_get(pTHX_ SV *obj)
 {
-    MAGIC *mg = mg_find(SvRV(obj), '~');
+    MAGIC *mg = mg_find(SvRV(obj), PERL_MAGIC_ext);
     return mg ? (modperl_filter_t *)mg->mg_ptr : NULL;
 }
 
@@ -372,13 +369,13 @@ int modperl_filter_resolve_init_handler(pTHX_ modperl_handler_t *handler,
                                         apr_pool_t *p)
 {
     char *init_handler_pv_code = NULL;
-    
+
     if (handler->mgv_cv) {
         GV *gv = modperl_mgv_lookup(aTHX_ handler->mgv_cv);
         if (gv) {
             CV *cv = modperl_mgv_cv(gv);
             if (cv && SvMAGICAL(cv)) {
-                MAGIC *mg = mg_find((SV*)(cv), '~');
+                MAGIC *mg = mg_find((SV*)(cv), PERL_MAGIC_ext);
                 init_handler_pv_code = mg ? mg->mg_ptr : NULL;
             }
             else {
@@ -387,7 +384,7 @@ int modperl_filter_resolve_init_handler(pTHX_ modperl_handler_t *handler,
             }
         }
     }
-    
+
     if (init_handler_pv_code) {
         char *package_name =
             modperl_mgv_as_string(aTHX_ handler->mgv_cv, p, 1);
@@ -396,11 +393,14 @@ int modperl_filter_resolve_init_handler(pTHX_ modperl_handler_t *handler,
         /* eval the code in the parent handler's package's context */
         char *code = apr_pstrcat(p, "package ", package_name, ";",
                                  init_handler_pv_code, NULL);
-        SV *sv = eval_pv(code, TRUE);
+        SV *sv;
+        modperl_handler_t *init_handler;
 
+        ENTER;SAVETMPS;
+        sv = eval_pv(code, TRUE);
         /* fprintf(stderr, "code: %s\n", code); */
-        modperl_handler_t *init_handler =
-            modperl_handler_new_from_sv(aTHX_ p, sv);
+        init_handler = modperl_handler_new_from_sv(aTHX_ p, sv);
+        FREETMPS;LEAVE;
 
         if (init_handler) {
             MP_TRACE_h(MP_FUNC, "found init handler %s\n",
@@ -411,13 +411,13 @@ int modperl_filter_resolve_init_handler(pTHX_ modperl_handler_t *handler,
                            "the FilterInitHandler attribute set",
                            modperl_handler_name(init_handler));
             }
-            
+
             handler->next = init_handler;
             return 1;
         }
         else {
             Perl_croak(aTHX_ "failed to eval code: %s", code);
-            
+
         }
     }
 
@@ -441,9 +441,9 @@ static int modperl_run_filter_init(ap_filter_t *f,
 
     MP_TRACE_h(MP_FUNC, "running filter init handler %s\n",
                modperl_handler_name(handler));
-            
+
     modperl_handler_make_args(aTHX_ &args,
-                              "Apache::Filter", f,
+                              "Apache2::Filter", f,
                               NULL);
 
     modperl_filter_mg_set(aTHX_ AvARRAY(args)[0], filter);
@@ -465,7 +465,7 @@ static int modperl_run_filter_init(ap_filter_t *f,
 
     MP_TRACE_f(MP_FUNC, MP_FILTER_NAME_FORMAT
                "return: %d\n", modperl_handler_name(handler), status);
-    
+
     return status;  
 }
 
@@ -486,9 +486,9 @@ int modperl_run_filter(modperl_filter_t *filter)
     MP_dINTERP_SELECT(r, c, s);
 
     MP_FILTER_SAVE_ERRSV(errsv);
-    
+
     modperl_handler_make_args(aTHX_ &args,
-                              "Apache::Filter", filter->f,
+                              "Apache2::Filter", filter->f,
                               "APR::Brigade",
                               (filter->mode == MP_INPUT_FILTER_MODE
                                ? filter->bb_out
@@ -535,20 +535,20 @@ int modperl_run_filter(modperl_filter_t *filter)
             filter->bb_in = NULL;
         }
         MP_RUN_CROAK(modperl_input_filter_flush(filter),
-                     "Apache::Filter");
+                     "Apache2::Filter");
     }
     else {
         MP_RUN_CROAK(modperl_output_filter_flush(filter),
-                     "Apache::Filter");
+                     "Apache2::Filter");
     }
 
+    MP_FILTER_RESTORE_ERRSV(errsv);
+
     MP_INTERP_PUTBACK(interp);
 
     MP_TRACE_f(MP_FUNC, MP_FILTER_NAME_FORMAT
                "return: %d\n", modperl_handler_name(handler), status);
-    
-    MP_FILTER_RESTORE_ERRSV(errsv);
- 
+
     return status;
 }
 
@@ -581,7 +581,7 @@ MP_INLINE static int get_bucket(modperl_filter_t *filter)
                    MP_FILTER_NAME(filter->f));
         return 0;
     }
-    
+
     if (!filter->bucket) {
         filter->bucket = MP_FILTER_FIRST(filter);
     }
@@ -596,7 +596,7 @@ MP_INLINE static int get_bucket(modperl_filter_t *filter)
         apr_brigade_cleanup(filter->bb_in);
         return 0;
     }
-    
+
     if (MP_FILTER_IS_EOS(filter)) {
         MP_TRACE_f(MP_FUNC, MP_FILTER_NAME_FORMAT
                    "read in: EOS bucket\n",
@@ -628,7 +628,7 @@ MP_INLINE static apr_size_t modperl_filter_read(pTHX_
 {
     int num_buckets = 0;
     apr_size_t len = 0;
-    
+
     (void)SvUPGRADE(buffer, SVt_PV);
     SvPOK_only(buffer);
     SvCUR(buffer) = 0;
@@ -641,8 +641,8 @@ MP_INLINE static apr_size_t modperl_filter_read(pTHX_
     if (filter->seen_eos) {
         return 0;
     }
-    
-    /*modperl_brigade_dump(filter->bb_in, stderr);*/
+
+    /* modperl_brigade_dump(filter->bb_in, NULL); */
 
     MP_TRACE_f(MP_FUNC, MP_FILTER_NAME_FORMAT
                "wanted: %db\n",
@@ -698,7 +698,7 @@ MP_INLINE static apr_size_t modperl_filter_read(pTHX_
         }
         else {
             SvREFCNT_dec(buffer);
-            modperl_croak(aTHX_ filter->rc, "Apache::Filter::read");
+            modperl_croak(aTHX_ filter->rc, "Apache2::Filter::read");
         }
 
         if (buf_len) {
@@ -746,7 +746,7 @@ MP_INLINE apr_size_t modperl_input_filter_read(pTHX_
         MP_RUN_CROAK(ap_get_brigade(filter->f->next, filter->bb_in,
                                     filter->input_mode, filter->block,
                                     filter->readbytes),
-                     "Apache::Filter::read");
+                     "Apache2::Filter::read");
     }
 
     len = modperl_filter_read(aTHX_ filter, buffer, wanted);
@@ -756,7 +756,7 @@ MP_INLINE apr_size_t modperl_input_filter_read(pTHX_
         apr_status_t rc = modperl_input_filter_flush(filter);
         if (rc != APR_SUCCESS) {
             SvREFCNT_dec(buffer);
-            modperl_croak(aTHX_ rc, "Apache::Filter::read");
+            modperl_croak(aTHX_ rc, "Apache2::Filter::read");
         }
     }
 
@@ -771,13 +771,13 @@ MP_INLINE apr_size_t modperl_output_filter_read(pTHX_
 {
     apr_size_t len = 0;
     len = modperl_filter_read(aTHX_ filter, buffer, wanted);
-    
+
     if (filter->flush && len == 0) {
         /* if len > 0 then $filter->write will flush */
         apr_status_t rc = modperl_output_filter_flush(filter);
         if (rc != APR_SUCCESS) {
             SvREFCNT_dec(buffer);
-            modperl_croak(aTHX_ rc, "Apache::Filter::read");
+            modperl_croak(aTHX_ rc, "Apache2::Filter::read");
         }
     }
 
@@ -801,14 +801,14 @@ MP_INLINE apr_status_t modperl_input_filter_flush(modperl_filter_t *filter)
         filter->rc = send_input_eos(filter);
         filter->eos = 0;
     }
-    
+
     return filter->rc;
 }
 
 MP_INLINE apr_status_t modperl_output_filter_flush(modperl_filter_t *filter)
 {
     int add_flush_bucket = FALSE;
-    
+
     if (((modperl_filter_ctx_t *)filter->f->ctx)->sent_eos) {
         /* no data should be sent after EOS has been sent */
         return filter->rc;
@@ -850,7 +850,7 @@ MP_INLINE apr_status_t modperl_input_filter_write(pTHX_
                MP_FILTER_NAME(filter->f), *len,
                MP_TRACE_STR_TRUNC(filter->pool, copy, *len));
     APR_BRIGADE_INSERT_TAIL(filter->bb_out, bucket);
-    /* modperl_brigade_dump(filter->bb_out, stderr); */
+    /* modperl_brigade_dump(filter->bb_out, NULL); */
     return APR_SUCCESS;
 }
 
@@ -884,7 +884,7 @@ apr_status_t modperl_output_filter_handler(ap_filter_t *f,
         status = modperl_run_filter(filter);
         FILTER_FREE(filter);
     }
-    
+
     switch (status) {
       case OK:
         return APR_SUCCESS;
@@ -918,12 +918,16 @@ apr_status_t modperl_input_filter_handler(ap_filter_t *f,
         status = modperl_run_filter(filter);
         FILTER_FREE(filter);
     }
-    
+
     switch (status) {
       case OK:
         return APR_SUCCESS;
       case DECLINED:
         return ap_get_brigade(f->next, bb, input_mode, block, readbytes);
+      case HTTP_INTERNAL_SERVER_ERROR:
+          /* XXX: later may introduce separate error codes for
+           * modperl_run_filter and modperl_run_filter_init */
+        return MODPERL_FILTER_ERROR;
       default:
         return status; /*XXX*/
     }
@@ -949,13 +953,31 @@ static int modperl_filter_add_connection(conn_rec *c,
 
             /* process non-mod_perl filter handlers */
             if ((handlers[i]->attrs & MP_FILTER_HTTPD_HANDLER)) {
+
+                /* non-mp2 filters below PROTOCOL level can't be added
+                 * at the connection level, so we need to go through
+                 * the pain of figuring out the type of the filter */
+                ap_filter_rec_t *frec;
+                char *normalized_name = apr_pstrdup(c->pool,
+                                                    handlers[i]->name);
+                ap_str_tolower(normalized_name);
+                frec = idx == MP_INPUT_FILTER_HANDLER
+                    ? ap_get_input_filter_handle(normalized_name)
+                    : ap_get_output_filter_handle(normalized_name);
+                if (frec && frec->ftype < AP_FTYPE_PROTOCOL) {
+                    MP_TRACE_f(MP_FUNC, "a non-mod_perl %s handler %s "
+                               "skipped (not a connection filter)",
+                               type, handlers[i]->name);
+                    continue;
+                }
+
                 addfunc(handlers[i]->name, NULL, NULL, c);
                 MP_TRACE_f(MP_FUNC,
-                           "a non-mod_perl %s handler %s configured (connection)\n",
-                           type, handlers[i]->name);
+                           "a non-mod_perl %s handler %s configured "
+                           "(connection)\n", type, handlers[i]->name);
                 continue;
             }
-            
+
             /* skip non-connection level filters, e.g. request filters
              * configured outside the resource container */
             if (!(handlers[i]->attrs & MP_FILTER_CONNECTION_HANDLER)) {
@@ -986,7 +1008,7 @@ static int modperl_filter_add_connection(conn_rec *c,
                     return status;
                 }
             }
-            
+
             MP_TRACE_h(MP_FUNC, "%s handler %s configured (connection)\n",
                        type, handlers[i]->name);
         }
@@ -1094,7 +1116,7 @@ static int modperl_filter_add_request(request_rec *r,
                     return status;
                 }
             }
-            
+
             MP_TRACE_h(MP_FUNC, "%s handler %s configured (%s)\n",
                        type, handlers[i]->name, r->uri);
         }
@@ -1204,11 +1226,11 @@ void modperl_filter_runtime_add(pTHX_ request_rec *r, conn_rec *c,
             int status = modperl_run_filter_init(f, mode, handler->next);
             if (status != OK) {
                 modperl_croak(aTHX_ status, strEQ("InputFilter", type)
-                              ? "Apache::Filter::add_input_filter"
-                              : "Apache::Filter::add_output_filter");
+                              ? "Apache2::Filter::add_input_filter"
+                              : "Apache2::Filter::add_output_filter");
             }
         }
-        
+
         MP_TRACE_h(MP_FUNC, "%s handler %s configured (connection)\n",
                    type, name);
 
@@ -1219,29 +1241,29 @@ void modperl_filter_runtime_add(pTHX_ request_rec *r, conn_rec *c,
                (unsigned long)callback);
 }
 
-void modperl_brigade_dump(apr_bucket_brigade *bb, FILE *fp)
+void modperl_brigade_dump(apr_bucket_brigade *bb, apr_file_t *file)
 {
     apr_bucket *bucket;
     int i = 0;
 #ifndef WIN32
-    if (fp == NULL) {
-        fp = stderr;
+    if (file == NULL) {
+        file = modperl_global_get_server_rec()->error_log;
     }
 
-    fprintf(fp, "dump of brigade 0x%lx\n",
-            (unsigned long)bb);
+    apr_file_printf(file, "dump of brigade 0x%lx\n", (unsigned long)bb);
 
     for (bucket = APR_BRIGADE_FIRST(bb);
          bucket != APR_BRIGADE_SENTINEL(bb);
          bucket = APR_BUCKET_NEXT(bucket))
     {
-        fprintf(fp, "   %d: bucket=%s(0x%lx), length=%ld, data=0x%lx\n",
-                i, bucket->type->name,
-                (unsigned long)bucket,
-                (long)bucket->length,
-                (unsigned long)bucket->data);
-        /* fprintf(fp, "       : %s\n", (char *)bucket->data); */
-        
+        apr_file_printf(file,
+                        "   %d: bucket=%s(0x%lx), length=%ld, data=0x%lx\n",
+                        i, bucket->type->name,
+                        (unsigned long)bucket,
+                        (long)bucket->length,
+                        (unsigned long)bucket->data);
+        /* apr_file_printf(file, "       : %s\n", (char *)bucket->data); */
+
         i++;
     }
 #endif
