@@ -1,3 +1,18 @@
+/* Copyright 2001-2005 The Apache Software Foundation
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
 #ifndef MODPERL_FILTER_H
 #define MODPERL_FILTER_H
 
@@ -68,7 +83,7 @@ MP_INLINE apr_status_t modperl_output_filter_write(pTHX_
                                                    const char *buf,
                                                    apr_size_t *len);
 
-void modperl_brigade_dump(apr_bucket_brigade *bb, FILE *fp);
+void modperl_brigade_dump(apr_bucket_brigade *bb, apr_file_t *file);
 
 /* input filters */
 apr_status_t modperl_input_filter_handler(ap_filter_t *f,
