@@ -1,4 +1,4 @@
-/* Copyright 2001-2004 The Apache Software Foundation
+/* Copyright 2001-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -162,8 +162,40 @@ void modperl_global_set_##gname(void *data)              \
     modperl_global_set(&MP_global_##gname, data);        \
 }                                                        \
 
-MP_GLOBAL_IMPL(pconf, apr_pool_t *);
-MP_GLOBAL_IMPL(server_rec, server_rec *);
+MP_GLOBAL_IMPL(pconf, apr_pool_t *)
+MP_GLOBAL_IMPL(server_rec, server_rec *)
+
+
+
+
+
+/*** anon handlers code ***/
+
+static modperl_global_t MP_global_anon_cnt;
+
+void modperl_global_anon_cnt_init(apr_pool_t *p)
+{
+    int *data = (int *)apr_pcalloc(p, sizeof(int));
+    *data = 0;
+    modperl_global_init(&MP_global_anon_cnt, p, (void *)data, "anon_cnt");
+}
+
+int modperl_global_anon_cnt_next(void)
+{
+    int next;
+    /* XXX: inline lock/unlock? */
+    modperl_global_lock(&MP_global_anon_cnt); 
+
+    next = ++*(int *)(MP_global_anon_cnt.data);
+
+    modperl_global_unlock(&MP_global_anon_cnt); 
+
+    return next;
+}
+
+
+
+/*** TLS ***/
 
 #if MP_THREADED
 static apr_status_t modperl_tls_cleanup(void *data)
@@ -269,4 +301,4 @@ void modperl_tls_reset_cleanup_##gname(apr_pool_t *p,    \
                               (void *)data);             \
 }
 
-MP_TLS_IMPL(request_rec, request_rec *);
+MP_TLS_IMPL(request_rec, request_rec *)
