@@ -1,4 +1,4 @@
-/* Copyright 2001-2004 The Apache Software Foundation
+/* Copyright 2001-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -43,6 +43,115 @@ modperl_handler_t *modperl_handler_new(apr_pool_t *p, const char *name)
     return handler;
 }
 
+/* How anon-subs are handled:
+ * We have two ways anon-subs can be registered
+ * A) at startup from httpd.conf:
+ *    PerlTransHandler 'sub { ... }'
+ * B) run-time perl code
+ *    $r->push_handlers(PerlTransHandler => sub { .... });
+ *    $s->push_handlers(PerlTransHandler => sub { .... });
+ *
+ * In the case of non-threaded perl, we just compile A or grab B and
+ * store it in the mod_perl struct and call it when it's used. No
+ * problems here
+ *
+ * In the case of threads, things get more complicated. we no longer
+ * can store the CV value of the compiled anon-sub, since when
+ * perl_clone is called each interpreter will have a different CV
+ * value. since we need to be able to have 1 entry for each anon-sub
+ * across all interpreters a different solution is needed. to remind
+ * in the case of named subs, we just store the name of the sub and
+ * look its corresponding CV when we need it.
+ *
+ * The used solution: each process has a global counter, which always
+ * grows. Every time a new anon-sub is encountered, a new ID is
+ * allocated from that process-global counter and that ID is stored in
+ * the mod_perl struct. The compiled CV is stored as
+ *     $PL_modglobal{ANONSUB}{$id} = CV;
+ * when perl_clone is called, each clone will clone that CV value, but
+ * we will still be able to find it, since we stored it in the
+ * hash. so we retrieve the CV value, whatever it is and we run it.
+ * 
+ * that explanation can be written and run in perl:
+ *
+ * use threads;
+ * our %h;
+ * $h{x} = eval 'sub { print qq[this is sub @_\n] }';
+ * $h{x}->("main");
+ * threads->new(sub { $h{x}->(threads->self->tid)});
+ *
+ * XXX: more nuances will follow
+ */
+
+void modperl_handler_anon_init(pTHX_ apr_pool_t *p)
+{
+    modperl_modglobal_key_t *gkey =
+        modperl_modglobal_lookup(aTHX_ "ANONSUB");
+    MP_TRACE_h(MP_FUNC, "init $PL_modglobal{ANONSUB} = []");
+    MP_MODGLOBAL_STORE_HV(gkey);
+}
+
+/* allocate and populate the anon handler sub-struct */
+MP_INLINE modperl_mgv_t *modperl_handler_anon_next(pTHX_ apr_pool_t *p)
+{
+    /* re-use modperl_mgv_t entry which is otherwise is not used
+     * by anon handlers */
+    modperl_mgv_t *anon = 
+        (modperl_mgv_t *)apr_pcalloc(p, sizeof(*anon));
+
+    anon->name = apr_psprintf(p, "anon%d", modperl_global_anon_cnt_next());
+    anon->len  = strlen(anon->name);
+    PERL_HASH(anon->hash, anon->name, anon->len);
+
+    MP_TRACE_h(MP_FUNC, "[%s] new anon handler: '%s'",
+               modperl_pid_tid(p), anon->name);
+    return anon;
+}
+
+MP_INLINE void modperl_handler_anon_add(pTHX_ modperl_mgv_t *anon, CV *cv)
+{
+    modperl_modglobal_key_t *gkey =
+        modperl_modglobal_lookup(aTHX_ "ANONSUB");
+    HE *he = MP_MODGLOBAL_FETCH(gkey);
+    HV *hv;
+
+    if (!(he && (hv = (HV*)HeVAL(he)))) {
+        Perl_croak(aTHX_ "modperl_handler_anon_add: "
+                   "can't find ANONSUB top entry (get)");
+    }
+
+    SvREFCNT_inc(cv);
+    if (!(*hv_store(hv, anon->name, anon->len, (SV*)cv, anon->hash))) {
+        SvREFCNT_dec(cv);
+        Perl_croak(aTHX_ "hv_store of anonsub '%s' has failed!", anon->name);
+    }
+
+    MP_TRACE_h(MP_FUNC, "anonsub '%s' added", anon->name);
+}
+
+MP_INLINE CV *modperl_handler_anon_get(pTHX_ modperl_mgv_t *anon)
+{
+    modperl_modglobal_key_t *gkey =
+        modperl_modglobal_lookup(aTHX_ "ANONSUB");
+    HE *he = MP_MODGLOBAL_FETCH(gkey);
+    HV *hv;
+    SV *sv;
+
+    if (!(he && (hv = (HV*)HeVAL(he)))) {
+        Perl_croak(aTHX_ "modperl_handler_anon_get: "
+                   "can't find ANONSUB top entry (get)");
+    }
+
+    if ((he = hv_fetch_he(hv, anon->name, anon->len, anon->hash))) {
+        sv = HeVAL(he);
+        MP_TRACE_h(MP_FUNC, "anonsub gets name '%s'", anon->name);
+    }
+    else {
+        Perl_croak(aTHX_ "can't find ANONSUB's '%s' entry", anon->name);
+    }
+
+    return (CV*)sv;
+}
 
 static
 modperl_handler_t *modperl_handler_new_anon(pTHX_ apr_pool_t *p, CV *cv)
@@ -53,34 +162,17 @@ modperl_handler_t *modperl_handler_new_anon(pTHX_ apr_pool_t *p, CV *cv)
     MpHandlerANON_On(handler);
 
 #ifdef USE_ITHREADS
-    /* XXX: perhaps we can optimize this further. At the moment when
-     * perl w/ ithreads is used, we always deparse the anon subs
-     * before storing them and then eval them each time they are
-     * used. This is because we don't know whether the same perl that
-     * compiled the anonymous sub is used to run it.
-     *
-     * A possible optimization is to cache the CV and use that cached
-     * value w/ or w/o deparsing at all if:
-     *
-     * - the mpm is non-threaded mpm and no +Clone/+Parent is used
-     *   (i.e. no perl pools) (no deparsing is needed at all)
-     * 
-     * - the interpreter that has supplied the anon cv is the same
-     *   interpreter that is executing that cv (requires storing aTHX
-     *   in the handler's struct) (need to deparse in case the
-     *   interpreter gets switched)
-     *
-     * - other cases?
-     */
-    handler->cv = NULL;
-    handler->name = modperl_coderef2text(aTHX_ p, cv);
-    MP_TRACE_h(MP_FUNC, "[%s] new deparsed anon handler:\n%s\n",
-               modperl_pid_tid(p), handler->name);
+    handler->cv      = NULL;
+    handler->name    = NULL;
+    handler->mgv_obj = modperl_handler_anon_next(aTHX_ p);
+    modperl_handler_anon_add(aTHX_ handler->mgv_obj, cv);
 #else
     /* it's safe to cache and later use the cv, since the same perl
      * interpeter is always used */
-    handler->cv = cv;
+    SvREFCNT_inc((SV*)cv);
+    handler->cv   = cv;
     handler->name = NULL;
+
     MP_TRACE_h(MP_FUNC, "[%s] new cached cv anon handler\n",
                modperl_pid_tid(p));
 #endif
@@ -93,7 +185,13 @@ const char *modperl_handler_name(modperl_handler_t *handler)
 {
     /* a handler containing an anonymous sub doesn't have a normal sub
      * name */
-    return handler->name ? handler->name : "anonymous sub";
+    if (handler->name) {
+        return handler->name;
+    }
+    else {
+        /* anon sub stores the internal modperl name in mgv_obj */
+        return handler->mgv_obj ? handler->mgv_obj->name : "anonsub";
+    }
 }
 
 
@@ -104,28 +202,30 @@ int modperl_handler_resolve(pTHX_ modperl_handler_t **handp,
     modperl_handler_t *handler = *handp;
 
 #ifdef USE_ITHREADS
-    if (p && !MpHandlerPARSED(handler) && !MpHandlerDYNAMIC(handler)) {
+    if (modperl_threaded_mpm() && p &&
+        !MpHandlerPARSED(handler) && !MpHandlerDYNAMIC(handler)) {
         /*
-         * cannot update the handler structure at request time without
-         * locking, so just copy it
+         * under threaded mpm we cannot update the handler structure
+         * at request time without locking, so just copy it
          */
         handler = *handp = modperl_handler_dup(p, handler);
         duped = 1;
     }
 #endif
 
-    MP_TRACE_h_do(MpHandler_dump_flags(handler, handler->name));
+    MP_TRACE_h_do(MpHandler_dump_flags(handler,
+                                       modperl_handler_name(handler)));
 
     if (!MpHandlerPARSED(handler)) {
         apr_pool_t *rp = duped ? p : s->process->pconf;
         MpHandlerAUTOLOAD_On(handler);
 
         MP_TRACE_h(MP_FUNC,
-                   "[%s %s] handler %s was not compiled at startup, "
+                   "[%s %s] handler %s hasn't yet been resolved, "
                    "attempting to resolve using %s pool 0x%lx\n",
                    modperl_pid_tid(p),
                    modperl_server_desc(s, p),
-                   handler->name,
+                   modperl_handler_name(handler),
                    duped ? "current" : "server conf",
                    (unsigned long)rp);
 
@@ -143,7 +243,7 @@ int modperl_handler_resolve(pTHX_ modperl_handler_t **handp,
 modperl_handler_t *modperl_handler_dup(apr_pool_t *p,
                                        modperl_handler_t *h)
 {
-    MP_TRACE_h(MP_FUNC, "dup handler %s\n", h->name);
+    MP_TRACE_h(MP_FUNC, "dup handler %s\n", modperl_handler_name(h));
     return modperl_handler_new(p, h->name);
 }
 
@@ -206,7 +306,7 @@ void modperl_handler_make_args(pTHX_ AV **avp, ...)
         char *classname = va_arg(args, char *);
         void *ptr;
         SV *sv;
-            
+
         if (classname == NULL) {
             break;
         }
@@ -218,10 +318,10 @@ void modperl_handler_make_args(pTHX_ AV **avp, ...)
             if (strEQ(classname, "APR::Table")) {
                 sv = modperl_hash_tie(aTHX_ classname, Nullsv, ptr);
                 break;
-            }  
+            }
           case 'I':
             if (strEQ(classname, "IV")) {
-                sv = ptr ? newSViv((IV)ptr) : &PL_sv_undef;
+                sv = ptr ? newSViv(PTR2IV(ptr)) : &PL_sv_undef;
                 break;
             }
           case 'P':
@@ -245,15 +345,15 @@ void modperl_handler_make_args(pTHX_ AV **avp, ...)
     va_end(args);
 }
 
-#define set_desc(dtype) \
+#define set_desc(dtype)                                 \
     if (desc) *desc = modperl_handler_desc_##dtype(idx)
 
-#define check_modify(dtype) \
-if ((action > MP_HANDLER_ACTION_GET) && rcfg) { \
-    dTHX; \
-    Perl_croak(aTHX_ "too late to modify %s handlers", \
-               modperl_handler_desc_##dtype(idx)); \
-}
+#define check_modify(dtype)                                     \
+    if ((action > MP_HANDLER_ACTION_GET) && rcfg) {             \
+        MP_dSCFG_dTHX;                                          \
+        Perl_croak(aTHX_ "too late to modify %s handlers",      \
+                   modperl_handler_desc_##dtype(idx));          \
+    }
 
 /*
  * generic function to lookup handlers for use in modperl_callback(),
