@@ -1,4 +1,4 @@
-/* Copyright 2001-2004 The Apache Software Foundation
+/* Copyright 2001-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,10 +16,10 @@
 #include "mod_perl.h"
 
 #define TIEHANDLE(handle,r) \
-modperl_io_handle_tie(aTHX_ handle, "Apache::RequestRec", (void *)r)
+modperl_io_handle_tie(aTHX_ handle, "Apache2::RequestRec", (void *)r)
 
 #define TIED(handle) \
-modperl_io_handle_tied(aTHX_ handle, "Apache::RequestRec")
+modperl_io_handle_tied(aTHX_ handle, "Apache2::RequestRec")
 
 MP_INLINE void modperl_io_handle_tie(pTHX_ GV *handle,
                                      char *classname, void *ptr)
@@ -28,7 +28,7 @@ MP_INLINE void modperl_io_handle_tie(pTHX_ GV *handle,
 
     modperl_io_handle_untie(aTHX_ handle);
 
-    sv_magic(TIEHANDLE_SV(handle), obj, 'q', Nullch, 0);
+    sv_magic(TIEHANDLE_SV(handle), obj, PERL_MAGIC_tiedscalar, Nullch, 0);
 
     SvREFCNT_dec(obj); /* since sv_magic did SvREFCNT_inc */
 
@@ -78,7 +78,7 @@ MP_INLINE int modperl_io_handle_tied(pTHX_ GV *handle, char *classname)
     MAGIC *mg;
     SV *sv = TIEHANDLE_SV(handle);
 
-    if (SvMAGICAL(sv) && (mg = mg_find(sv, 'q'))) {
+    if (SvMAGICAL(sv) && (mg = mg_find(sv, PERL_MAGIC_tiedscalar))) {
 	char *package = HvNAME(SvSTASH((SV*)SvRV(mg->mg_obj)));
 
 	if (!strEQ(package, classname)) {
@@ -93,14 +93,14 @@ MP_INLINE int modperl_io_handle_tied(pTHX_ GV *handle, char *classname)
 MP_INLINE void modperl_io_handle_untie(pTHX_ GV *handle)
 {
 #ifdef MP_TRACE
-    if (mg_find(TIEHANDLE_SV(handle), 'q')) {
+    if (mg_find(TIEHANDLE_SV(handle), PERL_MAGIC_tiedscalar)) {
         MP_TRACE_r(MP_FUNC, "untie *%s(0x%lx), REFCNT=%d\n",
                    GvNAME(handle), (unsigned long)handle,
                    SvREFCNT(TIEHANDLE_SV(handle)));
     }
 #endif
 
-    sv_unmagic(TIEHANDLE_SV(handle), 'q');
+    sv_unmagic(TIEHANDLE_SV(handle), PERL_MAGIC_tiedscalar);
 }
 
 MP_INLINE GV *modperl_io_perlio_override_stdin(pTHX_ request_rec *r)
@@ -116,7 +116,7 @@ MP_INLINE GV *modperl_io_perlio_override_stdin(pTHX_ request_rec *r)
     if (handle && SvTYPE(handle) == SVt_PVGV &&
         IoTYPE(GvIO(handle)) != IoTYPE_CLOSED) {
         handle_save = gv_fetchpv(Perl_form(aTHX_
-                                           "Apache::RequestIO::_GEN_%ld",
+                                           "Apache2::RequestIO::_GEN_%ld",
                                            (long)PL_gensym++),
                                  TRUE, SVt_PVIO);
 
@@ -133,8 +133,8 @@ MP_INLINE GV *modperl_io_perlio_override_stdin(pTHX_ request_rec *r)
         Perl_do_close(aTHX_ handle, TRUE);
     }
 
-    sv_setref_pv(sv, "Apache::RequestRec", (void*)r);
-    status = Perl_do_open9(aTHX_ handle, "<:Apache", 8, FALSE, O_RDONLY,
+    sv_setref_pv(sv, "Apache2::RequestRec", (void*)r);
+    status = Perl_do_open9(aTHX_ handle, "<:Apache2", 9, FALSE, O_RDONLY,
                            0, Nullfp, sv, 1);
     if (status == 0) {
         Perl_croak(aTHX_ "Failed to open STDIN: %_", get_sv("!", TRUE));
@@ -159,10 +159,10 @@ MP_INLINE GV *modperl_io_perlio_override_stdout(pTHX_ request_rec *r)
     if (handle && SvTYPE(handle) == SVt_PVGV &&
         IoTYPE(GvIO(handle)) != IoTYPE_CLOSED) {
         handle_save = gv_fetchpv(Perl_form(aTHX_
-                                           "Apache::RequestIO::_GEN_%ld",
+                                           "Apache2::RequestIO::_GEN_%ld",
                                            (long)PL_gensym++),
                                  TRUE, SVt_PVIO);
-        
+
         /* open my $oldout, ">&STDOUT" or die "Can't dup STDOUT: $!"; */
         status = Perl_do_open(aTHX_ handle_save, ">&STDOUT", 8, FALSE,
                               O_WRONLY, 0, Nullfp);
@@ -176,8 +176,8 @@ MP_INLINE GV *modperl_io_perlio_override_stdout(pTHX_ request_rec *r)
         Perl_do_close(aTHX_ handle, TRUE);
     }
 
-    sv_setref_pv(sv, "Apache::RequestRec", (void*)r);
-    status = Perl_do_open9(aTHX_ handle, ">:Apache", 8, FALSE, O_WRONLY,
+    sv_setref_pv(sv, "Apache2::RequestRec", (void*)r);
+    status = Perl_do_open9(aTHX_ handle, ">:Apache2", 9, FALSE, O_WRONLY,
                            0, Nullfp, sv, 1);
     if (status == 0) {
         Perl_croak(aTHX_ "Failed to open STDOUT: %_", get_sv("!", TRUE));
@@ -190,7 +190,7 @@ MP_INLINE GV *modperl_io_perlio_override_stdout(pTHX_ request_rec *r)
     IoFLUSH_off(handle); /* STDOUT's $|=0 */
 
     return handle_save;
-    
+
 }
 
 MP_INLINE void modperl_io_perlio_restore_stdin(pTHX_ GV *handle)
@@ -208,16 +208,16 @@ MP_INLINE void modperl_io_perlio_restore_stdin(pTHX_ GV *handle)
      */
     if (handle != (GV*)Nullsv) {
         SV *err = Nullsv;
-        
+
         MP_TRACE_o(MP_FUNC, "restoring STDIN");
 
         if (Perl_do_open9(aTHX_ handle_orig, "<&", 2, FALSE,
                           O_RDONLY, 0, Nullfp, (SV*)handle, 1) == 0) {
             err = get_sv("!", TRUE);
         }
-        
+
         Perl_do_close(aTHX_ handle, FALSE);
-        (void)hv_delete(gv_stashpv("Apache::RequestIO", TRUE), 
+        (void)hv_delete(gv_stashpv("Apache2::RequestIO", TRUE), 
                         GvNAME(handle), GvNAMELEN(handle), G_DISCARD);
 
         if (err != Nullsv) {
@@ -248,23 +248,23 @@ MP_INLINE void modperl_io_perlio_restore_stdout(pTHX_ GV *handle)
 
     /* close the overriding filehandle */
     Perl_do_close(aTHX_ handle_orig, FALSE);
-    
+
     /*
      * open STDOUT, ">&STDOUT_SAVED" or die "Can't dup STDOUT_SAVED: $!";
      * close STDOUT_SAVED;
      */
     if (handle != (GV*)Nullsv) {
         SV *err = Nullsv;
-        
+
         MP_TRACE_o(MP_FUNC, "restoring STDOUT");
 
         if (Perl_do_open9(aTHX_ handle_orig, ">&", 2, FALSE,
                           O_WRONLY, 0, Nullfp, (SV*)handle, 1) == 0) {
             err = get_sv("!", TRUE);
         }
-        
+
         Perl_do_close(aTHX_ handle, FALSE);
-        (void)hv_delete(gv_stashpv("Apache::RequestIO", TRUE), 
+        (void)hv_delete(gv_stashpv("Apache2::RequestIO", TRUE), 
                         GvNAME(handle), GvNAMELEN(handle), G_DISCARD);
 
         if (err != Nullsv) {
