@@ -1,4 +1,4 @@
-/* Copyright 2001-2004 The Apache Software Foundation
+/* Copyright 2001-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -191,11 +191,37 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
         /* already resolved anonymous handler */
         return 1;
     }
-    
+
     if (strnEQ(name, "sub ", 4)) {
-        MP_TRACE_h(MP_FUNC, "handler is anonymous\n");
-        MpHandlerANON_On(handler);
+        SV *sv;
+        CV *cv;
         MpHandlerPARSED_On(handler);
+        MpHandlerANON_On(handler);
+
+        ENTER;SAVETMPS;
+        sv = eval_pv(name, TRUE);
+        if (!(SvROK(sv) && (cv = (CV*)SvRV(sv)) && (CvFLAGS(cv) & CVf_ANON))) {
+
+            Perl_croak(aTHX_ "expected anonymous sub, got '%s'\n", name);
+        }
+
+#ifdef USE_ITHREADS
+        handler->cv      = NULL;
+        handler->name    = NULL;
+        handler->mgv_obj = modperl_handler_anon_next(aTHX_ p);
+        modperl_handler_anon_add(aTHX_ handler->mgv_obj, cv);
+        MP_TRACE_h(MP_FUNC, "[%s] new anon handler",
+                   modperl_pid_tid(p));
+#else
+        SvREFCNT_inc(cv);
+        handler->cv      = cv;
+        handler->name    = NULL;
+        MP_TRACE_h(MP_FUNC, "[%s] new cached-cv anon handler",
+                   modperl_pid_tid(p));
+#endif
+
+        FREETMPS;LEAVE;
+
         return 1;
     }
 
@@ -302,7 +328,7 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
         /* note: this is the real function after @ISA lookup */
         handler->mgv_cv = modperl_mgv_compile(aTHX_ p, HvNAME(GvSTASH(gv)));
         modperl_mgv_append(aTHX_ p, handler->mgv_cv, handler_name);
-  
+
         MpHandlerPARSED_On(handler);
         MP_TRACE_h(MP_FUNC, "[%s] found `%s' in class `%s' as a %s\n",
                    modperl_pid_tid(p),
@@ -319,7 +345,7 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
     if (MpHandlerAUTOLOAD(handler)) {
         Perl_croak(aTHX_ "failed to resolve handler %s\n", name);
     }
-    
+
 #ifdef MP_TRACE
     /* complain only if the class was actually loaded/created */
     if (stash) {
@@ -425,7 +451,8 @@ static void modperl_hash_handlers(pTHX_ apr_pool_t *p, server_rec *s,
             }
 #endif
             MP_TRACE_h(MP_FUNC, "%s already resolved in server %s\n",
-                       handler->name, modperl_server_desc(s, p));
+                       modperl_handler_name(handler),
+                       modperl_server_desc(s, p));
         }
         else {
             if (MpSrvAUTOLOAD(scfg)) {
