@@ -1,4 +1,4 @@
-/* Copyright 2002-2004 The Apache Software Foundation
+/* Copyright 2002-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -76,7 +76,7 @@ static SV **modperl_module_config_hash_get(pTHX_ int create)
 void modperl_module_config_table_set(pTHX_ PTR_TBL_t *table)
 {
     SV **svp = modperl_module_config_hash_get(aTHX_ TRUE);
-    sv_setiv(*svp, (IV)table);
+    sv_setiv(*svp, PTR2IV(table));
 }
 
 PTR_TBL_t *modperl_module_config_table_get(pTHX_ int create)
@@ -84,7 +84,7 @@ PTR_TBL_t *modperl_module_config_table_get(pTHX_ int create)
     PTR_TBL_t *table = NULL;
 
     SV *sv, **svp = modperl_module_config_hash_get(aTHX_ create);
-    
+
     if (!svp) {
         return NULL;
     }
@@ -92,10 +92,10 @@ PTR_TBL_t *modperl_module_config_table_get(pTHX_ int create)
     sv = *svp;
     if (!SvIOK(sv) && create) {
         table = modperl_svptr_table_new(aTHX);
-        sv_setiv(sv, (IV)table);
+        sv_setiv(sv, PTR2IV(table));
     }
     else {
-        table = (PTR_TBL_t *)SvIV(sv);
+        table = INT2PTR(PTR_TBL_t *, SvIV(sv));
     }
 
     return table;
@@ -165,12 +165,11 @@ static void *modperl_module_config_merge(apr_pool_t *p,
     int is_startup;
     PTR_TBL_t *table;
     SV *mrg_obj = Nullsv, *base_obj, *add_obj;
-
 #ifdef USE_ITHREADS
     modperl_interp_t *interp;
-    pTHX;
+    MP_PERL_CONTEXT_DECLARE;
 #endif
-    
+
     /* if the module is loaded in vhost, base==NULL */
     tmp = (base && base->server) ? base : add;
 
@@ -178,13 +177,13 @@ static void *modperl_module_config_merge(apr_pool_t *p,
         /* no directives for this module were encountered so far */
         return basev;
     }
-    
+
     s = tmp->server;
     is_startup = (p == s->process->pconf);
 
 #ifdef USE_ITHREADS
     interp = modperl_interp_pool_select(p, s);
-    aTHX = interp->perl;
+    MP_PERL_CONTEXT_STORE_OVERRIDE(interp->perl);
 #endif
 
     table = modperl_module_config_table_get(aTHX_ TRUE);
@@ -192,6 +191,11 @@ static void *modperl_module_config_merge(apr_pool_t *p,
     add_obj  = modperl_svptr_table_fetch(aTHX_ table, add);
 
     if (!base_obj || (base_obj == add_obj)) {
+#ifdef USE_ITHREADS
+        /* XXX: breaks prefork
+           modperl_interp_unselect(interp); */
+        MP_PERL_CONTEXT_RESTORE;
+#endif
         return addv;
     }
 
@@ -240,6 +244,12 @@ static void *modperl_module_config_merge(apr_pool_t *p,
         modperl_module_config_obj_cleanup_register(aTHX_ p, table, mrg);
     }
 
+#ifdef USE_ITHREADS
+    /* XXX: breaks prefork
+       modperl_interp_unselect(interp); */
+    MP_PERL_CONTEXT_RESTORE;
+#endif
+    
     return (void *)mrg;
 }
 
@@ -258,7 +268,7 @@ static void *modperl_module_config_srv_merge(apr_pool_t *p,
 }
 
 #define modperl_bless_cmd_parms(parms) \
-    sv_2mortal(modperl_ptr2obj(aTHX_ "Apache::CmdParms", (void *)parms))
+    sv_2mortal(modperl_ptr2obj(aTHX_ "Apache2::CmdParms", (void *)parms))
 
 static const char *
 modperl_module_config_create_obj(pTHX_
@@ -362,7 +372,7 @@ static const char *modperl_module_cmd_take123(cmd_parms *parms,
     PTR_TBL_t *table = modperl_module_config_table_get(aTHX_ TRUE);
     SV *obj = Nullsv;
     dSP;
-    
+
     if (s->is_virtual) {
         MP_dSCFG(s);
 
@@ -385,7 +395,7 @@ static const char *modperl_module_cmd_take123(cmd_parms *parms,
         /* if PerlLoadModule Foo is called from the base server, but
          * Foo's directives are used inside a vhost, we need to
          * temporary link to the base server config's 'modules'
-         * member. e.g. so Apache::Module->get_config() can be called
+         * member. e.g. so Apache2::Module->get_config() can be called
          * from a custom directive's callback, before the server/vhost
          * config merge is performed
          */
@@ -397,9 +407,9 @@ static const char *modperl_module_cmd_take123(cmd_parms *parms,
                 modules_alias = 1;
             }
         }
-        
+
     }
-    
+
     errmsg = modperl_module_config_create_obj(aTHX_ p, table, cfg, info,
                                               minfo->dir_create,
                                               parms, &obj);
@@ -415,7 +425,7 @@ static const char *modperl_module_cmd_take123(cmd_parms *parms,
     }
 
     /* XXX: could delay creation of srv_obj until
-     * Apache::ModuleConfig->get is called.
+     * Apache2::ModuleConfig->get is called.
      */
     srv_cfg = ap_get_module_config(s->module_config, info->modp);
 
@@ -566,13 +576,6 @@ static apr_status_t modperl_module_remove(void *data)
     return APR_SUCCESS;
 }
 
-static AV *modperl_module_cmds_get(pTHX_ module *modp)
-{
-    char *name = Perl_form(aTHX_ "%s::%s", modp->name,
-                           "APACHE_MODULE_COMMANDS");
-    return get_av(name, FALSE);
-}
-
 static const char *modperl_module_cmd_fetch(pTHX_ SV *obj,
                                             const char *name, SV **retval)
 {
@@ -609,7 +612,7 @@ static const char *modperl_module_cmd_fetch(pTHX_ SV *obj,
 
         PUTBACK;
         FREETMPS;LEAVE;
-        
+
         if (SvTRUE(ERRSV)) {
             errmsg = SvPVX(ERRSV);
         }
@@ -633,7 +636,7 @@ static const char *modperl_module_cmd_fetch(pTHX_ SV *obj,
 }
 
 static const char *modperl_module_add_cmds(apr_pool_t *p, server_rec *s,
-                                           module *modp)
+                                           module *modp, SV *mod_cmds)
 {
     const char *errmsg;
     apr_array_header_t *cmds;
@@ -643,12 +646,8 @@ static const char *modperl_module_add_cmds(apr_pool_t *p, server_rec *s,
 #ifdef USE_ITHREADS
     MP_dSCFG(s);
     dTHXa(scfg->mip->parent->perl);
-#endif
-
-    if (!(module_cmds = modperl_module_cmds_get(aTHX_ modp))) {
-        return apr_pstrcat(p, "module ", modp->name,
-                           " does not define @APACHE_MODULE_COMMANDS", NULL);
-    }
+#endif 
+    module_cmds = (AV*)SvRV(mod_cmds);
 
     fill = AvFILL(module_cmds);
     cmds = apr_array_make(p, fill+1, sizeof(command_rec));
@@ -679,7 +678,7 @@ static const char *modperl_module_add_cmds(apr_pool_t *p, server_rec *s,
             }
             else {
                 cmd->args_how =
-                    SvIV(modperl_constants_lookup_apache(aTHX_ SvPV(val, len)));
+                    SvIV(modperl_constants_lookup_apache2_const(aTHX_ SvPV(val, len)));
             }
         }
 
@@ -705,7 +704,7 @@ static const char *modperl_module_add_cmds(apr_pool_t *p, server_rec *s,
             }
             else {
                 cmd->req_override =
-                    SvIV(modperl_constants_lookup_apache(aTHX_ SvPV(val, len)));
+                    SvIV(modperl_constants_lookup_apache2_const(aTHX_ SvPV(val, len)));
             }
         }
 
@@ -788,7 +787,7 @@ static modperl_mgv_t *modperl_module_fetch_method(pTHX_
 }
 
 const char *modperl_module_add(apr_pool_t *p, server_rec *s,
-                               const char *name)
+                               const char *name, SV *mod_cmds)
 {
     MP_dSCFG(s);
 #ifdef USE_ITHREADS
@@ -834,7 +833,7 @@ const char *modperl_module_add(apr_pool_t *p, server_rec *s,
 
     modp->cmds = NULL;
 
-    if ((errmsg = modperl_module_add_cmds(p, s, modp))) {
+    if ((errmsg = modperl_module_add_cmds(p, s, modp, mod_cmds))) {
         return errmsg;
     }
 
@@ -851,7 +850,7 @@ const char *modperl_module_add(apr_pool_t *p, server_rec *s,
         scfg->modules = apr_hash_make(p);
     }
 
-    apr_hash_set(scfg->modules, name, APR_HASH_KEY_STRING, modp);
+    apr_hash_set(scfg->modules, apr_pstrdup(p, name), APR_HASH_KEY_STRING, modp);
 
 #ifdef USE_ITHREADS
     /* 
