@@ -1,4 +1,4 @@
-/* Copyright 2002-2004 The Apache Software Foundation
+/* Copyright 2002-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@ PTR_TBL_t *modperl_module_config_table_get(pTHX_ int create);
 void modperl_module_config_table_set(pTHX_ PTR_TBL_t *table);
 
 const char *modperl_module_add(apr_pool_t *p, server_rec *s,
-                               const char *name);
+                               const char *name, SV *mod_cmds);
 
 SV *modperl_module_config_get_obj(pTHX_ SV *pmodule, server_rec *s, 
                                   ap_conf_vector_t *v);
