@@ -1,4 +1,4 @@
-/* Copyright 2000-2004 The Apache Software Foundation
+/* Copyright 2000-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -130,7 +130,7 @@ typedef struct {
     MpHV *configvars;
     MpHV *SetEnv;
     MpHV *PassEnv;
-    MpAV *PerlRequire, *PerlModule;
+    MpAV *PerlRequire, *PerlModule, *PerlPostConfigRequire;
     MpAV *handlers_per_srv[MP_HANDLER_NUM_PER_SRV];
     MpAV *handlers_files[MP_HANDLER_NUM_FILES];
     MpAV *handlers_process[MP_HANDLER_NUM_PROCESS];
@@ -165,6 +165,11 @@ typedef struct {
 #endif
 } modperl_config_dir_t;
 
+typedef struct {
+    const char *file;
+    modperl_config_dir_t *dcfg;
+} modperl_require_file_t;
+
 typedef struct modperl_mgv_t modperl_mgv_t;
 
 struct modperl_mgv_t {
@@ -176,13 +181,16 @@ struct modperl_mgv_t {
 
 typedef struct modperl_handler_t modperl_handler_t;
 
-struct modperl_handler_t{
+struct modperl_handler_t {
+    /* could be:
+     * - the lightweight gv for named subs
+     * - the lookup data in $PL_modperl{ANONSUB}
+     */
     modperl_mgv_t *mgv_obj;
     modperl_mgv_t *mgv_cv;
     /* could be:
-     * - a subroutine name
-     * - a subroutine source code as a string (anon subs)
-     * - NULL, when .cv is set (anon subs)
+     * - a subroutine name for named subs
+     * - NULL for anon subs
      */
     const char *name; 
     CV *cv;
