@@ -6,12 +6,14 @@ package TestFilter::both_str_req_add;
 use strict;
 use warnings FATAL => 'all';
 
-use Apache::RequestRec ();
-use Apache::RequestIO ();
+use Apache2::RequestRec ();
+use Apache2::RequestIO ();
 
-use Apache::Filter ();
+use Apache2::Filter ();
 
-use Apache::Const -compile => qw(OK M_POST);
+use TestCommon::Utils ();
+
+use Apache2::Const -compile => qw(OK M_POST);
 
 sub header_parser {
     my $r = shift;
@@ -20,7 +22,19 @@ sub header_parser {
     # test adding by sub's name
     $r->add_output_filter("out_filter");
 
-    return Apache::DECLINED;
+    # test adding anon sub
+    $r->add_output_filter(sub {
+        my $filter = shift;
+
+        while ($filter->read(my $buffer, 1024)) {
+            $buffer .= "end";
+            $filter->print($buffer);
+        }
+
+        return Apache2::Const::OK;
+    });
+
+    return Apache2::Const::DECLINED;
 }
 
 sub in_filter {
@@ -30,7 +44,7 @@ sub in_filter {
         $filter->print(lc $buffer);
     }
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 sub out_filter {
@@ -41,7 +55,7 @@ sub out_filter {
         $filter->print($buffer);
     }
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 sub handler {
@@ -49,11 +63,11 @@ sub handler {
 
     $r->content_type('text/plain');
 
-    if ($r->method_number == Apache::M_POST) {
-        $r->print(ModPerl::Test::read_post($r));
+    if ($r->method_number == Apache2::Const::M_POST) {
+        $r->print(TestCommon::Utils::read_post($r));
     }
 
-    return Apache::OK;
+    return Apache2::Const::OK;
 }
 
 
