@@ -3,14 +3,16 @@ package TestFilter::in_bbs_body;
 use strict;
 use warnings FATAL => 'all';
 
-use base qw(Apache::Filter); #so we inherit MODIFY_CODE_ATTRIBUTES
+use base qw(Apache2::Filter); #so we inherit MODIFY_CODE_ATTRIBUTES
 
-use Apache::RequestRec ();
-use Apache::RequestIO ();
+use Apache2::RequestRec ();
+use Apache2::RequestIO ();
 use APR::Brigade ();
 use APR::Bucket ();
 
-use Apache::Const -compile => qw(OK M_POST);
+use TestCommon::Utils ();
+
+use Apache2::Const -compile => qw(OK M_POST);
 use APR::Const -compile => ':common';
 
 sub handler : FilterRequestHandler {
@@ -31,7 +33,7 @@ sub handler : FilterRequestHandler {
         }
     }
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 sub response {
@@ -39,15 +41,15 @@ sub response {
 
     $r->content_type('text/plain');
 
-    if ($r->method_number == Apache::M_POST) {
-        my $data = ModPerl::Test::read_post($r);
+    if ($r->method_number == Apache2::Const::M_POST) {
+        my $data = TestCommon::Utils::read_post($r);
         $r->puts($data);
     }
     else {
-        $r->puts("1..1\nok 1\n");
+        $r->puts("1..3\nok 1\n");
     }
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 1;
