@@ -5,14 +5,14 @@ use warnings FATAL => 'all';
 
 use Apache::Test;
 
-use Apache::RequestRec ();
-use Apache::RequestIO ();
-use Apache::Filter ();
+use Apache2::RequestRec ();
+use Apache2::RequestIO ();
+use Apache2::Filter ();
 use APR::Brigade ();
 use APR::Bucket ();
 use APR::BucketType ();
 
-use Apache::Const -compile => 'OK';
+use Apache2::Const -compile => 'OK';
 
 #XXX: Not implemented yet, required by Test.pm
 sub Apache::TestToString::PRINTF {}
@@ -39,12 +39,12 @@ sub handler {
         my $tests = Apache::TestToString->finish;
 
         my $brigade = APR::Brigade->new($filter->r->pool, $ba);
-        my $b = APR::Bucket->new($tests);
+        my $b = APR::Bucket->new($ba, $tests);
 
         $brigade->insert_tail($b);
 
         my $ok = $brigade->first->type->name =~ /mod_perl/ ? 4 : 0;
-        $brigade->insert_tail(APR::Bucket->new("ok $ok\n"));
+        $brigade->insert_tail(APR::Bucket->new($ba, "ok $ok\n"));
 
         $brigade->insert_tail(APR::Bucket::eos_create($ba));
 
@@ -53,7 +53,7 @@ sub handler {
         $filter->ctx(1); # flag that we have run this already
     }
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 sub response {
