@@ -6,18 +6,18 @@ package TestFilter::out_bbs_ctx;
 use strict;
 use warnings FATAL => 'all';
 
-use Apache::RequestRec ();
-use Apache::RequestIO ();
+use Apache2::RequestRec ();
+use Apache2::RequestIO ();
 
 use APR::Brigade ();
 use APR::Bucket ();
 use APR::BucketType ();
 
-use base qw(Apache::Filter);
+use base qw(Apache2::Filter);
 
 use Apache::TestTrace;
 
-use Apache::Const -compile => qw(OK M_POST);
+use Apache2::Const -compile => qw(OK M_POST);
 use APR::Const -compile => ':common';
 
 use constant BLOCK_SIZE => 5003;
@@ -28,7 +28,8 @@ sub handler {
     debug "filter got called";
 
     my $c = $filter->c;
-    my $bb_ctx = APR::Brigade->new($c->pool, $c->bucket_alloc);
+    my $ba = $c->bucket_alloc;
+    my $bb_ctx = APR::Brigade->new($c->pool, $ba);
 
     my $ctx = $filter->ctx;
     $ctx->{invoked}++;
@@ -40,10 +41,10 @@ sub handler {
         if ($b->is_eos) {
             debug "got EOS";
             # flush the remainings and send a stats signature
-            $bb_ctx->insert_tail(APR::Bucket->new("$data\n")) if $data;
+            $bb_ctx->insert_tail(APR::Bucket->new($ba, "$data\n")) if $data;
             my $sig = join "\n", "received $ctx->{blocks} complete blocks",
                 "filter invoked $ctx->{invoked} times\n";
-            $bb_ctx->insert_tail(APR::Bucket->new($sig));
+            $bb_ctx->insert_tail(APR::Bucket->new($ba, $sig));
             $b->remove;
             $bb_ctx->insert_tail($b);
             last;
@@ -63,7 +64,7 @@ sub handler {
                 $ctx->{blocks} += $blocks;
             }
             if ($blocks) {
-                my $nb = APR::Bucket->new("#" x $blocks);
+                my $nb = APR::Bucket->new($ba, "#" x $blocks);
                 $bb_ctx->insert_tail($nb);
             }
         }
@@ -79,9 +80,9 @@ sub handler {
     $filter->ctx($ctx);
 
     my $rv = $filter->next->pass_brigade($bb_ctx);
-    return $rv unless $rv == APR::SUCCESS;
+    return $rv unless $rv == APR::Const::SUCCESS;
 
-    return Apache::OK;
+    return Apache2::Const::OK;
 }
 
 sub response {
@@ -106,7 +107,7 @@ sub response {
         $r->rflush; # so the filter reads a chunk at a time
     }
 
-    return Apache::OK;
+    return Apache2::Const::OK;
 }
 
 1;
