@@ -3,22 +3,24 @@ package TestHooks::authz;
 use strict;
 use warnings FATAL => 'all';
 
-use Apache::Access ();
+use Apache2::Access ();
 
-use Apache::Const -compile => qw(OK HTTP_UNAUTHORIZED);
+use Apache2::Const -compile => qw(OK HTTP_UNAUTHORIZED);
 
 sub auth_any {
     my $r = shift;
 
     my($res, $sent_pw) = $r->get_basic_auth_pw;
-    return $res if $res != Apache::OK;
+    return $res if $res != Apache2::Const::OK;
 
     unless($r->user and $sent_pw) {
-	$r->note_basic_auth_failure;
-	return Apache::HTTP_UNAUTHORIZED;
+        # testing $r->note_auth_failure:
+        # AuthType Basic + note_auth_failure == note_basic_auth_failure;
+	$r->note_auth_failure;
+	return Apache2::Const::HTTP_UNAUTHORIZED;
     }
 
-    return Apache::OK;
+    return Apache2::Const::OK;
 }
 
 sub handler {
@@ -26,13 +28,13 @@ sub handler {
 
     my $user = $r->user;
 
-    return Apache::HTTP_UNAUTHORIZED unless $user;
+    return Apache2::Const::HTTP_UNAUTHORIZED unless $user;
 
     my($u, @allowed) = split /\s+/, $r->requires->[0]->{requirement};
 
-    return Apache::HTTP_UNAUTHORIZED unless grep { $_ eq $user } @allowed;
+    return Apache2::Const::HTTP_UNAUTHORIZED unless grep { $_ eq $user } @allowed;
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 1;
