@@ -7,40 +7,42 @@ package TestHooks::stacked_handlers2;
 use strict;
 use warnings FATAL => 'all';
 
-use Apache::RequestRec ();
-use Apache::RequestIO ();
-use Apache::Filter ();
+use Apache2::RequestRec ();
+use Apache2::RequestIO ();
+use Apache2::Filter ();
+
+use ModPerl::Util ();
 
 use APR::Table;
 
-use Apache::Const -compile => qw(OK DECLINED AUTH_REQUIRED SERVER_ERROR);
+use Apache2::Const -compile => qw(OK DECLINED AUTH_REQUIRED SERVER_ERROR);
 
 sub ok { 
 
     callback(shift);
 
-    return Apache::OK;
+    return Apache2::Const::OK;
 }
 
 sub declined { 
 
     callback(shift);
 
-    return Apache::DECLINED;
+    return Apache2::Const::DECLINED;
 }
 
 sub auth_required { 
 
     callback(shift);
 
-    return Apache::AUTH_REQUIRED;
+    return Apache2::Const::AUTH_REQUIRED;
 }
 
 sub server_error { 
 
     callback(shift);
 
-    return Apache::SERVER_ERROR;
+    return Apache2::Const::SERVER_ERROR;
 }
 
 sub callback {
@@ -49,7 +51,7 @@ sub callback {
 
     my ($r, $callback);
 
-    if ($obj->isa('Apache::Filter')) {
+    if ($obj->isa('Apache2::Filter')) {
         $r = $obj->r;
         $callback = 'PerlOutputFilterHandler';
     }
@@ -57,7 +59,7 @@ sub callback {
         $r = $obj
     }
 
-    $callback ||= Apache::current_callback;
+    $callback ||= ModPerl::Util::current_callback;
 
     my $count = $r->notes->get($callback) || 0;
 
@@ -84,11 +86,11 @@ sub handler {
                              PerlResponseHandler)) {
 
         my $count = $r->notes->get($callback) || 0;
- 
+
         $r->print("ran $count $callback handlers\n");
     }
 
-    return Apache::OK;
+    return Apache2::Const::OK;
 }
 
 sub passthru {
@@ -104,7 +106,7 @@ sub passthru {
     }
 
     # this should be ignored?
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 sub filter {
@@ -126,7 +128,7 @@ sub filter {
     }
 
     # this should be ignored?
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 1;
@@ -150,32 +152,32 @@ __DATA__
         # all 4 run
         PerlHeaderParserHandler TestHooks::stacked_handlers2::ok TestHooks::stacked_handlers2::declined
         PerlHeaderParserHandler TestHooks::stacked_handlers2::declined TestHooks::stacked_handlers2::ok
-  
+
         # all 2 run
         PerlAccessHandler TestHooks::stacked_handlers2::ok TestHooks::stacked_handlers2::ok 
-  
+
         # 2 run, 1 left behind
         PerlAuthenHandler TestHooks::stacked_handlers2::declined TestHooks::stacked_handlers2::ok 
         PerlAuthenHandler TestHooks::stacked_handlers2::auth_required
-  
+
         # 2 run, 1 left behind
         PerlAuthzHandler TestHooks::stacked_handlers2::declined TestHooks::stacked_handlers2::ok 
         PerlAuthzHandler TestHooks::stacked_handlers2::auth_required
-  
+
         # 1 run, 1 left behind
         PerlTypeHandler  TestHooks::stacked_handlers2::ok TestHooks::stacked_handlers3::server_error
-  
+
         # all 2 run
         PerlFixupHandler TestHooks::stacked_handlers2::ok TestHooks::stacked_handlers2::ok
-  
+
         # 2 run, 2 left behind
         PerlResponseHandler TestHooks::stacked_handlers2::declined TestHooks::stacked_handlers2 
         PerlResponseHandler TestHooks::stacked_handlers2::ok TestHooks::stacked_handlers2::server_error
-  
+
         SetHandler modperl
         AuthType Basic
         Require valid-user
-  
+
         PerlOutputFilterHandler TestHooks::stacked_handlers2::passthru TestHooks::stacked_handlers2::filter
     </Location>
 
