@@ -3,17 +3,20 @@ package TestAPI::lookup_uri2;
 use strict;
 use warnings FATAL => 'all';
 
-use Apache::SubRequest ();
-use Apache::RequestIO ();
+use Apache2::SubRequest ();
+use Apache2::RequestIO ();
+use Apache2::RequestUtil ();
 
-use Apache::Const -compile => 'OK';
+use Apache2::Const -compile => 'OK';
 
 sub myplan {
     my $r = shift;
 
     $r->puts("1..3\nok 1\n");
 
-    Apache::OK;
+    die "must indicate a sub-request" if $r->is_initial_req();
+
+    Apache2::Const::OK;
 }
 
 sub ok3 {
@@ -21,7 +24,7 @@ sub ok3 {
 
     $r->puts("ok 3\n");
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 sub subrequest {
@@ -39,7 +42,7 @@ sub handler {
 
     subrequest($r, 'ok3');
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 1;
