@@ -5,10 +5,10 @@ package TestAPI::status;
 use strict;
 use warnings FATAL => 'all';
 
-use Apache::RequestRec ();
-use Apache::RequestIO ();
+use Apache2::RequestRec ();
+use Apache2::RequestIO ();
 
-use Apache::Const -compile => 'OK';
+use Apache2::Const -compile => 'OK';
 
 my $body = "This is a response string";
 
@@ -27,7 +27,7 @@ sub handler {
         $r->status($code);
     }
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 1;
