@@ -7,17 +7,17 @@ use Apache::Test;
 use Apache::TestUtil;
 use Apache::TestConfig ();
 
-use Apache::Directive ();
+use Apache2::Directive ();
 
-use Apache::Const -compile => 'OK';
+use Apache2::Const -compile => 'OK';
 
 sub handler {
     my $r = shift;
 
     my $cfg = Apache::Test::config();
-    
+
     my $node_count = node_count();
-    
+
     plan $r, tests => 8 + (5*$node_count);
 
     ok $cfg;
@@ -26,7 +26,7 @@ sub handler {
 
     ok $vars;
 
-    my $tree = Apache::Directive::conftree();
+    my $tree = Apache2::Directive::conftree();
 
     ok $tree;
 
@@ -63,10 +63,10 @@ sub handler {
 
         ok !$vhost_failed;
     }
-    
+
     traverse_tree ( \&test_node );
-    
-    Apache::OK;
+
+    Apache2::Const::OK;
 }
 
 sub test_node {
@@ -82,7 +82,7 @@ sub test_node {
 
 sub traverse_tree {
     my ($sub, $data) = @_;
-    my $node = Apache::Directive::conftree();
+    my $node = Apache2::Directive::conftree();
     while ($node) {
         $sub->($data, $node);
         if (my $kid = $node->first_child) {
@@ -105,9 +105,9 @@ sub traverse_tree {
 
 sub node_count {
     my $node_count = 0;
-    
+
     traverse_tree( sub { ${$_[0]}++ }, \$node_count );
-    
+
     return $node_count;
 }
 1;
