@@ -10,7 +10,7 @@ use warnings FATAL => 'all';
 use Apache::TestUtil;
 use Apache::Test;
 
-use Apache::compat ();
+use Apache2::compat ();
 use Apache::Constants qw(OK);
 
 my %string_size = (
@@ -52,45 +52,46 @@ sub handler {
     (my $esc_uri = $uri) =~ s/ /\%20/g;
     my $uri2 = $uri;
 
-    $uri = Apache::Util::escape_uri($uri);
+    $uri  = Apache::Util::escape_uri($uri);
     $uri2 = Apache::Util::escape_path($uri2, $r->pool);
 
-    ok t_cmp($esc_uri, $uri, "Apache::Util::escape_uri");
-    ok t_cmp($esc_uri, $uri2, "Apache::Util::escape_path");
+    ok t_cmp($uri, $esc_uri, "Apache::Util::escape_uri");
+    ok t_cmp($uri2, $esc_uri, "Apache::Util::escape_path");
 
-    ok t_cmp(Apache::unescape_url($uri),
-             Apache::Util::unescape_uri($uri2),
-             "Apache::URI::unescape_uri vs Apache::Util::unescape_uri");
+    ok t_cmp(Apache::Util::unescape_uri($uri2),
+             Apache2::URI::unescape_url($uri),
+             "Apache2::URI::unescape_uri vs Apache::Util::unescape_uri");
 
-    ok t_cmp($uri,
-             $uri2,
-             "Apache::URI::unescape_uri vs Apache::Util::unescape_uri");
+    ok t_cmp($uri2,
+             $uri,
+             "Apache2::URI::unescape_uri vs Apache::Util::unescape_uri");
 
     # escape_html()
     my $html = '<p>"hi"&foo</p>';
     my $esc_html = '&lt;p&gt;&quot;hi&quot;&amp;foo&lt;/p&gt;';
 
-    ok t_cmp($esc_html, Apache::Util::escape_html($html),
-             "Apache::Util::escape_html");
+    ok t_cmp(Apache::Util::escape_html($html), $esc_html,
+             "Apache2::Util::escape_html");
 
 
     # ht_time(), parsedate()
     my $time = time;
-    
-    my $fmtdate = Apache::Util::ht_time($time);
+    Apache2::compat::override_mp2_api('Apache2::Util::ht_time');
+    my $fmtdate = Apache2::Util::ht_time($time);
+    Apache2::compat::restore_mp2_api('Apache2::Util::ht_time');
 
     ok t_cmp($fmtdate, $fmtdate, "Apache::Util::ht_time");
 
     if ($parse_time_ok) {
         my $ptime = Apache::Util::parsedate($fmtdate);
-        ok t_cmp($time, $ptime, "Apache::Util::parsedate");
+        ok t_cmp($ptime, $time, "Apache::Util::parsedate");
     }
 
     if ($crypt_ok) {
         # not all platforms support C-level crypt
         my $hash = "aX9eP53k4DGfU";
-        ok t_cmp(1, Apache::Util::validate_password("dougm", $hash));
-        ok t_cmp(0, Apache::Util::validate_password("mguod", $hash));
+        ok t_cmp(Apache::Util::validate_password("dougm", $hash), 1);
+        ok t_cmp(Apache::Util::validate_password("mguod", $hash), 0);
     }
 
     OK;
