@@ -3,13 +3,13 @@ package TestDirective::cmdparms;
 use strict;
 use warnings FATAL => 'all';
 
-use Apache::CmdParms ();
-use base qw(Apache::Module);
+use Apache2::CmdParms ();
+use base qw(Apache2::Module);
 
 use Apache::Test;
 use Apache::TestUtil;
 
-use Apache::Const -compile => qw(
+use Apache2::Const -compile => qw(
     ACCESS_CONF
     M_GET
     M_POST
@@ -27,7 +27,7 @@ use Apache::Const -compile => qw(
 
 use constant KEY => "TestCmdParms";
 
-our @APACHE_MODULE_COMMANDS = (
+my @directives = (
     {
         name        => +KEY,
         cmd_data    => 'cmd_data',
@@ -35,6 +35,8 @@ our @APACHE_MODULE_COMMANDS = (
     },
 );
 
+Apache2::Module::add(__PACKAGE__, \@directives);
+
 my @methods = qw(cmd context directive info override path
                  pool server temp_pool);
 
@@ -45,8 +47,8 @@ sub TestCmdParms {
         $srv_cfg->{$args}{$method} = $parms->$method();
     }
     $srv_cfg->{$args}{check_ctx} = 
-        $parms->check_cmd_context(Apache::NOT_IN_LOCATION);
-    
+        $parms->check_cmd_context(Apache2::Const::NOT_IN_LOCATION);
+
     $srv_cfg->{$args}{limited} = $parms->method_is_limited('GET');    
 }
 
@@ -59,12 +61,12 @@ sub handler : method {
     plan $r, tests => 9 + ( 7 * keys(%$srv_cfg) );
 
     foreach my $cfg (values %$srv_cfg) {
-        ok t_cmp(ref($cfg->{cmd}), 'Apache::Command', 'cmd');
-        ok t_cmp(ref($cfg->{context}), 'Apache::ConfVector', 'context');
-        ok t_cmp(ref($cfg->{directive}), 'Apache::Directive', 'directive');
+        ok t_cmp(ref($cfg->{cmd}), 'Apache2::Command', 'cmd');
+        ok t_cmp(ref($cfg->{context}), 'Apache2::ConfVector', 'context');
+        ok t_cmp(ref($cfg->{directive}), 'Apache2::Directive', 'directive');
         ok t_cmp(ref($cfg->{pool}), 'APR::Pool', 'pool');
         ok t_cmp(ref($cfg->{temp_pool}), 'APR::Pool', 'temp_pool');
-        ok t_cmp(ref($cfg->{server}), 'Apache::ServerRec', 'server');
+        ok t_cmp(ref($cfg->{server}), 'Apache2::ServerRec', 'server');
         ok t_cmp($cfg->{info}, 'cmd_data', 'cmd_data');
     }
 
@@ -72,10 +74,10 @@ sub handler : method {
     {
         my $vhost = $srv_cfg->{Vhost};
 
-        my $wanted = Apache::RSRC_CONF   |
-                     Apache::OR_INDEXES  |
-                     Apache::OR_FILEINFO |
-                     Apache::OR_OPTIONS;
+        my $wanted = Apache2::Const::RSRC_CONF   |
+                     Apache2::Const::OR_INDEXES  |
+                     Apache2::Const::OR_FILEINFO |
+                     Apache2::Const::OR_OPTIONS;
         my $masked = $vhost->{override} & $wanted;
 
         ok t_cmp($masked, $wanted, 'override bitmask');
@@ -88,12 +90,12 @@ sub handler : method {
     {
         my $loc = $srv_cfg->{Location};
 
-        my $wanted = Apache::ACCESS_CONF |
-                     Apache::OR_INDEXES  |
-                     Apache::OR_AUTHCFG  |
-                     Apache::OR_FILEINFO |
-                     Apache::OR_OPTIONS  |
-                     Apache::OR_LIMIT;
+        my $wanted = Apache2::Const::ACCESS_CONF |
+                     Apache2::Const::OR_INDEXES  |
+                     Apache2::Const::OR_AUTHCFG  |
+                     Apache2::Const::OR_FILEINFO |
+                     Apache2::Const::OR_OPTIONS  |
+                     Apache2::Const::OR_LIMIT;
         my $masked = $loc->{override} & $wanted;
 
         ok t_cmp($masked, $wanted, 'override bitmask');
@@ -102,14 +104,14 @@ sub handler : method {
                   ' cannot occur within <Location> section', 'check_cmd_ctx');
         ok $loc->{limited};
     }
-    
+
     # Limit
     {
         my $limit = $srv_cfg->{Limit};
         ok !$limit->{limited};
     }
 
-    return Apache::OK;
+    return Apache2::Const::OK;
 }
 
 1;
