@@ -5,19 +5,20 @@ use warnings FATAL => 'all';
 
 use Apache::Test;
 use Apache::TestUtil;
+use Apache::TestTrace;
 
-use Apache::Const -compile => qw(OK OR_ALL RSRC_CONF TAKE1 TAKE23);
+use Apache2::Const -compile => qw(OK OR_ALL RSRC_CONF TAKE1 TAKE23);
 
-use Apache::CmdParms ();
-use Apache::Module ();
+use Apache2::CmdParms ();
+use Apache2::Module ();
 
-our @APACHE_MODULE_COMMANDS = (
+my @directives = (
     {
      name => 'MyTest',
      func => __PACKAGE__ . '::MyTest',
-     req_override => Apache::RSRC_CONF,
+     req_override => Apache2::Const::RSRC_CONF,
 #     req_override => 'RSRC_CONF', #test 1.x compat for strings
-#     args_how => Apache::TAKE23,
+#     args_how => Apache2::Const::TAKE23,
      args_how => 'TAKE23', #test 1.x compat for strings
      errmsg => 'A test',
     },
@@ -27,10 +28,12 @@ our @APACHE_MODULE_COMMANDS = (
     },
     {
      name => 'ServerTest',
-     req_override => Apache::RSRC_CONF,
+     req_override => Apache2::Const::RSRC_CONF,
     }
 );
 
+Apache2::Module::add(__PACKAGE__, \@directives);
+
 sub DIR_CREATE {
     my($class, $parms) = @_;
 
@@ -52,19 +55,19 @@ sub merge {
 
 sub DIR_MERGE {
     my $class = ref $_[0];
-#    warn "$class->DIR_MERGE\n";
+    debug "$class->DIR_MERGE\n";
     merge(@_);
 }
 
 #sub SERVER_MERGE {
 #    my $class = ref $_[0];
-#    warn "$class->SERVER_MERGE\n";
+#    debug "$class->SERVER_MERGE\n";
 #    merge(@_);
 #}
 
 sub SERVER_CREATE {
     my($class, $parms) = @_;
-#    warn "$class->SERVER_CREATE\n";
+    debug "$class->SERVER_CREATE\n";
     return bless {
 	name => __PACKAGE__,
     }, $class;
@@ -90,7 +93,7 @@ sub ServerTest {
 
 sub get_config {
     my($self, $s) = (shift, shift);
-    Apache::Module->get_config($self, $s, @_);
+    Apache2::Module::get_config($self, $s, @_);
 }
 
 sub handler : method {
@@ -129,7 +132,7 @@ sub handler : method {
 
     ok t_cmp($srv_cfg->{ServerTest}, 'per-server');
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 1;
