@@ -3,7 +3,10 @@ package TestDirective::setupenv;
 use strict;
 use warnings FATAL => 'all';
 
-use Apache::Const -compile => 'OK';
+use Apache2::RequestRec ();
+use Apache2::RequestIO ();
+
+use Apache2::Const -compile => 'OK';
 
 sub handler {
     my $r = shift;
@@ -15,7 +18,7 @@ sub handler {
         $r->puts("$key=$val\n");
     }
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 1;
