@@ -8,22 +8,22 @@ use Apache::TestUtil;
 use ModPerl::Util;
 
 use APR::Table ();
-use Apache::RequestRec ();
+use Apache2::RequestRec ();
 
-use Apache::Const -compile => qw(OK DECLINED);
+use Apache2::Const -compile => qw(OK DECLINED);
 
 sub handler {
     my $r = shift;
 
     plan $r, tests => 1;
-    my $callback = Apache::current_callback();
+    my $callback = ModPerl::Util::current_callback();
     ok t_cmp($callback,
              'PerlResponseHandler',
              'inside PerlResponseHandler');
 
     #warn "in callback: $callback\n";
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 sub log          { check('Log')          }
@@ -32,11 +32,11 @@ sub headerparser { check('HeaderParser') }
 
 sub check {
     my $expected = 'Perl' . shift() . 'Handler';
-    my $callback = Apache::current_callback();
+    my $callback = ModPerl::Util::current_callback();
     die "expecting $expected callback, instead got $callback" 
         unless $callback eq $expected;
     #warn "in callback: $callback\n";
-    return Apache::OK;
+    return Apache2::Const::OK;
 }
 
 1;
