@@ -7,7 +7,7 @@ use Apache::Test;
 use Apache::TestUtil;
 use Apache::TestRequest;
 
-use Apache::Const -compile => 'OK';
+use Apache2::Const -compile => 'OK';
 
 sub new {
     my $class = shift;
@@ -30,7 +30,7 @@ sub handler : method {
 
     plan $r, tests => $tests;
 
-    ok t_cmp(2, scalar @_,
+    ok t_cmp(scalar @_, 2,
              '@_ == 2');
 
     my $class = ref($self) || $self;
@@ -39,16 +39,16 @@ sub handler : method {
              'handler class');
 
     ok t_cmp(
-        '/' . Apache::TestRequest::module2path($class), 
         $r->uri,
+        '/' . Apache::TestRequest::module2path($class), 
         '$r->uri eq $location');
 
     if ($is_obj) {
-        ok t_cmp($], $self->{perl_version},
+        ok t_cmp($self->{perl_version}, $],
                  'object handler data');
     }
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 1;
