@@ -3,9 +3,11 @@ package TestModules::cgiupload;
 use strict;
 use warnings FATAL => 'all';
 
-use Apache::compat ();
+use Apache2::compat ();
 use CGI ();
 
+use Apache2::Const -compile => 'OK';
+
 sub handler {
     my $r = shift;
 
@@ -17,7 +19,7 @@ sub handler {
         print;
     }
 
-    Apache::OK;
+    Apache2::Const::OK;
 }
 
 1;
