@@ -1,4 +1,4 @@
-/* Copyright 2001-2004 The Apache Software Foundation
+/* Copyright 2001-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -13,14 +13,18 @@
  * limitations under the License.
  */
 
+/* apr_base64_encode_len and apr_base64_encode_binary give length that
+ * includes the terminating '\0' */
+#define mpxs_APR__Base64_encode_len(len) apr_base64_encode_len(len) - 1;
+    
 static MP_INLINE void mpxs_apr_base64_encode(pTHX_ SV *sv, SV *arg)
 {
     STRLEN len;
     int encoded_len;
     char *data = SvPV(arg, len);
-    mpxs_sv_grow(sv, apr_base64_encode_len(len));
+    mpxs_sv_grow(sv, apr_base64_encode_len(len) - 1);
     encoded_len = apr_base64_encode_binary(SvPVX(sv), data, len);
-    mpxs_sv_cur_set(sv, encoded_len);
+    mpxs_sv_cur_set(sv, encoded_len - 1);
 }
 
 static MP_INLINE void mpxs_apr_base64_decode(pTHX_ SV *sv, SV *arg)
@@ -33,7 +37,7 @@ static MP_INLINE void mpxs_apr_base64_decode(pTHX_ SV *sv, SV *arg)
     mpxs_sv_cur_set(sv, decoded_len);
 }
 
-static XS(MPXS_apr_base64_encode)
+MP_STATIC XS(MPXS_apr_base64_encode)
 {
     dXSARGS;
 
@@ -42,7 +46,7 @@ static XS(MPXS_apr_base64_encode)
     mpxs_set_targ(mpxs_apr_base64_encode, ST(0));
 }
 
-static XS(MPXS_apr_base64_decode)
+MP_STATIC XS(MPXS_apr_base64_decode)
 {
     dXSARGS;
 
