@@ -1,4 +1,4 @@
-/* Copyright 2001-2004 The Apache Software Foundation
+/* Copyright 2001-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -13,11 +13,35 @@
  * limitations under the License.
  */
 
-#define mpxs_APR__Table_FETCH   apr_table_get
 #define mpxs_APR__Table_STORE   apr_table_set
 #define mpxs_APR__Table_DELETE  apr_table_unset
 #define mpxs_APR__Table_CLEAR   apr_table_clear
 
+#define MPXS_DO_TABLE_N_MAGIC_RETURN(call)                              \
+    apr_pool_t *p = mp_xs_sv2_APR__Pool(p_sv);                          \
+    apr_table_t *t = call;                                              \
+    SV *t_sv = modperl_hash_tie(aTHX_ "APR::Table", Nullsv, t);         \
+    mpxs_add_pool_magic(t_sv, p_sv);                                    \
+    return t_sv;
+
+static MP_INLINE SV *mpxs_APR__Table_make(pTHX_ SV *p_sv, int nelts)
+{
+    MPXS_DO_TABLE_N_MAGIC_RETURN(apr_table_make(p, nelts));
+}
+
+
+static MP_INLINE SV *mpxs_APR__Table_copy(pTHX_ apr_table_t *base, SV *p_sv)
+{
+    MPXS_DO_TABLE_N_MAGIC_RETURN(apr_table_copy(p, base));
+}
+
+static MP_INLINE SV *mpxs_APR__Table_overlay(pTHX_ apr_table_t *base,
+                                             apr_table_t *overlay, SV *p_sv)
+{
+    MPXS_DO_TABLE_N_MAGIC_RETURN(apr_table_overlay(p, overlay, base));
+}
+
+
 typedef struct {
     SV *cv;
     apr_hash_t *filter;
@@ -45,7 +69,7 @@ static int mpxs_apr_table_do_cb(void *data,
             return 1;
         }
     }
-    
+
     ENTER;
     SAVETMPS;
 
@@ -61,7 +85,7 @@ static int mpxs_apr_table_do_cb(void *data,
 
     FREETMPS;
     LEAVE;
-    
+
     /* rv of 0 aborts the traversal */
     return rv;
 }
@@ -72,9 +96,9 @@ int mpxs_apr_table_do(pTHX_ I32 items, SV **MARK, SV **SP)
     apr_table_t *table;
     SV *sub;
     mpxs_table_do_cb_data_t tdata;
-    
+
     mpxs_usage_va_2(table, sub, "$table->do(sub, [@filter])");
-         
+
     tdata.cv = sub;
     tdata.filter = NULL;
 #ifdef USE_ITHREADS
@@ -84,7 +108,7 @@ int mpxs_apr_table_do(pTHX_ I32 items, SV **MARK, SV **SP)
     if (items > 2) {
         char *filter_entry;
         STRLEN len;
-        
+
         tdata.filter = apr_hash_make(apr_table_elts(table)->pool);
 
         while (MARK <= SP) {
@@ -93,16 +117,16 @@ int mpxs_apr_table_do(pTHX_ I32 items, SV **MARK, SV **SP)
             MARK++;
         }
     }
-  
+
     /* XXX: would be nice to be able to call apr_table_vdo directly, 
      * but I don't think it's possible to create/populate something 
      * that smells like a va_list with our list of filters specs
      */
-    
+
     apr_table_do(mpxs_apr_table_do_cb, (void *)&tdata, table, NULL);
-    
+
     /* Free tdata.filter or wait for the pool to go away? */
-    
+
     /* XXX: return return value of apr_table_do once we require newer httpd */
     return 1;
 }
@@ -122,27 +146,54 @@ SvCUR(SvRV(sv))
 
 static MP_INLINE const char *mpxs_APR__Table_NEXTKEY(pTHX_ SV *tsv, SV *key)
 {
-    apr_table_t *t = mp_xs_sv2_APR__Table(tsv); 
+    apr_table_t *t;
+    SV *rv = modperl_hash_tied_object_rv(aTHX_ "APR::Table", tsv);
+    if (!SvROK(rv)) {
+        Perl_croak(aTHX_ "Usage: $table->NEXTKEY($key): "
+                   "first argument not an APR::Table object");
+    }
+
+    t = INT2PTR(apr_table_t *, SvIVX(SvRV(rv))); 
 
     if (apr_is_empty_table(t)) {
         return NULL;
     }
 
-    if (mpxs_apr_table_iterix(tsv) < apr_table_elts(t)->nelts) {
-        return mpxs_apr_table_nextkey(t, tsv);
+    if (key == NULL) {
+        mpxs_apr_table_iterix(rv) = 0; /* reset iterator index */
     }
 
+    if (mpxs_apr_table_iterix(rv) < apr_table_elts(t)->nelts) {
+        return mpxs_apr_table_nextkey(t, rv);
+    }
+
+    mpxs_apr_table_iterix(rv) = 0;
+
     return NULL;
 }
 
-static MP_INLINE const char *mpxs_APR__Table_FIRSTKEY(pTHX_ SV *tsv)
+/* Try to shortcut apr_table_get by fetching the key using the current
+ * iterator (unless it's inactive or points at different key).
+ */
+static MP_INLINE const char *mpxs_APR__Table_FETCH(pTHX_ SV *tsv,
+                                                   const char *key)
 {
-    mpxs_apr_table_iterix(tsv) = 0; /* reset iterator index */
+    SV* rv = modperl_hash_tied_object_rv(aTHX_ "APR::Table", tsv);
+    const int i = mpxs_apr_table_iterix(rv);
+    apr_table_t *t = INT2PTR(apr_table_t *, SvIVX(SvRV(rv)));
+    const apr_array_header_t *arr = apr_table_elts(t);
+    apr_table_entry_t *elts = (apr_table_entry_t *)arr->elts;
 
-    return mpxs_APR__Table_NEXTKEY(aTHX_ tsv, Nullsv);
+    if (i > 0 && i <= arr->nelts && !strcasecmp(key, elts[i-1].key)) {
+        return elts[i-1].val;
+    }
+    else {
+        return apr_table_get(t, key);
+    }
 }
 
-static XS(MPXS_apr_table_get)
+
+MP_STATIC XS(MPXS_apr_table_get)
 {
     dXSARGS;
 
@@ -153,11 +204,11 @@ static XS(MPXS_apr_table_get)
     mpxs_PPCODE({
         APR__Table t = modperl_hash_tied_object(aTHX_ "APR::Table", ST(0));
         const char *key = (const char *)SvPV_nolen(ST(1));
-    
+
         if (!t) {
             XSRETURN_UNDEF;
         }
-        
+
         if (GIMME_V == G_SCALAR) {
             const char *val = apr_table_get(t, key);
 
@@ -167,9 +218,9 @@ static XS(MPXS_apr_table_get)
         }
         else {
             const apr_array_header_t *arr = apr_table_elts(t);
-            apr_table_entry_t *elts  = (apr_table_entry_t *)arr->elts;
+            apr_table_entry_t *elts = (apr_table_entry_t *)arr->elts;
             int i;
-            
+
             for (i = 0; i < arr->nelts; i++) {
                 if (!elts[i].key || strcasecmp(elts[i].key, key)) {
                     continue;
@@ -178,5 +229,5 @@ static XS(MPXS_apr_table_get)
             }
         }
     });
-    
+
 }
