@@ -1,4 +1,4 @@
-/* Copyright 2001-2004 The Apache Software Foundation
+/* Copyright 2001-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,22 +30,24 @@ char *mpxs_apr_uri_unparse(pTHX_
         uptr->scheme = "";
     }
 #endif
-    
+
     return apr_uri_unparse(((modperl_uri_t *)uptr)->pool,
                            uptr, flags);
 }
 
 static MP_INLINE
-apr_uri_t *mpxs_apr_uri_parse(pTHX_
-                              SV *classname,
-                              apr_pool_t *p,
-                              const char *uri_string)
+SV *mpxs_apr_uri_parse(pTHX_ SV *classname, SV *p_sv, const char *uri_string)
 {
+    SV *uri_sv;
+    apr_pool_t *p = mp_xs_sv2_APR__Pool(p_sv);
     modperl_uri_t *uri = modperl_uri_new(p);
 
     (void)apr_uri_parse(p, uri_string, &uri->uri);
 
-    return (apr_uri_t *)uri;
+    uri_sv = sv_setref_pv(NEWSV(0, 0), "APR::URI", (void*)uri);
+    mpxs_add_pool_magic(uri_sv, p_sv);
+    
+    return uri_sv;
 }
 
 static MP_INLINE
@@ -74,7 +76,7 @@ static MP_INLINE
 SV *mpxs_APR__URI_rpath(pTHX_ apr_uri_t *apr_uri)
 {
     modperl_uri_t *uri = (modperl_uri_t *)apr_uri;
-    
+
     if (uri->path_info) {
         int uri_len = strlen(uri->uri.path);
         int n = strlen(uri->path_info);
