@@ -1,4 +1,4 @@
-/* Copyright 2001-2004 The Apache Software Foundation
+/* Copyright 2001-2005 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -13,6 +13,14 @@
  * limitations under the License.
  */
 
+#ifdef USE_ITHREADS
+#define mpxs_ModPerl__Util_current_perl_id() \
+    Perl_newSVpvf(aTHX_ "0x%lx", (unsigned long)aTHX)
+#else
+#define mpxs_ModPerl__Util_current_perl_id() \
+    Perl_newSVpvf(aTHX_ "0x%lx", 0)
+#endif
+
 static MP_INLINE void mpxs_ModPerl__Util_untaint(pTHX_ I32 items,
                                                  SV **MARK, SV **SP)
 {
@@ -24,12 +32,10 @@ static MP_INLINE void mpxs_ModPerl__Util_untaint(pTHX_ I32 items,
     }
 }
 
-#define mpxs_ModPerl__Util_exit(status) modperl_perl_exit(aTHX_ status)
-
 #define mpxs_ModPerl__Util_current_callback \
     modperl_callback_current_callback_get
 
-#define mpxs_ModPerl__Util_unload_package(pkg) \
+#define mpxs_ModPerl__Util_unload_package_xs(pkg) \
     modperl_package_unload(aTHX_ pkg)
 
-
+/* ModPerl::Util::exit lives in mod_perl.so, see modperl_perl.c */
